@@ -400,9 +400,10 @@ do_write_module_dep_file_to_stream(Stream, Globals, Baggage, AugCompUnit,
     MaybeTopModule = Baggage ^ mb_maybe_top_module,
     NestedSubModules = get_nested_children_list_of_top_module(MaybeTopModule),
     get_fact_tables(ParseTreeModuleSrc, FactTableFilesSet),
-    set.to_sorted_list(FactTableFilesSet, FactTableFiles),
+    FactTableFilesStrs = list.map(term_io.quoted_string,
+        set.to_sorted_list(FactTableFilesSet)),
     globals.get_backend_foreign_languages(Globals, BackendLangsList),
-    BackendLangs = set.list_to_set(BackendLangsList), 
+    BackendLangs = set.list_to_set(BackendLangsList),
     get_foreign_code_langs(ParseTreeModuleSrc, CodeLangs),
     get_foreign_export_langs(ParseTreeModuleSrc, ExportLangs),
     set.intersect(BackendLangs, CodeLangs, BackendCodeLangs),
@@ -445,7 +446,7 @@ do_write_module_dep_file_to_stream(Stream, Globals, Baggage, AugCompUnit,
         s(bracketed_sym_names_to_comma_list_string(ImpDeps)),
         s(bracketed_sym_names_to_comma_list_string(Children)),
         s(bracketed_sym_names_to_comma_list_string(NestedSubModules)),
-        s(string.join_list(", ", FactTableFiles)),
+        s(string.join_list(", ", FactTableFilesStrs)),
         s(string.join_list(", ", CodeLangStrs)),
         s(string.join_list(", ", FIMSpecStrs)),
         s(ContainsForeignExportStr),
