@@ -1,16 +1,16 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 1993-2012 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: make_hlds_warn.m.
 %
 % Generate whatever warnings the module being transformed to HLDS deserves.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module hlds.make_hlds.make_hlds_warn.
 :- interface.
@@ -18,26 +18,30 @@
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.quantification.
+:- import_module parse_tree.
 :- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_foreign.
 
 :- import_module list.
-:- import_module pair.
+:- import_module maybe.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Warn about variables with overlapping scopes.
     %
-:- pred warn_overlap(list(quant_warning)::in, prog_varset::in,
-    simple_call_id::in, list(error_spec)::in, list(error_spec)::out) is det.
+:- pred add_quant_warnings(pf_sym_name_arity::in, prog_varset::in,
+    list(quant_warning)::in, list(error_spec)::in, list(error_spec)::out)
+    is det.
 
     % Warn about variables which occur only once but don't start with
     % an underscore, or about variables which do start with an underscore
     % but occur more than once, or about variables that do not occur in
     % C code strings when they should.
     %
-:- pred warn_singletons(module_info::in, simple_call_id::in, prog_varset::in,
-    hlds_goal::in, list(error_spec)::in, list(error_spec)::out) is det.
+:- pred warn_singletons(module_info::in, pf_sym_name_arity::in,
+    prog_varset::in, hlds_goal::in,
+    list(error_spec)::in, list(error_spec)::out) is det.
 
     % warn_singletons_in_pragma_foreign_proc checks to see if each variable
     % is mentioned at least once in the foreign code fragments that ought to
@@ -48,9 +52,9 @@
     % Mercury variable names into identifiers for that foreign language).
     %
 :- pred warn_singletons_in_pragma_foreign_proc(module_info::in,
-    pragma_foreign_code_impl::in, foreign_language::in,
-    list(maybe(pair(string, mer_mode)))::in, prog_context::in,
-    simple_call_id::in, pred_id::in, proc_id::in,
+    pragma_foreign_proc_impl::in, foreign_language::in,
+    list(maybe(foreign_arg_name_mode))::in, prog_context::in,
+    pf_sym_name_arity::in, pred_id::in, proc_id::in,
     list(error_spec)::in, list(error_spec)::out) is det.
 
     % This predicate performs the following checks on promise ex declarations
@@ -65,56 +69,67 @@
 :- pred check_promise_ex_decl(prog_vars::in, promise_type::in, goal::in,
     prog_context::in, list(error_spec)::in, list(error_spec)::out) is det.
 
-%----------------------------------------------------------------------------%
-%----------------------------------------------------------------------------%
+    % Warn about suspicious things in the bodies of foreign_code pragmas.
+    % Currently, this just checks for the presence of the MR_ALLOC_ID macro
+    % inside the bodies of a foreign_code pragmas.
+    %
+:- pred warn_suspicious_foreign_code(foreign_language::in,
+    foreign_literal_or_include::in, prog_context::in,
+    list(error_spec)::in, list(error_spec)::out) is det.
+
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
-:- import_module check_hlds.mode_util.
 :- import_module hlds.goal_util.
+:- import_module libs.
 :- import_module libs.globals.
 :- import_module libs.options.
-:- import_module parse_tree.mercury_to_mercury.
+:- import_module parse_tree.parse_tree_out_term.
 :- import_module parse_tree.prog_out.
 :- import_module parse_tree.set_of_var.
 
+:- import_module assoc_list.
 :- import_module bool.
 :- import_module char.
 :- import_module require.
 :- import_module string.
 :- import_module varset.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-warn_overlap(Warnings, VarSet, PredCallId, !Specs) :-
-    !:Specs =
-        list.map(warn_overlap_to_spec(VarSet, PredCallId), Warnings)
-        ++ !.Specs.
+add_quant_warnings(PredCallId, VarSet, Warnings, !Specs) :-
+    WarningSpecs =
+        list.map(quant_warning_to_spec(PredCallId, VarSet), Warnings),
+    !:Specs = WarningSpecs ++ !.Specs.
 
-:- func warn_overlap_to_spec(prog_varset, simple_call_id, quant_warning)
+:- func quant_warning_to_spec(pf_sym_name_arity, prog_varset, quant_warning)
     = error_spec.
 
-warn_overlap_to_spec(VarSet, PredCallId, Warn) = Spec :-
-    Warn = warn_overlap(Vars, Context),
-    Pieces1 =
-        [words("In clause for"), simple_call(PredCallId), suffix(":"), nl],
-    ( Vars = [Var] ->
-        Pieces2 = [words("warning: variable"),
-            quote(mercury_var_to_string(VarSet, no, Var)),
-            words("has overlapping scopes.")]
+quant_warning_to_spec(PredCallId, VarSet, Warning) = Spec :-
+    Warning = warn_overlap(Vars, Context),
+    Pieces1 = [words("In clause for"),
+        unqual_pf_sym_name_orig_arity(PredCallId), suffix(":"), nl],
+    (
+        Vars = [],
+        unexpected($pred, "Vars = []")
     ;
+        Vars = [Var],
+        Pieces2 = [words("warning: variable"),
+            quote(mercury_var_to_name_only(VarSet, Var)),
+            words("has overlapping scopes."), nl]
+    ;
+        Vars = [_, _ | _],
         Pieces2 = [words("warning: variables"),
-            quote(mercury_vars_to_string(VarSet, no, Vars)),
-            words("each have overlapping scopes.")]
+            quote(mercury_vars_to_name_only(VarSet, Vars)),
+            words("each have overlapping scopes."), nl]
     ),
-    Msg = simple_msg(Context,
-        [option_is_set(warn_overlapping_scopes, yes,
-            [always(Pieces1 ++ Pieces2)])]),
-    Severity = severity_conditional(warn_overlapping_scopes, yes,
-        severity_warning, no),
-    Spec = error_spec(Severity, phase_parse_tree_to_hlds, [Msg]).
+    Spec = conditional_spec($pred, warn_overlapping_scopes, yes,
+        severity_warning, phase_parse_tree_to_hlds,
+        [simplest_msg(Context, Pieces1 ++ Pieces2)]).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 warn_singletons(ModuleInfo, PredCallId, VarSet, Body, !Specs) :-
     % We handle warnings about variables in the clause head specially.
@@ -173,7 +188,7 @@ warn_singletons(ModuleInfo, PredCallId, VarSet, Body, !Specs) :-
 
                 % The id and the varset of the procedure whose body
                 % we are checking.
-                wi_pred_call_id         :: simple_call_id,
+                wi_pred_call_id         :: pf_sym_name_arity,
                 wi_varset               :: prog_varset,
 
                 % The warnings we have generated while checking.
@@ -233,10 +248,25 @@ warn_singletons_in_goal(Goal, QuantVars, !Info) :-
                 SubQuantVars = QuantVars
             ),
             warn_singletons_in_goal(SubGoal, SubQuantVars, !Info)
+        ;
+            Reason = disable_warnings(HeadWarning, TailWarnings),
+            ( if
+                ( HeadWarning = goal_warning_singleton_vars
+                ; list.member(goal_warning_singleton_vars, TailWarnings)
+                )
+            then
+                % Since we don't want to generate any singleton variable
+                % warnings inside this scope, there is no point in examining
+                % the goals inside this scope.
+                true
+            else
+                warn_singletons_in_goal(SubGoal, QuantVars, !Info)
+            )
         ;
             ( Reason = promise_purity(_)
             ; Reason = require_detism(_)
             ; Reason = require_complete_switch(_)
+            ; Reason = require_switch_arms_detism(_, _)
             ; Reason = commit(_)
             ; Reason = barrier(_)
             ; Reason = trace_goal(_, _, _, _, _)
@@ -255,7 +285,7 @@ warn_singletons_in_goal(Goal, QuantVars, !Info) :-
             Reason = loop_control(_, _, _),
             % These scopes are introduced only by compiler passes
             % that execute after us.
-            sorry($module, $pred, "loop_control")
+            sorry($pred, "loop_control")
         )
     ;
         GoalExpr = if_then_else(Vars, Cond, Then, Else),
@@ -355,11 +385,12 @@ warn_singletons_in_unify(X, RHS, GoalInfo, QuantVars, !Info) :-
             !Info)
     ;
         RHS = rhs_lambda_goal(_Purity, _Groundness, _PredOrFunc,
-            _Eval, _NonLocals, LambdaVars, _Modes, _Det, LambdaGoal),
+            _Eval, _NonLocals, ArgVarsModes, _Det, LambdaGoal),
+        assoc_list.keys(ArgVarsModes, ArgVars),
         % Warn if any lambda-quantified variables occur only in the quantifier.
         LambdaGoal = hlds_goal(_, LambdaGoalInfo),
         LambdaNonLocals = goal_info_get_nonlocals(LambdaGoalInfo),
-        warn_singletons_goal_vars(LambdaVars, GoalInfo, LambdaNonLocals,
+        warn_singletons_goal_vars(ArgVars, GoalInfo, LambdaNonLocals,
             QuantVars, !Info),
 
         % Warn if X (the variable we're unifying the lambda expression with)
@@ -371,7 +402,7 @@ warn_singletons_in_unify(X, RHS, GoalInfo, QuantVars, !Info) :-
         warn_singletons_in_goal(LambdaGoal, QuantVars, !Info)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % warn_singletons_goal_vars(Vars, GoalInfo, NonLocals, QuantVars, ...):
     %
@@ -398,20 +429,20 @@ warn_singletons_goal_vars(GoalVars, GoalInfo, NonLocals, QuantVars, !Info) :-
         SingleVars),
 
     % If there were any such variables, issue a warning.
-    (
+    ( if
         ( SingleVars = []
         ; goal_info_has_feature(GoalInfo, feature_dont_warn_singleton)
         )
-    ->
+    then
         true
-    ;
-        ( goal_info_has_feature(GoalInfo, feature_from_head) ->
+    else
+        ( if goal_info_has_feature(GoalInfo, feature_from_head) then
             SingleHeadVars0 = !.Info ^ wi_singleton_headvars,
             set_of_var.insert_list(SingleVars,
                 SingleHeadVars0, SingleHeadVars),
             !Info ^ wi_singleton_headvars := SingleHeadVars,
             !Info ^ wi_head_context := goal_info_get_context(GoalInfo)
-        ;
+        else
             generate_variable_warning(sm_single, Context, CallId, VarSet,
                 SingleVars, SingleSpec),
             add_warn_spec(SingleSpec, !Info)
@@ -427,12 +458,12 @@ warn_singletons_goal_vars(GoalVars, GoalInfo, NonLocals, QuantVars, !Info) :-
         MultiVars = []
     ;
         MultiVars = [_ | _],
-        ( goal_info_has_feature(GoalInfo, feature_from_head) ->
+        ( if goal_info_has_feature(GoalInfo, feature_from_head) then
             MultiHeadVars0 = !.Info ^ wi_multi_headvars,
             set_of_var.insert_list(MultiVars, MultiHeadVars0, MultiHeadVars),
             !Info ^ wi_multi_headvars := MultiHeadVars,
             !Info ^ wi_head_context := goal_info_get_context(GoalInfo)
-        ;
+        else
             generate_variable_warning(sm_multi, Context, CallId, VarSet,
                 MultiVars, MultiSpec),
             add_warn_spec(MultiSpec, !Info)
@@ -444,8 +475,8 @@ warn_singletons_goal_vars(GoalVars, GoalInfo, NonLocals, QuantVars, !Info) :-
     ;       sm_multi.
 
 :- pred generate_variable_warning(single_or_multi::in, prog_context::in,
-    simple_call_id::in, prog_varset::in, list(prog_var)::in, error_spec::out)
-    is det.
+    pf_sym_name_arity::in, prog_varset::in, list(prog_var)::in,
+    error_spec::out) is det.
 
 generate_variable_warning(SingleMulti, Context, CallId, VarSet, Vars, Spec) :-
     (
@@ -455,23 +486,29 @@ generate_variable_warning(SingleMulti, Context, CallId, VarSet, Vars, Spec) :-
         SingleMulti = sm_multi,
         Count = "more than once"
     ),
-    Preamble = [words("In clause for"), simple_call(CallId), suffix(":"), nl],
-    VarStrs0 = list.map(mercury_var_to_string(VarSet, no), Vars),
+    Preamble = [words("In clause for"),
+        unqual_pf_sym_name_orig_arity(CallId), suffix(":"), nl],
+    VarStrs0 = list.map(mercury_var_to_name_only(VarSet), Vars),
     list.sort_and_remove_dups(VarStrs0, VarStrs),
-    VarsPiece = quote(string.join_list(", ", VarStrs)),
-    ( VarStrs = [_] ->
+    VarsStr = "`" ++ string.join_list(", ", VarStrs) ++ "'",
+    % We want VarsPiece to be breakable into two or more lines
+    % in case VarsStr does not fit on one line.
+    VarsPiece = words(VarsStr),
+    (
+        VarStrs = [],
+        unexpected($pred, "VarStrs = []")
+    ;
+        VarStrs = [_],
         Pieces = [words("warning: variable"), VarsPiece,
             words("occurs"), words(Count), words("in this scope."), nl]
     ;
+        VarStrs = [_, _ | _],
         Pieces = [words("warning: variables"), VarsPiece,
             words("occur"), words(Count), words("in this scope."), nl]
     ),
-    Msg = simple_msg(Context,
-        [option_is_set(warn_singleton_vars, yes,
-            [always(Preamble ++ Pieces)])]),
-    Severity = severity_conditional(warn_singleton_vars, yes,
-        severity_warning, no),
-    Spec = error_spec(Severity, phase_parse_tree_to_hlds, [Msg]).
+    Spec = conditional_spec($pred, warn_singleton_vars, yes,
+        severity_warning, phase_parse_tree_to_hlds,
+        [simplest_msg(Context, Preamble ++ Pieces)]).
 
 :- pred add_warn_spec(error_spec::in, warn_info::in, warn_info::out) is det.
 
@@ -480,67 +517,44 @@ add_warn_spec(Spec, !Info) :-
     Specs = [Spec | Specs0],
     !Info ^ wi_specs := Specs.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 warn_singletons_in_pragma_foreign_proc(ModuleInfo, PragmaImpl, Lang,
-        Args, Context, SimpleCallId, PredId, ProcId, !Specs) :-
+        Args, Context, PFSymNameArity, PredId, ProcId, !Specs) :-
     LangStr = foreign_language_string(Lang),
-    PragmaImpl = fc_impl_ordinary(C_Code, _),
-    c_code_to_name_list(C_Code, C_CodeList),
+    PragmaImpl = fp_impl_ordinary(Code, _),
+    c_code_to_name_list(Code, C_CodeList),
     list.filter_map(var_is_unmentioned(C_CodeList), Args, UnmentionedVars),
     (
         UnmentionedVars = []
     ;
         UnmentionedVars = [_ | _],
         Pieces = [words("In the"), words(LangStr), words("code for"),
-            simple_call(SimpleCallId), suffix(":"), nl] ++
+            unqual_pf_sym_name_orig_arity(PFSymNameArity), suffix(":"), nl] ++
             variable_warning_start(UnmentionedVars) ++
             [words("not occur in the"), words(LangStr), words("code."), nl],
-        Msg = simple_msg(Context,
-            [option_is_set(warn_singleton_vars, yes, [always(Pieces)])]),
-        Severity = severity_conditional(warn_singleton_vars, yes,
-            severity_warning, no),
-        Spec = error_spec(Severity, phase_parse_tree_to_hlds, [Msg]),
+        Spec = conditional_spec($pred, warn_singleton_vars, yes,
+            severity_warning, phase_parse_tree_to_hlds,
+            [simplest_msg(Context, Pieces)]),
         !:Specs = [Spec | !.Specs]
     ),
-    pragma_foreign_proc_body_checks(ModuleInfo, Lang, Context, SimpleCallId,
+    pragma_foreign_proc_body_checks(ModuleInfo, Lang, Context, PFSymNameArity,
         PredId, ProcId, C_CodeList, !Specs).
 
-:- pred var_is_unmentioned(list(string)::in, maybe(pair(string, mer_mode))::in,
+:- pred var_is_unmentioned(list(string)::in, maybe(foreign_arg_name_mode)::in,
     string::out) is semidet.
 
 var_is_unmentioned(NameList1, MaybeArg, Name) :-
-    MaybeArg = yes(Name - _Mode),
-    \+ string.prefix(Name, "_"),
-    \+ list.member(Name, NameList1).
-
-:- pred input_var_is_unmentioned(module_info::in,
-    list(string)::in, maybe(pair(string, mer_mode))::in,
-    string::out) is semidet.
-
-input_var_is_unmentioned(ModuleInfo, NameList1, MaybeArg, Name) :-
-    MaybeArg = yes(Name - Mode),
-    mode_is_input(ModuleInfo, Mode),
-    \+ string.prefix(Name, "_"),
-    \+ list.member(Name, NameList1).
-
-:- pred output_var_is_unmentioned(module_info::in,
-    list(string)::in, list(string)::in, maybe(pair(string, mer_mode))::in,
-    string::out) is semidet.
-
-output_var_is_unmentioned(ModuleInfo, NameList1, NameList2, MaybeArg, Name) :-
-    MaybeArg = yes(Name - Mode),
-    mode_is_output(ModuleInfo, Mode),
-    \+ string.prefix(Name, "_"),
-    \+ list.member(Name, NameList1),
-    \+ list.member(Name, NameList2).
+    MaybeArg = yes(foreign_arg_name_mode(Name, _Mode)),
+    not string.prefix(Name, "_"),
+    not list.member(Name, NameList1).
 
 :- func variable_warning_start(list(string)) = list(format_component).
 
 variable_warning_start(UnmentionedVars) = Pieces :-
-    ( UnmentionedVars = [Var] ->
+    ( if UnmentionedVars = [Var] then
         Pieces = [words("warning: variable"), quote(Var), words("does")]
-    ;
+    else
         Pieces = [words("warning: variables"),
             words(add_quotes(string.join_list(", ", UnmentionedVars))),
             words("do")]
@@ -575,10 +589,10 @@ c_code_to_name_list_2(C_Code, List) :-
 
 get_first_c_name([], [], []).
 get_first_c_name([C | CodeChars], NameCharList, TheRest) :-
-    ( char.is_alnum_or_underscore(C) ->
+    ( if char.is_alnum_or_underscore(C) then
         get_first_c_name_in_word(CodeChars, NameCharList0, TheRest),
         NameCharList = [C | NameCharList0]
-    ;
+    else
         % Strip off any characters in the C code which don't form part
         % of an identifier.
         get_first_c_name(CodeChars, NameCharList, TheRest)
@@ -589,11 +603,11 @@ get_first_c_name([C | CodeChars], NameCharList, TheRest) :-
 
 get_first_c_name_in_word([], [], []).
 get_first_c_name_in_word([C | CodeChars], NameCharList, TheRest) :-
-    ( char.is_alnum_or_underscore(C) ->
+    ( if char.is_alnum_or_underscore(C) then
         % There are more characters in the word.
         get_first_c_name_in_word(CodeChars, NameCharList0, TheRest),
         NameCharList = [C|NameCharList0]
-    ;
+    else
         % The word is finished.
         NameCharList = [],
         TheRest = CodeChars
@@ -603,11 +617,11 @@ get_first_c_name_in_word([C | CodeChars], NameCharList, TheRest) :-
     set_of_progvar::in, prog_varset::in, prog_var::in) is semidet.
 
 is_singleton_var(NonLocals, QuantVars, VarSet, Var) :-
-    \+ set_of_var.member(NonLocals, Var),
+    not set_of_var.member(NonLocals, Var),
     varset.search_name(VarSet, Var, Name),
-    \+ string.prefix(Name, "_"),
-    \+ string.prefix(Name, "DCG_"),
-    \+ (
+    not string.prefix(Name, "_"),
+    not string.prefix(Name, "DCG_"),
+    not (
         set_of_var.member(QuantVars, QuantVar),
         varset.search_name(VarSet, QuantVar, Name)
     ).
@@ -621,100 +635,81 @@ is_multi_var(NonLocals, VarSet, Var) :-
     string.prefix(Name, "_").
 
 :- pred pragma_foreign_proc_body_checks(module_info::in, foreign_language::in,
-    prog_context::in, simple_call_id::in, pred_id::in, proc_id::in,
+    prog_context::in, pf_sym_name_arity::in, pred_id::in, proc_id::in,
     list(string)::in, list(error_spec)::in, list(error_spec)::out) is det.
 
-pragma_foreign_proc_body_checks(ModuleInfo, Lang, Context, SimpleCallId,
+pragma_foreign_proc_body_checks(ModuleInfo, Lang, Context, PFSymNameArity,
         PredId, ProcId, BodyPieces, !Specs) :-
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
-    pred_info_get_import_status(PredInfo, ImportStatus),
-    IsImported = status_is_imported(ImportStatus),
+    pred_info_get_status(PredInfo, PredStatus),
+    IsImported = pred_status_is_imported(PredStatus),
     (
         IsImported = yes
     ;
         IsImported = no,
         check_fp_body_for_success_indicator(ModuleInfo, Lang, Context,
-            SimpleCallId, PredId, ProcId, BodyPieces, !Specs),
-        check_fp_body_for_return(Lang, Context, SimpleCallId, BodyPieces,
+            PFSymNameArity, PredId, ProcId, BodyPieces, !Specs),
+        check_fp_body_for_return(Lang, Context, PFSymNameArity, BodyPieces,
             !Specs)
     ).
 
 :- pred check_fp_body_for_success_indicator(module_info::in,
-    foreign_language::in, prog_context::in, simple_call_id::in,
+    foreign_language::in, prog_context::in, pf_sym_name_arity::in,
     pred_id::in, proc_id::in, list(string)::in,
     list(error_spec)::in, list(error_spec)::out) is det.
 
-check_fp_body_for_success_indicator(ModuleInfo, Lang, Context, SimpleCallId,
+check_fp_body_for_success_indicator(ModuleInfo, Lang, Context, PFSymNameArity,
         PredId, ProcId, BodyPieces, !Specs) :-
     module_info_proc_info(ModuleInfo, PredId, ProcId, ProcInfo),
     proc_info_get_declared_determinism(ProcInfo, MaybeDeclDetism),
     (
         MaybeDeclDetism = yes(Detism),
+        SuccIndStr = "SUCCESS_INDICATOR",
         (
-            ( Lang = lang_c
-            ; Lang = lang_csharp
-            ; Lang = lang_erlang
-            ; Lang = lang_java
+            ( Detism = detism_det
+            ; Detism = detism_cc_multi
+            ; Detism = detism_erroneous
             ),
-            SuccIndStr = "SUCCESS_INDICATOR",
-            (
-                ( Detism = detism_det
-                ; Detism = detism_cc_multi
-                ; Detism = detism_erroneous
-                ),
-                ( list.member(SuccIndStr, BodyPieces) ->
-                    LangStr = foreign_language_string(Lang),
-                    Pieces = [
-                        words("warning: the "), fixed(LangStr),
-                        words("code for"), simple_call(SimpleCallId),
-                        words("may set"), quote(SuccIndStr), suffix(","),
-                        words("but it cannot fail.")
-                    ],
-                    Msg = simple_msg(Context,
-                        [option_is_set(warn_suspicious_foreign_procs, yes,
-                            [always(Pieces)])]),
-                    Severity = severity_conditional(
-                        warn_suspicious_foreign_procs, yes,
-                        severity_warning, no),
-                    Spec = error_spec(Severity, phase_parse_tree_to_hlds,
-                        [Msg]),
-                    !:Specs = [Spec | !.Specs]
-                ;
-                    true
-                )
-            ;
-                ( Detism = detism_semi
-                ; Detism = detism_cc_non
-                ),
-                ( list.member(SuccIndStr, BodyPieces) ->
-                    true
-                ;
-                    LangStr = foreign_language_string(Lang),
-                    Pieces = [
-                        words("warning: the "), fixed(LangStr),
-                        words("code for"), simple_call(SimpleCallId),
-                        words("does not appear to set"),
-                        quote(SuccIndStr), suffix(","),
-                        words("but it can fail.")
-                    ],
-                    Msg = simple_msg(Context,
-                        [option_is_set(warn_suspicious_foreign_procs, yes,
-                            [always(Pieces)])]),
-                    Severity = severity_conditional(
-                        warn_suspicious_foreign_procs, yes,
-                        severity_warning, no),
-                    Spec = error_spec(Severity, phase_parse_tree_to_hlds,
-                        [Msg]),
-                    !:Specs = [Spec | !.Specs]
-                )
-            ;
-                ( Detism = detism_multi
-                ; Detism = detism_non
-                ; Detism = detism_failure
-                )
+            ( if list.member(SuccIndStr, BodyPieces) then
+                LangStr = foreign_language_string(Lang),
+                Pieces = [words("Warning: the"), fixed(LangStr),
+                    words("code for"),
+                    unqual_pf_sym_name_orig_arity(PFSymNameArity),
+                    words("may set"), quote(SuccIndStr), suffix(","),
+                    words("but it cannot fail.")],
+                Spec = conditional_spec($pred,
+                    warn_suspicious_foreign_procs, yes,
+                    severity_warning, phase_parse_tree_to_hlds,
+                    [simplest_msg(Context, Pieces)]),
+                !:Specs = [Spec | !.Specs]
+            else
+                true
             )
         ;
-            Lang = lang_il
+            ( Detism = detism_semi
+            ; Detism = detism_cc_non
+            ),
+            ( if list.member(SuccIndStr, BodyPieces) then
+                true
+            else
+                LangStr = foreign_language_string(Lang),
+                Pieces = [words("Warning: the"), fixed(LangStr),
+                    words("code for"),
+                    unqual_pf_sym_name_orig_arity(PFSymNameArity),
+                    words("does not appear to set"),
+                    quote(SuccIndStr), suffix(","),
+                    words("but it can fail.")],
+                Spec = conditional_spec($pred,
+                    warn_suspicious_foreign_procs, yes,
+                    severity_warning, phase_parse_tree_to_hlds,
+                    [simplest_msg(Context, Pieces)]),
+                !:Specs = [Spec | !.Specs]
+            )
+        ;
+            ( Detism = detism_multi
+            ; Detism = detism_non
+            ; Detism = detism_failure
+            )
         )
     ;
         MaybeDeclDetism = no
@@ -724,63 +719,26 @@ check_fp_body_for_success_indicator(ModuleInfo, Lang, Context, SimpleCallId,
     % (or whatever the foreign language equivalent is).
     %
 :- pred check_fp_body_for_return(foreign_language::in, prog_context::in,
-    simple_call_id::in, list(string)::in,
+    pf_sym_name_arity::in, list(string)::in,
     list(error_spec)::in, list(error_spec)::out) is det.
 
-check_fp_body_for_return(Lang, Context, SimpleCallId, BodyPieces, !Specs) :-
-    (
-        ( Lang = lang_c
-        ; Lang = lang_csharp
-        ; Lang = lang_java
-        ),
-        ( list.member("return", BodyPieces) ->
-            LangStr = foreign_language_string(Lang),
-            Pieces = [
-                words("warning: the "), fixed(LangStr),
-                words("code for"), simple_call(SimpleCallId),
-                words("may contain a"), quote("return"),
-                words("statement."), nl
-            ],
-            Msg = simple_msg(Context,
-                [option_is_set(warn_suspicious_foreign_procs, yes,
-                    [always(Pieces)])]
-            ),
-            Severity = severity_conditional(
-                warn_suspicious_foreign_procs, yes, severity_warning, no),
-            Spec = error_spec(Severity, phase_parse_tree_to_hlds, [Msg]),
-            !:Specs = [Spec | !.Specs]
-        ;
-            true
-        )
-    ;
-        Lang = lang_il,
-        (
-            ( list.member("ret", BodyPieces)
-            ; list.member("jmp", BodyPieces)
-            )
-        ->
-            Pieces = [
-                words("warning: the IL code for"), simple_call(SimpleCallId),
-                words("may contain a"), quote("ret"),
-                words("or"), quote("jmp"),
-                words("instruction."), nl
-            ],
-            Msg = simple_msg(Context,
-                [option_is_set(warn_suspicious_foreign_procs, yes,
-                    [always(Pieces)])]
-            ),
-            Severity = severity_conditional(
-                warn_suspicious_foreign_procs, yes, severity_warning, no),
-            Spec = error_spec(Severity, phase_parse_tree_to_hlds, [Msg]),
-            !:Specs = [Spec | !.Specs]
-        ;
-            true
-        )
-    ;
-        Lang = lang_erlang
+check_fp_body_for_return(Lang, Context, PFSymNameArity, BodyPieces, !Specs) :-
+    ( if list.member("return", BodyPieces) then
+        LangStr = foreign_language_string(Lang),
+        Pieces = [words("Warning: the"), fixed(LangStr),
+            words("code for"),
+            unqual_pf_sym_name_orig_arity(PFSymNameArity),
+            words("may contain a"), quote("return"),
+            words("statement."), nl],
+        Spec = conditional_spec($pred, warn_suspicious_foreign_procs, yes,
+            severity_warning, phase_parse_tree_to_hlds,
+            [simplest_msg(Context, Pieces)]),
+        !:Specs = [Spec | !.Specs]
+    else
+        true
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Promise_ex error checking.
 %
@@ -802,54 +760,61 @@ check_promise_ex_decl(UnivVars, PromiseType, Goal, Context, !Specs) :-
 :- pred check_promise_ex_goal(promise_type::in, goal::in,
     list(error_spec)::in, list(error_spec)::out) is det.
 
-check_promise_ex_goal(PromiseType, GoalExpr - Context, !Specs) :-
-    ( GoalExpr = some_expr(_, Goal) ->
-        check_promise_ex_goal(PromiseType, Goal, !Specs)
-    ; GoalExpr = disj_expr(_, _) ->
-        flatten_to_disj_list(GoalExpr - Context, DisjList),
+check_promise_ex_goal(PromiseType, Goal, !Specs) :-
+    ( if
+        Goal = quant_expr(quant_some, quant_ordinary_vars, _, _, SubGoal)
+    then
+        check_promise_ex_goal(PromiseType, SubGoal, !Specs)
+    else if
+        Goal = disj_expr(_, _, _)
+    then
+        flatten_to_disj_list(Goal, DisjList),
         list.map(flatten_to_conj_list, DisjList, DisjConjList),
         check_promise_ex_disjunction(PromiseType, DisjConjList, !Specs)
-    ; GoalExpr = all_expr(_UnivVars, Goal) ->
+    else if
+        Goal = quant_expr(quant_all, quant_ordinary_vars, Context, _UnivVars,
+            SubGoal)
+    then
         promise_ex_error(PromiseType, Context,
             "universal quantification should come before " ++
             "the declaration name", !Specs),
-        check_promise_ex_goal(PromiseType, Goal, !Specs)
-    ;
-        promise_ex_error(PromiseType, Context,
+        check_promise_ex_goal(PromiseType, SubGoal, !Specs)
+    else
+        promise_ex_error(PromiseType, get_goal_context(Goal),
             "goal in declaration is not a disjunction", !Specs)
     ).
 
     % Turns the goal of a promise_ex declaration into a list of goals,
     % where each goal is an arm of the disjunction.
     %
-:- pred flatten_to_disj_list(goal::in, goals::out) is det.
+:- pred flatten_to_disj_list(goal::in, list(goal)::out) is det.
 
-flatten_to_disj_list(GoalExpr - Context, GoalList) :-
-    ( GoalExpr = disj_expr(GoalA, GoalB) ->
+flatten_to_disj_list(Goal, GoalList) :-
+    ( if Goal = disj_expr(_, GoalA, GoalB) then
         flatten_to_disj_list(GoalA, GoalListA),
         flatten_to_disj_list(GoalB, GoalListB),
         GoalList = GoalListA ++ GoalListB
-    ;
-        GoalList = [GoalExpr - Context]
+    else
+        GoalList = [Goal]
     ).
 
     % Takes a goal representing an arm of a disjunction and turns it into
     % a list of conjunct goals.
     %
-:- pred flatten_to_conj_list(goal::in, goals::out) is det.
+:- pred flatten_to_conj_list(goal::in, list(goal)::out) is det.
 
-flatten_to_conj_list(GoalExpr - Context, GoalList) :-
-    ( GoalExpr = conj_expr(GoalA, GoalB) ->
+flatten_to_conj_list(Goal, GoalList) :-
+    ( if Goal = conj_expr(_, GoalA, GoalB) then
         flatten_to_conj_list(GoalA, GoalListA),
         flatten_to_conj_list(GoalB, GoalListB),
         GoalList = GoalListA ++ GoalListB
-    ;
-        GoalList = [GoalExpr - Context]
+    else
+        GoalList = [Goal]
     ).
 
     % Taking a list of arms of the disjunction, check each arm individually.
     %
-:- pred check_promise_ex_disjunction(promise_type::in, list(goals)::in,
+:- pred check_promise_ex_disjunction(promise_type::in, list(list(goal))::in,
     list(error_spec)::in, list(error_spec)::out) is det.
 
 check_promise_ex_disjunction(PromiseType, DisjConjList, !Specs) :-
@@ -864,20 +829,28 @@ check_promise_ex_disjunction(PromiseType, DisjConjList, !Specs) :-
     % Only one goal in an arm is allowed to be a call, the rest must be
     % unifications.
     %
-:- pred check_promise_ex_disj_arm(promise_type::in, goals::in, bool::in,
+:- pred check_promise_ex_disj_arm(promise_type::in, list(goal)::in, bool::in,
     list(error_spec)::in, list(error_spec)::out) is det.
 
 check_promise_ex_disj_arm(PromiseType, Goals, CallUsed, !Specs) :-
     (
         Goals = []
     ;
-        Goals = [GoalExpr - Context | Rest],
-        ( GoalExpr = unify_expr(_, _, _) ->
-            check_promise_ex_disj_arm(PromiseType, Rest, CallUsed, !Specs)
-        ; GoalExpr = some_expr(_, Goal) ->
-            check_promise_ex_disj_arm(PromiseType, [Goal | Rest], CallUsed,
-                !Specs)
-        ; GoalExpr = call_expr(_, _, _) ->
+        Goals = [HeadGoal | TailGoals],
+        ( if
+            HeadGoal = unify_expr(_, _, _, _)
+        then
+            check_promise_ex_disj_arm(PromiseType, TailGoals,
+                CallUsed, !Specs)
+        else if
+            HeadGoal = quant_expr(quant_some, quant_ordinary_vars, _, _,
+                HeadSubGoal)
+            then
+            check_promise_ex_disj_arm(PromiseType, [HeadSubGoal | TailGoals],
+                CallUsed, !Specs)
+        else if
+            HeadGoal = call_expr(Context, _, _, _)
+        then
             (
                 CallUsed = no
             ;
@@ -885,11 +858,11 @@ check_promise_ex_disj_arm(PromiseType, Goals, CallUsed, !Specs) :-
                 promise_ex_error(PromiseType, Context,
                     "disjunct contains more than one call", !Specs)
             ),
-            check_promise_ex_disj_arm(PromiseType, Rest, yes, !Specs)
-        ;
-            promise_ex_error(PromiseType, Context,
+            check_promise_ex_disj_arm(PromiseType, TailGoals, yes, !Specs)
+        else
+            promise_ex_error(PromiseType, get_goal_context(HeadGoal),
                 "disjunct is not a call or unification", !Specs),
-            check_promise_ex_disj_arm(PromiseType, Rest, CallUsed, !Specs)
+            check_promise_ex_disj_arm(PromiseType, TailGoals, CallUsed, !Specs)
         )
     ).
 
@@ -902,10 +875,43 @@ promise_ex_error(PromiseType, Context, Message, !Specs) :-
     Pieces = [words("In"), quote(prog_out.promise_to_string(PromiseType)),
         words("declaration:"), nl,
         words("error:"), words(Message), nl],
-    Msg = simple_msg(Context, [always(Pieces)]),
-    Spec = error_spec(severity_error, phase_parse_tree_to_hlds, [Msg]),
+    Spec = simplest_spec($pred, severity_error, phase_parse_tree_to_hlds,
+        Context, Pieces),
     !:Specs = [Spec | !.Specs].
 
-%-----------------------------------------------------------------------------%
-:- end_module make_hlds_warn.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+
+warn_suspicious_foreign_code(Lang, BodyCode, Context, !Specs) :-
+    (
+        BodyCode = floi_include_file(_)
+    ;
+        BodyCode = floi_literal(Code),
+        (
+            Lang = lang_c,
+            c_code_to_name_list(Code, C_CodeList),
+            ( if list.member("MR_ALLOC_ID", C_CodeList) then
+                Pieces = [
+                    words("Warning: the body of this"),
+                    pragma_decl("foreign_code"),
+                    words("declaration may refer to the"),
+                    quote("MR_ALLOC_ID"), words("macro."),
+                    words("That macro is only defined within the body of"),
+                    pragma_decl("foreign_proc"), words("declarations.")
+                ],
+                Spec = conditional_spec($pred, warn_suspicious_foreign_code,
+                    yes, severity_warning, phase_parse_tree_to_hlds,
+                    [simplest_msg(Context, Pieces)]),
+                !:Specs = [Spec | !.Specs]
+            else
+                true
+            )
+        ;
+            ( Lang = lang_csharp
+            ; Lang = lang_java
+            )
+        )
+    ).
+
+%---------------------------------------------------------------------------%
+:- end_module hlds.make_hlds.make_hlds_warn.
+%---------------------------------------------------------------------------%
