@@ -1,17 +1,17 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: mercury_compile_mlds_back_end.m.
 %
 % This module implements the MLDS backend for the top level of the Mercury
 % compiler. It invokes the different passes of this backend as appropriate.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module top_level.mercury_compile_mlds_back_end.
 :- interface.
@@ -21,90 +21,72 @@
 :- import_module hlds.passes_aux.
 :- import_module libs.
 :- import_module libs.globals.
+:- import_module libs.maybe_succeeded.
+:- import_module ml_backend.
 :- import_module ml_backend.mlds.
 :- import_module parse_tree.
-:- import_module parse_tree.prog_data.
+:- import_module parse_tree.error_util.
 
-:- import_module bool.
 :- import_module io.
-
-    % Return `yes' iff this module defines the main/2 entry point.
-    %
-:- func mlds_has_main(mlds) = has_main.
+:- import_module list.
 
 :- pred mlds_backend(module_info::in, module_info::out, mlds::out,
-    dump_info::in, dump_info::out, io::di, io::uo) is det.
+    list(error_spec)::out, dump_info::in, dump_info::out, io::di, io::uo)
+    is det.
 
-:- pred maybe_mark_static_terms(bool::in, bool::in,
-    module_info::in, module_info::out, io::di, io::uo) is det.
+:- pred mlds_to_high_level_c(globals::in, mlds::in, maybe_succeeded::out,
+    io::di, io::uo) is det.
 
-:- pred mlds_to_high_level_c(globals::in, mlds::in, io::di, io::uo) is det.
+:- pred mlds_to_java(module_info::in, mlds::in, maybe_succeeded::out,
+    io::di, io::uo) is det.
 
-:- pred mlds_to_java(module_info::in, mlds::in, io::di, io::uo) is det.
+:- pred mlds_to_csharp(module_info::in, mlds::in, maybe_succeeded::out,
+    io::di, io::uo) is det.
 
-:- pred mlds_to_csharp(module_info::in, mlds::in, io::di, io::uo) is det.
-
-:- pred mlds_to_il_assembler(globals::in, mlds::in, io::di, io::uo) is det.
-
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module backend_libs.
 :- import_module backend_libs.base_typeclass_info.
 :- import_module backend_libs.type_class_info.
 :- import_module backend_libs.type_ctor_info.
+:- import_module hlds.hlds_dependency_graph.        % HLDS -> HLDS
 :- import_module hlds.mark_static_terms.            % HLDS -> HLDS
+:- import_module hlds.mark_tail_calls.              % HLDS -> HLDS
 :- import_module libs.file_util.
-:- import_module libs.globals.
+:- import_module libs.optimization_options.
 :- import_module libs.options.
 :- import_module ml_backend.add_trail_ops.          % HLDS -> HLDS
 :- import_module ml_backend.add_heap_ops.           % HLDS -> HLDS
-:- import_module ml_backend.ml_proc_gen.            % HLDS -> MLDS
+:- import_module ml_backend.ml_top_gen.             % HLDS -> MLDS
 :- import_module ml_backend.rtti_to_mlds.           % HLDS/RTTI -> MLDS
 :- import_module ml_backend.ml_elim_nested.         % MLDS -> MLDS
-:- import_module ml_backend.ml_tailcall.            % MLDS -> MLDS
 :- import_module ml_backend.ml_optimize.            % MLDS -> MLDS
-:- import_module ml_backend.mlds_to_c.              % MLDS -> C
-:- import_module ml_backend.mlds_to_java.           % MLDS -> Java
-:- import_module ml_backend.mlds_to_cs.             % MLDS -> C#
-:- import_module ml_backend.mlds_to_ilasm.          % MLDS -> IL assembler
-:- import_module ml_backend.ml_util.                % MLDS utility predicates
-:- import_module parse_tree.
-:- import_module parse_tree.error_util.
+:- import_module ml_backend.mlds_to_c_file.         % MLDS -> C
+:- import_module ml_backend.mlds_to_c_util.
+:- import_module ml_backend.mlds_to_java_file.      % MLDS -> Java
+:- import_module ml_backend.mlds_to_cs_file.        % MLDS -> C#
 :- import_module parse_tree.file_names.
 :- import_module top_level.mercury_compile_front_end.
 :- import_module top_level.mercury_compile_llds_back_end.
 
 :- import_module bool.
-:- import_module getopt_io.
-:- import_module list.
 :- import_module pprint.
 :- import_module require.
 :- import_module string.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-mlds_has_main(MLDS) =
-    (
-        Defns = MLDS ^ mlds_defns,
-        defns_contain_main(Defns)
-    ->
-        has_main
-    ;
-        no_main
-    ).
-
-%-----------------------------------------------------------------------------%
-
-mlds_backend(!HLDS, !:MLDS, !DumpInfo, !IO) :-
+mlds_backend(!HLDS, !:MLDS, !:Specs, !DumpInfo, !IO) :-
     module_info_get_globals(!.HLDS, Globals),
     globals.lookup_bool_option(Globals, verbose, Verbose),
     globals.lookup_bool_option(Globals, statistics, Stats),
 
     maybe_simplify(no, simplify_pass_ml_backend, Verbose, Stats, !HLDS, [],
         SimplifySpecs, !IO),
-    expect(unify(contains_errors(Globals, SimplifySpecs), no), $module, $pred,
+    expect(unify(contains_errors(Globals, SimplifySpecs), no), $pred,
         "simplify has errors"),
     maybe_dump_hlds(!.HLDS, 405, "ml_backend_simplify", !DumpInfo, !IO),
 
@@ -128,68 +110,52 @@ mlds_backend(!HLDS, !:MLDS, !DumpInfo, !IO) :-
     map_args_to_regs(Verbose, Stats, !HLDS, !IO),
     maybe_dump_hlds(!.HLDS, 425, "args_to_regs", !DumpInfo, !IO),
 
+    !:Specs = [],
+    maybe_mark_tail_rec_calls_hlds(Verbose, Stats, !HLDS, !Specs, !IO),
+    maybe_dump_hlds(!.HLDS, 430, "mark_tail_calls", !DumpInfo, !IO),
+
+    globals.get_target(Globals, Target),
+    (
+        Target = target_c,
+        MLDS_Target = ml_target_c
+    ;
+        Target = target_java,
+        MLDS_Target = ml_target_java
+    ;
+        Target = target_csharp,
+        MLDS_Target = ml_target_csharp
+    ),
     maybe_write_string(Verbose, "% Converting HLDS to MLDS...\n", !IO),
-    ml_code_gen(!HLDS, !:MLDS),
+    ml_code_gen(MLDS_Target, !:MLDS, !HLDS, !Specs),
     maybe_write_string(Verbose, "% done.\n", !IO),
     maybe_report_stats(Stats, !IO),
     maybe_dump_hlds(!.HLDS, 499, "final", !DumpInfo, !IO),
     maybe_dump_mlds(Globals, !.MLDS, 0, "initial", !IO),
 
     maybe_write_string(Verbose, "% Generating RTTI data...\n", !IO),
-    mlds_gen_rtti_data(!.HLDS, !MLDS),
+    mlds_gen_rtti_data(!.HLDS, MLDS_Target, !MLDS),
     maybe_write_string(Verbose, "% done.\n", !IO),
     maybe_report_stats(Stats, !IO),
     maybe_dump_mlds(Globals, !.MLDS, 10, "rtti", !IO),
 
-    % Detection of tail calls needs to occur before the
-    % chain_gc_stack_frame pass of ml_elim_nested, because we need to
-    % unlink the stack frame from the stack chain before tail calls.
-    globals.lookup_bool_option(Globals, optimize_tailcalls, OptimizeTailCalls),
-    (
-        OptimizeTailCalls = yes,
-        maybe_write_string(Verbose, "% Detecting tail calls...\n", !IO),
-        ml_mark_tailcalls(!MLDS, !IO),
-        maybe_write_string(Verbose, "% done.\n", !IO)
-    ;
-        OptimizeTailCalls = no
-    ),
-    maybe_report_stats(Stats, !IO),
-    maybe_dump_mlds(Globals, !.MLDS, 20, "tailcalls", !IO),
-
-    % Warning about non-tail calls must come after detection of tail calls.
-    globals.lookup_bool_option(Globals, warn_non_tail_recursion,
-        WarnTailCalls),
-    (
-        OptimizeTailCalls = yes,
-        WarnTailCalls = yes
-    ->
-        maybe_write_string(Verbose,
-            "% Warning about non-tail recursive calls...\n", !IO),
-        ml_warn_tailcalls(Globals, !.MLDS, !IO),
-        maybe_write_string(Verbose, "% done.\n", !IO)
-    ;
-        true
-    ),
-    maybe_report_stats(Stats, !IO),
-
     % Run the ml_optimize pass before ml_elim_nested, so that we eliminate
     % as many local variables as possible before the ml_elim_nested
-    % transformations. We also want to do tail recursion optimization before
-    % ml_elim_nested, since this means that the stack-handling code for
-    % accurate GC will go outside the loop rather than inside the loop.
+    % transformations.
     %
     % However, we need to disable optimize_initializations, because
     % ml_elim_nested doesn't correctly handle code containing initializations.
-    globals.lookup_bool_option(Globals, optimize, Optimize),
+    globals.get_opt_tuple(Globals, OptTuple),
+    Optimize = OptTuple ^ ot_optimize,
     (
-        Optimize = yes,
-        globals.set_option(optimize_initializations, bool(no),
-            Globals, NoInitOptGlobals),
+        Optimize = optimize,
+        NoInitOptTuple = OptTuple ^ ot_opt_initializations
+            := do_not_opt_initializations,
+        globals.set_opt_tuple(NoInitOptTuple, Globals, NoInitOptGlobals),
         maybe_write_string(Verbose, "% Optimizing MLDS...\n", !IO),
         mlds_optimize(NoInitOptGlobals, !MLDS),
         maybe_write_string(Verbose, "% done.\n", !IO)
     ;
-        Optimize = no
+        Optimize = do_not_optimize
     ),
     maybe_report_stats(Stats, !IO),
     maybe_dump_mlds(Globals, !.MLDS, 25, "optimize1", !IO),
@@ -206,7 +172,7 @@ mlds_backend(!HLDS, !:MLDS, !DumpInfo, !IO) :-
         GC = gc_accurate,
         maybe_write_string(Verbose,
             "% Threading GC stack frames...\n", !IO),
-        ml_elim_nested(chain_gc_stack_frames, Globals, !MLDS),
+        ml_elim_nested(chain_gc_stack_frames, Globals, MLDS_Target, !MLDS),
         maybe_write_string(Verbose, "% done.\n", !IO)
     ;
         ( GC = gc_automatic
@@ -214,21 +180,14 @@ mlds_backend(!HLDS, !:MLDS, !DumpInfo, !IO) :-
         ; GC = gc_boehm
         ; GC = gc_boehm_debug
         ; GC = gc_hgc
-        ; GC = gc_mps
         )
     ),
     maybe_report_stats(Stats, !IO),
     maybe_dump_mlds(Globals, !.MLDS, 30, "gc_frames", !IO),
 
-    globals.lookup_bool_option(Globals, gcc_nested_functions, NestedFuncs),
-    (
-        NestedFuncs = no,
-        maybe_write_string(Verbose, "% Flattening nested functions...\n", !IO),
-        ml_elim_nested(hoist_nested_funcs, Globals, !MLDS),
-        maybe_write_string(Verbose, "% done.\n", !IO)
-    ;
-        NestedFuncs = yes
-    ),
+    maybe_write_string(Verbose, "% Flattening nested functions...\n", !IO),
+    ml_elim_nested(hoist_nested_funcs, Globals, MLDS_Target, !MLDS),
+    maybe_write_string(Verbose, "% done.\n", !IO),
     maybe_report_stats(Stats, !IO),
     maybe_dump_mlds(Globals, !.MLDS, 35, "nested_funcs", !IO),
 
@@ -236,33 +195,19 @@ mlds_backend(!HLDS, !:MLDS, !DumpInfo, !IO) :-
     % optimize_initializations. (It may also help pick up some additional
     % optimization opportunities for the other optimizations in this pass.)
     (
-        Optimize = yes,
+        Optimize = optimize,
         maybe_write_string(Verbose, "% Optimizing MLDS again...\n", !IO),
         mlds_optimize(Globals, !MLDS),
         maybe_write_string(Verbose, "% done.\n", !IO)
     ;
-        Optimize = no
+        Optimize = do_not_optimize
     ),
     maybe_report_stats(Stats, !IO),
     maybe_dump_mlds(Globals, !.MLDS, 40, "optimize2", !IO),
 
     maybe_dump_mlds(Globals, !.MLDS, 99, "final", !IO).
 
-%-----------------------------------------------------------------------------%
-
-maybe_mark_static_terms(Verbose, Stats, !HLDS, !IO) :-
-    module_info_get_globals(!.HLDS, Globals),
-    globals.lookup_bool_option(Globals, static_ground_cells, SGCells),
-    (
-        SGCells = yes,
-        maybe_write_string(Verbose, "% Marking static ground terms...\n", !IO),
-        maybe_flush_output(Verbose, !IO),
-        process_all_nonimported_procs(update_proc(mark_static_terms), !HLDS),
-        maybe_write_string(Verbose, "% done.\n", !IO),
-        maybe_report_stats(Stats, !IO)
-    ;
-        SGCells = no
-    ).
+%---------------------------------------------------------------------------%
 
 :- pred maybe_add_trail_ops(bool::in, bool::in,
     module_info::in, module_info::out, io::di, io::uo) is det.
@@ -291,23 +236,20 @@ maybe_add_trail_ops(Verbose, Stats, !HLDS, !IO) :-
         globals.get_target(Globals, Target),
         (
             Target = target_c,
-            globals.lookup_bool_option(Globals, generate_trail_ops_inline,
-                GenerateInline)
+            globals.get_opt_tuple(Globals, OptTuple),
+            GenerateInline = OptTuple ^ ot_gen_trail_ops_inline
         ;
             % XXX Currently, we can only generate trail ops inline for
             % the C backends.
             %
-            ( Target = target_il
-            ; Target = target_csharp
+            ( Target = target_csharp
             ; Target = target_java
-            ; Target = target_x86_64
-            ; Target = target_erlang
             ),
-            GenerateInline = no
+            GenerateInline = do_not_gen_trail_ops_inline
         ),
         maybe_write_string(Verbose, "% Adding trailing operations...\n", !IO),
         maybe_flush_output(Verbose, !IO),
-        process_all_nonimported_procs(
+        process_valid_nonimported_procs(
             update_proc(add_trail_ops(OptTrailUse, GenerateInline)), !HLDS),
         maybe_write_string(Verbose, "% done.\n", !IO),
         maybe_report_stats(Stats, !IO)
@@ -325,38 +267,63 @@ maybe_add_heap_ops(Verbose, Stats, !HLDS, !IO) :-
         SemidetReclaim),
     globals.lookup_bool_option(Globals, reclaim_heap_on_nondet_failure,
         NondetReclaim),
-    (
+    ( if
         gc_is_conservative(GC) = yes
-    ->
+    then
         % We can't do heap reclamation with conservative GC.
         true
-    ;
+    else if
         SemidetReclaim = no,
         NondetReclaim = no
-    ->
+    then
         true
-    ;
+    else if
         SemidetReclaim = yes,
         NondetReclaim = yes
-    ->
+    then
         maybe_write_string(Verbose,
             "% Adding heap reclamation operations...\n", !IO),
         maybe_flush_output(Verbose, !IO),
-        process_all_nonimported_procs(update_proc(add_heap_ops), !HLDS),
+        process_valid_nonimported_procs(update_proc(add_heap_ops), !HLDS),
         maybe_write_string(Verbose, "% done.\n", !IO),
         maybe_report_stats(Stats, !IO)
-    ;
-        Msg = "Sorry, not implemented: `--high-level-code' and just one of " ++
-            "`--reclaim-heap-on-semidet-failure' and " ++
-            "`--reclaim-heap-on-nondet-failure'. " ++
-            "Use `--(no-)reclaim-heap-on-failure' instead.",
-        write_error_pieces_plain(Globals, [words(Msg)], !IO),
-        io.set_exit_status(1, !IO)
+    else
+        Pieces = [words("Sorry, not implemented:"),
+            quote("--high-level-code"), words("and just one of"),
+            quote("--reclaim-heap-on-semidet-failure"), words("and"),
+            quote("--reclaim-heap-on-nondet-failure"), suffix("."),
+            words("Use"), quote("--(no-)reclaim-heap-on-failure"),
+            words("instead."), nl],
+        Spec = simplest_no_context_spec($pred, severity_error,
+            phase_read_files, Pieces),
+        write_error_spec(Globals, Spec, !IO)
     ).
 
-:- pred mlds_gen_rtti_data(module_info::in, mlds::in, mlds::out) is det.
+:- pred maybe_mark_tail_rec_calls_hlds(bool::in, bool::in,
+    module_info::in, module_info::out,
+    list(error_spec)::in, list(error_spec)::out, io::di, io::uo) is det.
 
-mlds_gen_rtti_data(HLDS, !MLDS) :-
+maybe_mark_tail_rec_calls_hlds(Verbose, Stats, !HLDS, !Specs, !IO) :-
+    module_info_get_globals(!.HLDS, Globals),
+    globals.get_opt_tuple(Globals, OptTuple),
+    OptimizeTailCalls = OptTuple ^ ot_opt_mlds_tailcalls,
+    (
+        OptimizeTailCalls = opt_mlds_tailcalls,
+        maybe_write_string(Verbose, "% Marking tail recursive calls...", !IO),
+        maybe_flush_output(Verbose, !IO),
+        module_info_rebuild_dependency_info(!HLDS, DepInfo),
+        mark_self_and_mutual_tail_rec_calls_in_module_for_mlds_code_gen(
+            DepInfo, !HLDS, !Specs),
+        maybe_write_string(Verbose, " done.\n", !IO),
+        maybe_report_stats(Stats, !IO)
+    ;
+        OptimizeTailCalls = do_not_opt_mlds_tailcalls
+    ).
+
+:- pred mlds_gen_rtti_data(module_info::in, mlds_target_lang::in,
+    mlds::in, mlds::out) is det.
+
+mlds_gen_rtti_data(HLDS, Target, !MLDS) :-
     type_ctor_info.generate_rtti(HLDS, TypeCtorRtti),
     generate_base_typeclass_info_rtti(HLDS, TypeClassInfoRtti),
 
@@ -365,57 +332,46 @@ mlds_gen_rtti_data(HLDS, !MLDS) :-
     generate_type_class_info_rtti(HLDS, NewTypeClassRtti,
         NewTypeClassInfoRttiData),
     RttiDatas = TypeCtorRtti ++ TypeClassInfoRtti ++ NewTypeClassInfoRttiData,
-    !.MLDS = mlds(ModuleName, ForeignCode, Imports, GlobalData0, Defns,
-        InitPreds, FinalPreds, ExportedEnums),
-    add_rtti_datas_to_mlds(HLDS, RttiDatas, GlobalData0, GlobalData),
-    !:MLDS = mlds(ModuleName, ForeignCode, Imports, GlobalData, Defns,
-        InitPreds, FinalPreds, ExportedEnums).
+    GlobalData0 = !.MLDS ^ mlds_global_defns,
+    add_rtti_datas_to_mlds(HLDS, Target, RttiDatas, GlobalData0, GlobalData),
+    !MLDS ^ mlds_global_defns := GlobalData.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % The `--high-level-code' MLDS output pass.
 %
 
-mlds_to_high_level_c(Globals, MLDS, !IO) :-
+mlds_to_high_level_c(Globals, MLDS, Succeeded, !IO) :-
     globals.lookup_bool_option(Globals, verbose, Verbose),
     globals.lookup_bool_option(Globals, statistics, Stats),
 
     maybe_write_string(Verbose, "% Converting MLDS to C...\n", !IO),
-    output_c_mlds(MLDS, Globals, "", !IO),
+    output_c_mlds(MLDS, Globals, tod_target, "", Succeeded, !IO),
     maybe_write_string(Verbose, "% Finished converting MLDS to C.\n", !IO),
     maybe_report_stats(Stats, !IO).
 
-mlds_to_java(HLDS, MLDS, !IO) :-
+mlds_to_java(HLDS, MLDS, Succeeded, !IO) :-
     module_info_get_globals(HLDS, Globals),
     globals.lookup_bool_option(Globals, verbose, Verbose),
     globals.lookup_bool_option(Globals, statistics, Stats),
 
     maybe_write_string(Verbose, "% Converting MLDS to Java...\n", !IO),
-    output_java_mlds(HLDS, MLDS, !IO),
+    output_java_mlds(HLDS, MLDS, Succeeded, !IO),
     maybe_write_string(Verbose, "% Finished converting MLDS to Java.\n", !IO),
     maybe_report_stats(Stats, !IO).
 
-mlds_to_csharp(HLDS, MLDS, !IO) :-
+mlds_to_csharp(HLDS, MLDS, Succeeded, !IO) :-
     module_info_get_globals(HLDS, Globals),
     globals.lookup_bool_option(Globals, verbose, Verbose),
     globals.lookup_bool_option(Globals, statistics, Stats),
 
     maybe_write_string(Verbose, "% Converting MLDS to C#...\n", !IO),
-    output_csharp_mlds(HLDS, MLDS, !IO),
+    output_csharp_mlds(HLDS, MLDS, Succeeded, !IO),
     maybe_write_string(Verbose, "% Finished converting MLDS to C#.\n", !IO),
     maybe_report_stats(Stats, !IO).
 
-mlds_to_il_assembler(Globals, MLDS, !IO) :-
-    globals.lookup_bool_option(Globals, verbose, Verbose),
-    globals.lookup_bool_option(Globals, statistics, Stats),
-
-    maybe_write_string(Verbose, "% Converting MLDS to IL...\n", !IO),
-    output_mlds_via_ilasm(Globals, MLDS, !IO),
-    maybe_write_string(Verbose, "% Finished converting MLDS to IL.\n", !IO),
-    maybe_report_stats(Stats, !IO).
-
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred maybe_dump_mlds(globals::in, mlds::in, int::in, string::in,
     io::di, io::uo) is det.
@@ -425,31 +381,62 @@ maybe_dump_mlds(Globals, MLDS, StageNum, StageName, !IO) :-
     globals.lookup_accumulating_option(Globals, dump_mlds, DumpStages),
     globals.lookup_accumulating_option(Globals, verbose_dump_mlds,
         VerboseDumpStages),
+    globals.lookup_accumulating_option(Globals, dump_mlds_pred_name,
+        DumpPredNames),
     StageNumStr = stage_num_str(StageNum),
-    ( should_dump_stage(StageNum, StageNumStr, StageName, DumpStages) ->
+    globals.get_target(Globals, Target),
+    ( if
+        should_dump_stage(StageNum, StageNumStr, StageName, DumpStages),
+        Target = target_c
+    then
         maybe_write_string(Verbose, "% Dumping out MLDS as C...\n", !IO),
         maybe_flush_output(Verbose, !IO),
         DumpSuffix = "_dump." ++ StageNumStr ++ "-" ++ StageName,
-        output_c_mlds(MLDS, Globals, DumpSuffix, !IO),
+        (
+            DumpPredNames = [],
+            output_c_mlds(MLDS, Globals, tod_dump, DumpSuffix, _Succeeded, !IO)
+        ;
+            DumpPredNames = [_ | _],
+            output_c_dump_preds(MLDS, Globals, tod_dump, DumpSuffix,
+                DumpPredNames, !IO)
+        ),
         maybe_write_string(Verbose, "% done.\n", !IO)
-    ;
+    else
         true
     ),
-    ( should_dump_stage(StageNum, StageNumStr, StageName, VerboseDumpStages) ->
+    ( if
+        (
+            should_dump_stage(StageNum, StageNumStr, StageName,
+                VerboseDumpStages)
+        ;
+            Target \= target_c,
+            should_dump_stage(StageNum, StageNumStr, StageName, DumpStages)
+        )
+    then
+        (
+            DumpPredNames = [],
+            Doc = pprint.to_doc(MLDS)
+        ;
+            DumpPredNames = [_ | _],
+            ProcDefns = MLDS ^ mlds_proc_defns,
+            list.filter(func_defn_has_name_in_list(DumpPredNames), ProcDefns,
+                SelectedProcDefns),
+            Doc = pprint.to_doc(SelectedProcDefns)
+        ),
         maybe_write_string(Verbose, "% Dumping out raw MLDS...\n", !IO),
         ModuleName = mlds_get_module_name(MLDS),
-        module_name_to_file_name(Globals, ModuleName, ".mlds_dump",
-            do_create_dirs, BaseFileName, !IO),
+        module_name_to_file_name(Globals, $pred, do_create_dirs,
+            ext_other(other_ext(".mlds_dump")), ModuleName, BaseFileName, !IO),
         DumpFile = BaseFileName ++ "." ++ StageNumStr ++ "-" ++ StageName,
-        dump_mlds(Globals, DumpFile, MLDS, !IO),
+        dump_mlds_doc(Globals, DumpFile, Doc, !IO),
         maybe_write_string(Verbose, "% done.\n", !IO)
-    ;
+    else
         true
     ).
 
-:- pred dump_mlds(globals::in, string::in, mlds::in, io::di, io::uo) is det.
+:- pred dump_mlds_doc(globals::in, string::in, doc::in, io::di, io::uo) is det.
 
-dump_mlds(Globals, DumpFile, MLDS, !IO) :-
+dump_mlds_doc(Globals, DumpFile, Doc, !IO) :-
     globals.lookup_bool_option(Globals, verbose, Verbose),
     globals.lookup_bool_option(Globals, statistics, Stats),
     maybe_write_string(Verbose, "% Dumping out MLDS to `", !IO),
@@ -460,7 +447,7 @@ dump_mlds(Globals, DumpFile, MLDS, !IO) :-
     (
         Res = ok(FileStream),
         io.set_output_stream(FileStream, OutputStream, !IO),
-        pprint.write(80, pprint.to_doc(MLDS), !IO),
+        pprint.write(80, Doc, !IO),
         io.nl(!IO),
         io.set_output_stream(OutputStream, _, !IO),
         io.close_output(FileStream, !IO),
@@ -474,6 +461,6 @@ dump_mlds(Globals, DumpFile, MLDS, !IO) :-
         report_error(ErrorMessage, !IO)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module top_level.mercury_compile_mlds_back_end.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
