@@ -1,10 +1,11 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2000-2001, 2003-2011 The University of Melbourne.
+% Copyright (C) 2015-2018 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: ml_simplify_switch.m.
 % Main author: fjh.
@@ -20,7 +21,7 @@
 % depends on the target (e.g. whether it understands switches) and the
 % --prefer-switch option.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module ml_backend.ml_simplify_switch.
 :- interface.
@@ -28,21 +29,26 @@
 :- import_module ml_backend.mlds.
 :- import_module ml_backend.ml_gen_info.
 
-:- pred ml_simplify_switch(mlds_stmt::in, mlds_context::in,
-    statement::out, ml_gen_info::in, ml_gen_info::out) is det.
+:- pred ml_simplify_switch(mlds_stmt::in, mlds_stmt::out,
+    ml_gen_info::in, ml_gen_info::out) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module backend_libs.
 :- import_module backend_libs.builtin_ops.
 :- import_module backend_libs.switch_util.
+:- import_module libs.
 :- import_module libs.globals.
+:- import_module libs.optimization_options.
 :- import_module libs.options.
 :- import_module ml_backend.ml_code_util.
-:- import_module ml_backend.ml_switch_gen.
 :- import_module ml_backend.ml_target_util.
+:- import_module ml_backend.ml_util.
+:- import_module parse_tree.
+:- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_type.
 
 :- import_module bool.
@@ -52,87 +58,89 @@
 :- import_module maybe.
 :- import_module require.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-ml_simplify_switch(Stmt0, MLDS_Context, Statement, !Info) :-
+ml_simplify_switch(Stmt0, Stmt, !Info) :-
     ml_gen_info_get_globals(!.Info, Globals),
-    (
+    ( if
         % Convert dense int switches into computed gotos,
         % unless the target prefers switches.
 
         % Is this an int switch?
-        Stmt0 = ml_stmt_switch(Type, Rval, Range, Cases, Default),
-        is_integral_type(Type) = yes,
+        Stmt0 = ml_stmt_switch(Type, Rval, Range, Cases, Default, Context),
+        is_integral_type(Type, IntType),
 
         % Does the target want us to convert dense int switches
         % into computed gotos?
         globals_target_supports_computed_goto(Globals) = yes,
-        \+ (
-            globals_target_supports_int_switch(Globals) = yes,
+        not (
+            globals_target_supports_int_type_switch(Globals, IntType) = yes,
             globals.lookup_bool_option(Globals, prefer_switch, yes)
         ),
 
         % Is the switch big enough?
         list.length(Cases, NumCases),
-        globals.lookup_int_option(Globals, dense_switch_size, DenseSize),
+        globals.get_opt_tuple(Globals, OptTuple),
+        DenseSize = OptTuple ^ ot_dense_switch_size,
         NumCases >= DenseSize,
 
         % ... and dense enough?
-        globals.lookup_int_option(Globals, dense_switch_req_density,
-            ReqDensity),
+        ReqDensity = OptTuple ^ ot_dense_switch_req_density,
         is_dense_switch(Cases, ReqDensity)
-    ->
+    then
         maybe_eliminate_default(Range, Cases, Default, ReqDensity,
             FirstVal, LastVal, NeedRangeCheck),
         generate_dense_switch(Cases, Default, FirstVal, LastVal,
-            NeedRangeCheck, Type, Rval, MLDS_Context,
-            Decls, Statements, !Info),
-        Stmt = ml_stmt_block(Decls, Statements),
-        Statement = statement(Stmt, MLDS_Context)
-    ;
+            NeedRangeCheck, Type, Rval, Context, Stmts, !Info),
+        Stmt = ml_stmt_block([], [], Stmts, Context)
+    else if
         % Convert the remaining (sparse) int switches into if-then-else chains,
         % unless the target prefers switches.
 
-        Stmt0 = ml_stmt_switch(Type, Rval, _Range, Cases, Default),
-        is_integral_type(Type) = yes,
-        \+ (
-            globals_target_supports_int_switch(Globals) = yes,
+        Stmt0 = ml_stmt_switch(Type, Rval, _Range, Cases, Default, Context),
+        is_integral_type(Type, IntType),
+        not (
+            globals_target_supports_int_type_switch(Globals, IntType) = yes,
             globals.lookup_bool_option(Globals, prefer_switch, yes)
         )
-    ->
-        Statement = ml_switch_to_if_else_chain(Cases, Default, Rval,
-            MLDS_Context)
-    ;
+    then
+        Stmt = ml_switch_to_if_else_chain(Cases, Default, Rval, Context)
+    else if
         % Optimize away trivial switches (these can occur e.g. with
         % --tags none, where the tag test always has only one reachable case)
 
-        Stmt0 = ml_stmt_switch(_Type, _Rval, _Range, Cases, Default),
+        Stmt0 = ml_stmt_switch(_Type, _Rval, _Range, Cases, Default, _Context),
         Cases = [SingleCase],
         Default = default_is_unreachable
-    ->
-        SingleCase = mlds_switch_case(_FirstCond, _LaterConds, CaseStatement),
-        Statement = CaseStatement
-    ;
-        Stmt = Stmt0,
-        Statement = statement(Stmt, MLDS_Context)
+    then
+        SingleCase = mlds_switch_case(_FirstCond, _LaterConds, CaseStmt),
+        Stmt = CaseStmt
+    else
+        Stmt = Stmt0
     ).
 
-:- func is_integral_type(mlds_type) = bool.
+:- pred is_integral_type(mlds_type::in, int_type::out) is semidet.
 
-is_integral_type(MLDSType) = IsIntegral :-
+is_integral_type(MLDSType, IntType) :-
+    require_complete_switch [MLDSType]
     (
-        ( MLDSType = mlds_native_int_type
-        ; MLDSType = mlds_native_char_type
+        MLDSType = mlds_builtin_type_int(IntType)
+    ;
+        MLDSType = mlds_builtin_type_char,
+        IntType = int_type_int
+    ;
+        ( MLDSType = mlds_builtin_type_float
+        ; MLDSType = mlds_builtin_type_string
         ),
-        IsIntegral = yes
+        fail
     ;
         ( MLDSType = mlds_mercury_array_type(_)
         ; MLDSType = mlds_cont_type(_)
         ; MLDSType = mlds_commit_type
         ; MLDSType = mlds_native_bool_type
-        ; MLDSType = mlds_native_float_type
+        ; MLDSType = mlds_builtin_type_float
         ; MLDSType = mlds_foreign_type(_)
-        ; MLDSType = mlds_class_type(_, _, _)
+        ; MLDSType = mlds_class_type(_)
         ; MLDSType = mlds_ptr_type(_)
         ; MLDSType = mlds_func_type(_)
         ; MLDSType = mlds_type_info_type
@@ -145,34 +153,35 @@ is_integral_type(MLDSType) = IsIntegral :-
         ; MLDSType = mlds_tabling_type(_)
         ; MLDSType = mlds_unknown_type
         ),
-        IsIntegral = no
+        fail
     ;
-        MLDSType = mercury_type(_, CtorCat, _),
+        MLDSType = mercury_nb_type(_, CtorCat),
+        require_complete_switch [CtorCat]
         (
-            ( CtorCat = ctor_cat_builtin(cat_builtin_int)
-            ; CtorCat = ctor_cat_builtin(cat_builtin_char)
-            ; CtorCat = ctor_cat_enum(cat_enum_mercury)
-            ),
-            IsIntegral = yes
+            CtorCat = ctor_cat_enum(cat_enum_mercury),
+            IntType = int_type_int
         ;
-            ( CtorCat = ctor_cat_builtin(cat_builtin_string)
-            ; CtorCat = ctor_cat_builtin(cat_builtin_float)
-            ; CtorCat = ctor_cat_higher_order
+            ( CtorCat = ctor_cat_higher_order
             ; CtorCat = ctor_cat_tuple
             ; CtorCat = ctor_cat_builtin_dummy
             ; CtorCat = ctor_cat_variable
             ; CtorCat = ctor_cat_void
             ; CtorCat = ctor_cat_system(_)
-            ; CtorCat = ctor_cat_user(cat_user_notag)
             ; CtorCat = ctor_cat_user(cat_user_general)
+            ; CtorCat = ctor_cat_user(cat_user_notag)
+            ; CtorCat = ctor_cat_user(cat_user_abstract_notag)
             ; CtorCat = ctor_cat_user(cat_user_direct_dummy)
+            ; CtorCat = ctor_cat_user(cat_user_abstract_dummy)
             ),
-            IsIntegral = no
+            fail
         ;
             CtorCat = ctor_cat_enum(cat_enum_foreign),
             % XXX We can switch on foreign enumerations in C, but this may
             % not be the case for the other target languages.
-            IsIntegral = no
+            fail
+        ;
+            CtorCat = ctor_cat_builtin(_),
+            unexpected($pred, "mercury_nb_type but ctor_cat_builtin")
         )
     ).
 
@@ -197,37 +206,37 @@ is_dense_switch(Cases, ReqDensity) :-
     %
 :- pred maybe_eliminate_default(mlds_switch_range::in,
     list(mlds_switch_case)::in, mlds_switch_default::in, int::in,
-    int::out, int::out, bool::out) is det.
+    int::out, int::out, need_range_check::out) is det.
 
 maybe_eliminate_default(Range, Cases, Default, ReqDensity,
         FirstVal, LastVal, NeedRangeCheck) :-
-    (
+    ( if
         Default \= default_is_unreachable,
         Range = mlds_switch_range(Min, Max),
         TypeRange = Max - Min + 1,
         NumCases = list.length(Cases),
         NoDefaultDensity = switch_density(NumCases, TypeRange),
         NoDefaultDensity > ReqDensity
-    ->
-        NeedRangeCheck = no,
+    then
+        NeedRangeCheck = dont_need_range_check,
         FirstVal = Min,
         LastVal = Max
-    ;
+    else
         (
             Default = default_is_unreachable,
-            NeedRangeCheck = no
+            NeedRangeCheck = dont_need_range_check
         ;
             ( Default = default_do_nothing
             ; Default = default_case(_)
             ),
-            NeedRangeCheck = yes
+            NeedRangeCheck = need_range_check
         ),
         find_min_and_max_in_cases(Cases, FirstCaseVal, LastCaseVal),
         FirstVal = FirstCaseVal,
         LastVal = LastCaseVal
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Find the highest and lowest case values in a list of cases.
     %
@@ -242,7 +251,7 @@ find_min_and_max_in_cases(Cases, Min, Max) :-
     int::in, int::out, int::in, int::out) is det.
 
 find_min_and_max_in_case(Case, !Min, !Max) :-
-    Case = mlds_switch_case(FirstCond, LaterConds, _CaseStatement),
+    Case = mlds_switch_case(FirstCond, LaterConds, _CaseStmt),
     find_min_and_max_in_case_cond(FirstCond, !Min, !Max),
     list.foldl2(find_min_and_max_in_case_cond, LaterConds, !Min, !Max).
 
@@ -250,146 +259,128 @@ find_min_and_max_in_case(Case, !Min, !Max) :-
     int::in, int::out, int::in, int::out) is det.
 
 find_min_and_max_in_case_cond(match_value(Rval), !Min, !Max) :-
-    (
+    ( if
         Rval = ml_const(mlconst_int(Val))
-    ->
+    then
         int.min(Val, !Min),
         int.max(Val, !Max)
-    ;
-        unexpected($module, $pred, "non-int case")
+    else
+        unexpected($pred, "non-int case")
     ).
-find_min_and_max_in_case_cond(match_range(MinRval, MaxRval),
-        !Min, !Max) :-
-    (
+find_min_and_max_in_case_cond(match_range(MinRval, MaxRval), !Min, !Max) :-
+    ( if
         MinRval = ml_const(mlconst_int(RvalMin)),
         MaxRval = ml_const(mlconst_int(RvalMax))
-    ->
+    then
         int.min(RvalMin, !Min),
         int.max(RvalMax, !Max)
-    ;
-        unexpected($module, $pred, "non-int case")
+    else
+        unexpected($pred, "non-int case")
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Generate code for a switch using a dense jump table.
     %
 :- pred generate_dense_switch(list(mlds_switch_case)::in,
-    mlds_switch_default::in, int::in, int::in, bool::in,
-    mlds_type::in, mlds_rval::in, mlds_context::in,
-    list(mlds_defn)::out, list(statement)::out,
+    mlds_switch_default::in, int::in, int::in, need_range_check::in,
+    mlds_type::in, mlds_rval::in, prog_context::in, list(mlds_stmt)::out,
     ml_gen_info::in, ml_gen_info::out) is det.
 
 generate_dense_switch(Cases, Default, FirstVal, LastVal, NeedRangeCheck,
-        _Type, Rval, MLDS_Context, Decls, Statements, !Info) :-
+        _Type, Rval, Context, Stmts, !Info) :-
     % If the case values start at some number other than 0,
     % then subtract that number to give us a zero-based index.
-    ( FirstVal = 0 ->
+    ( if FirstVal = 0 then
         Index = Rval
-    ;
-        Index = ml_binop(int_sub, Rval, ml_const(mlconst_int(FirstVal)))
+    else
+        Index = ml_binop(int_sub(int_type_int), Rval,
+            ml_const(mlconst_int(FirstVal)))
     ),
 
     % Now generate the jump table.
     ml_gen_new_label(EndLabel, !Info),
     map.init(CaseLabelsMap0),
-    generate_cases(Cases, EndLabel, CaseLabelsMap0,
-        CaseLabelsMap, CasesDecls, CasesCode, !Info),
+    generate_cases(Cases, EndLabel, CaseLabelsMap0, CaseLabelsMap,
+        CasesCode, !Info),
     ml_gen_new_label(DefaultLabel, !Info),
     CaseLabels = get_case_labels(FirstVal, LastVal,
         CaseLabelsMap, DefaultLabel),
-    DefaultLabelStatement = statement(ml_stmt_label(DefaultLabel),
-        MLDS_Context),
+    DefaultLabelStmt = ml_stmt_label(DefaultLabel, Context),
     (
         Default = default_is_unreachable,
         % We still need the label, in case we inserted references to it
         % into (unreachable) slots in the jump table.
-        DefaultStatements = [DefaultLabelStatement]
+        DefaultStmts = [DefaultLabelStmt]
     ;
         Default = default_do_nothing,
-        DefaultStatements = [DefaultLabelStatement]
+        DefaultStmts = [DefaultLabelStmt]
     ;
         Default = default_case(DefaultCase),
-        DefaultStatements = [DefaultLabelStatement, DefaultCase]
+        DefaultStmts = [DefaultLabelStmt, DefaultCase]
     ),
 
-    StartComment = statement(
-        ml_stmt_atomic(comment("switch (using dense jump table)")),
-        MLDS_Context),
-    DoJump = statement(ml_stmt_computed_goto(Index, CaseLabels), MLDS_Context),
-    EndLabelStatement = statement(ml_stmt_label(EndLabel), MLDS_Context),
-    EndComment = statement(ml_stmt_atomic(comment("End of dense switch")),
-        MLDS_Context),
+    StartComment = ml_stmt_atomic(comment("switch (using dense jump table)"),
+        Context),
+    DoJump = ml_stmt_computed_goto(Index, CaseLabels, Context),
+    EndLabelStmt = ml_stmt_label(EndLabel, Context),
+    EndComment = ml_stmt_atomic(comment("End of dense switch"), Context),
 
     % We may need to check that the value of the variable lies within the
     % appropriate range.
     (
-        NeedRangeCheck = yes,
+        NeedRangeCheck = need_range_check,
         Difference = LastVal - FirstVal,
         InRange = ml_binop(unsigned_le,
-            Index,
-            ml_const(mlconst_int(Difference))),
-        Else = yes(statement(ml_stmt_block([], DefaultStatements),
-            MLDS_Context)),
-        SwitchBody = statement(ml_stmt_block([], [DoJump | CasesCode]),
-            MLDS_Context),
-        DoSwitch = statement(ml_stmt_if_then_else(InRange, SwitchBody, Else),
-            MLDS_Context),
-        Statements = [StartComment, DoSwitch, EndLabelStatement, EndComment]
+            Index, ml_const(mlconst_int(Difference))),
+        Else = yes(ml_stmt_block([], [], DefaultStmts, Context)),
+        SwitchBody = ml_stmt_block([], [], [DoJump | CasesCode], Context),
+        DoSwitch = ml_stmt_if_then_else(InRange, SwitchBody, Else, Context),
+        Stmts = [StartComment, DoSwitch, EndLabelStmt, EndComment]
     ;
-        NeedRangeCheck = no,
-        Statements =
+        NeedRangeCheck = dont_need_range_check,
+        Stmts =
             [StartComment, DoJump | CasesCode] ++
-            DefaultStatements ++
-            [EndLabelStatement, EndComment]
-    ),
-    Decls = CasesDecls.
+            DefaultStmts ++
+            [EndLabelStmt, EndComment]
+    ).
 
 :- pred generate_cases(list(mlds_switch_case)::in, mlds_label::in,
-    case_labels_map::in, case_labels_map::out,
-    list(mlds_defn)::out, list(statement)::out,
+    case_labels_map::in, case_labels_map::out, list(mlds_stmt)::out,
     ml_gen_info::in, ml_gen_info::out) is det.
 
-generate_cases([], _EndLabel, !CaseLabelsMap, [], [], !Info).
-generate_cases([Case | Cases], EndLabel, !CaseLabelsMap, Decls, Statements,
-        !Info) :-
-    generate_case(Case, EndLabel, !CaseLabelsMap,
-        CaseDecls, CaseStatements, !Info),
-    generate_cases(Cases, EndLabel, !CaseLabelsMap,
-        CasesDecls, CasesStatements, !Info),
-    Decls = CaseDecls ++ CasesDecls,
-    Statements = CaseStatements ++ CasesStatements.
+generate_cases([], _EndLabel, !CaseLabelsMap, [], !Info).
+generate_cases([Case | Cases], EndLabel, !CaseLabelsMap, Stmts, !Info) :-
+    generate_case(Case, EndLabel, !CaseLabelsMap, CaseStmts, !Info),
+    generate_cases(Cases, EndLabel, !CaseLabelsMap, CasesStmts, !Info),
+    Stmts = CaseStmts ++ CasesStmts.
 
     % This converts an MLDS switch case into code for a dense switch case,
     % by adding a label at the front and a `goto <EndLabel>' at the end.
     % It also inserts the label for this case into the CaseLabelsMap.
     %
 :- pred generate_case(mlds_switch_case::in, mlds_label::in,
-    case_labels_map::in, case_labels_map::out,
-    list(mlds_defn)::out, list(statement)::out,
+    case_labels_map::in, case_labels_map::out, list(mlds_stmt)::out,
     ml_gen_info::in, ml_gen_info::out) is det.
 
-generate_case(Case, EndLabel, !CaseLabelsMap, Decls, Statements, !Info) :-
-    Case = mlds_switch_case(FirstCond, LaterConds, CaseStatement),
+generate_case(Case, EndLabel, !CaseLabelsMap, Stmts, !Info) :-
+    Case = mlds_switch_case(FirstCond, LaterConds, CaseStmt),
     ml_gen_new_label(ThisLabel, !Info),
     insert_case_into_map(ThisLabel, FirstCond, !CaseLabelsMap),
     list.foldl(insert_case_into_map(ThisLabel), LaterConds, !CaseLabelsMap),
-    CaseStatement = statement(_, MLDS_Context),
-    LabelComment = statement(ml_stmt_atomic(comment("case of dense switch")),
-        MLDS_Context),
-    LabelCode = statement(ml_stmt_label(ThisLabel), MLDS_Context),
-    JumpComment = statement(
-        ml_stmt_atomic(comment("branch to end of dense switch")),
-        MLDS_Context),
-    JumpCode = statement(ml_stmt_goto(goto_label(EndLabel)), MLDS_Context),
-    Decls = [],
-    Statements = [LabelComment, LabelCode, CaseStatement,
-        JumpComment, JumpCode].
+    Context = get_mlds_stmt_context(CaseStmt),
+    LabelComment = ml_stmt_atomic(comment("case of dense switch"), Context),
+    LabelCode = ml_stmt_label(ThisLabel, Context),
+    JumpComment = ml_stmt_atomic(comment("branch to end of dense switch"),
+        Context),
+    JumpCode = ml_stmt_goto(goto_label(EndLabel), Context),
+    Stmts = [LabelComment, LabelCode, CaseStmt, JumpComment, JumpCode].
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % We build up a map which records which label should be used for
 % each case value.
+%
 
 :- type case_labels_map == map(int, mlds_label).
 
@@ -399,20 +390,20 @@ generate_case(Case, EndLabel, !CaseLabelsMap, Decls, Statements, !Info) :-
 insert_case_into_map(ThisLabel, Cond, !CaseLabelsMap) :-
     (
         Cond = match_value(Rval),
-        ( Rval = ml_const(mlconst_int(Val)) ->
+        ( if Rval = ml_const(mlconst_int(Val)) then
             map.det_insert(Val, ThisLabel, !CaseLabelsMap)
-        ;
-            unexpected($module, $pred, "non-int case")
+        else
+            unexpected($pred, "non-int case")
         )
     ;
         Cond = match_range(MinRval, MaxRval),
-        (
+        ( if
             MinRval = ml_const(mlconst_int(Min)),
             MaxRval = ml_const(mlconst_int(Max))
-        ->
+        then
             insert_range_into_map(Min, Max, ThisLabel, !CaseLabelsMap)
-        ;
-            unexpected($module, $pred, "non-int case")
+        else
+            unexpected($pred, "non-int case")
         )
     ).
 
@@ -420,14 +411,14 @@ insert_case_into_map(ThisLabel, Cond, !CaseLabelsMap) :-
     case_labels_map::in, case_labels_map::out) is det.
 
 insert_range_into_map(Min, Max, ThisLabel, !CaseLabelsMap) :-
-    ( Min > Max ->
+    ( if Min > Max then
         true
-    ;
+    else
         map.det_insert(Min, ThisLabel, !CaseLabelsMap),
         insert_range_into_map(Min + 1, Max, ThisLabel, !CaseLabelsMap)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Given the starting and ending case values, the mapping from case values
     % to labels, and the default label to use for case values which aren't in
@@ -438,12 +429,12 @@ insert_range_into_map(Min, Max, ThisLabel, !CaseLabelsMap) :-
     = list(mlds_label).
 
 get_case_labels(ThisVal, LastVal, CaseLabelsMap, DefaultLabel) = CaseLabels :-
-    ( ThisVal > LastVal ->
+    ( if ThisVal > LastVal then
         CaseLabels = []
-    ;
-        ( map.search(CaseLabelsMap, ThisVal, CaseLabel0) ->
+    else
+        ( if map.search(CaseLabelsMap, ThisVal, CaseLabel0) then
             CaseLabel = CaseLabel0
-        ;
+        else
             CaseLabel = DefaultLabel
         ),
         CaseLabels1 = get_case_labels(ThisVal + 1, LastVal,
@@ -451,40 +442,39 @@ get_case_labels(ThisVal, LastVal, CaseLabelsMap, DefaultLabel) = CaseLabels :-
         CaseLabels = [CaseLabel | CaseLabels1]
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Convert an int switch to a chain of if-then-elses that test each case
     % in turn.
     %
 :- func ml_switch_to_if_else_chain(list(mlds_switch_case), mlds_switch_default,
-    mlds_rval, mlds_context) = statement.
+    mlds_rval, prog_context) = mlds_stmt.
 
-ml_switch_to_if_else_chain([], Default, _Rval, MLDS_Context) = Statement :-
+ml_switch_to_if_else_chain([], Default, _Rval, Context) = Stmt :-
     (
         Default = default_do_nothing,
-        Statement = statement(ml_stmt_block([], []), MLDS_Context)
+        Stmt = ml_stmt_block([], [], [], Context)
     ;
         Default = default_is_unreachable,
-        Statement = statement(ml_stmt_block([], []), MLDS_Context)
+        Stmt = ml_stmt_block([], [], [], Context)
     ;
-        Default = default_case(Statement)
+        Default = default_case(Stmt)
     ).
-ml_switch_to_if_else_chain([Case | Cases], Default, SwitchRval, MLDS_Context) =
-        Statement :-
-    Case = mlds_switch_case(FirstMatchCond, LaterMatchConds, CaseStatement),
-    (
+ml_switch_to_if_else_chain([Case | Cases], Default, SwitchRval, Context) =
+        Stmt :-
+    Case = mlds_switch_case(FirstMatchCond, LaterMatchConds, CaseStmt),
+    ( if
         Cases = [],
         Default = default_is_unreachable
-    ->
-        Statement = CaseStatement
-    ;
+    then
+        Stmt = CaseStmt
+    else
         AllMatchConds = [FirstMatchCond | LaterMatchConds],
         CaseMatchedRval = ml_gen_case_match_conds(AllMatchConds, SwitchRval),
-        RestStatement = ml_switch_to_if_else_chain(Cases, Default, SwitchRval,
-            MLDS_Context),
-        IfStmt = ml_stmt_if_then_else(CaseMatchedRval, CaseStatement,
-            yes(RestStatement)),
-        Statement = statement(IfStmt, MLDS_Context)
+        RestStmt = ml_switch_to_if_else_chain(Cases, Default, SwitchRval,
+            Context),
+        Stmt = ml_stmt_if_then_else(CaseMatchedRval, CaseStmt, yes(RestStmt),
+            Context)
     ).
 
     % Generate an rval which will be true iff any of the specified list of
@@ -508,12 +498,12 @@ ml_gen_case_match_conds([Cond1, Cond2 | Conds], SwitchRval) =
 :- func ml_gen_case_match_cond(mlds_case_match_cond, mlds_rval) = mlds_rval.
 
 ml_gen_case_match_cond(match_value(CaseRval), SwitchRval) =
-    ml_binop(eq, CaseRval, SwitchRval).
+    ml_binop(eq(int_type_int), CaseRval, SwitchRval).
 ml_gen_case_match_cond(match_range(MinRval, MaxRval), SwitchRval) =
     ml_binop(logical_and,
-        ml_binop(int_gt, SwitchRval, MinRval),
-        ml_binop(int_le, SwitchRval, MaxRval)).
+        ml_binop(int_gt(int_type_int), SwitchRval, MinRval),
+        ml_binop(int_le(int_type_int), SwitchRval, MaxRval)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module ml_backend.ml_simplify_switch.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
