@@ -1,12 +1,14 @@
 %---------------------------------------------------------------------------%
+% vim: ft=mercury ts=4 sw=4 et
+%---------------------------------------------------------------------------%
 % Copyright (C) 2001-2006 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
-% 
+%
 % File: mode_robdd.m.
 % Main author: dmo
-% 
+%
 % This module contains the package for mode_robbds, data structures built
 % around reduced ordered binary decision diagrams (ROBDDs) to help implement
 % mode checking via constraints.
@@ -39,4 +41,6 @@
 % :- import_module mode_robdd__check.
 % :- type mode_robdd(T) == check_robdd(T).
 
-:- implementation.
+%---------------------------------------------------------------------------%
+:- end_module mode_robdd.
+%---------------------------------------------------------------------------%
