@@ -30,7 +30,7 @@
 %
 % NOTE: many of the operations in this module require you to pass in
 % the varset that the variables in the constraints that define the polyhedron
-% were allocated from.  This because the code for computing the convex hull
+% were allocated from. This because the code for computing the convex hull
 % and the linear solver in lp_rational.m need to allocate fresh temporary
 % variables.
 %
@@ -60,96 +60,91 @@
 
 %-----------------------------------------------------------------------------%
 
-    % The `empty' polyhedron.  Equivalent to the constraint `false'.
+    % The `empty' polyhedron. Equivalent to the constraint `false'.
     %
-:- func polyhedron.empty = polyhedron.
+:- func empty = polyhedron.
 
-    % The `universe' polyhedron.  Equivalent to the constraint `true'.
+    % The `universe' polyhedron. Equivalent to the constraint `true'.
     %
-:- func polyhedron.universe = polyhedron.
+:- func universe = polyhedron.
 
     % Constructs a convex polyhedron from a system of linear constraints.
     %
-:- func polyhedron.from_constraints(constraints) = polyhedron.
+:- func from_constraints(constraints) = polyhedron.
 
     % Returns a system of constraints whose solution space defines
     % the given polyhedron.
     %
-:- func polyhedron.constraints(polyhedron) = constraints.
+:- func constraints(polyhedron) = constraints.
 
     % As above but throws an exception if the given polyhedron is empty.
     %
-:- func polyhedron.non_false_constraints(polyhedron) = constraints.
+:- func non_false_constraints(polyhedron) = constraints.
 
     % Succeeds iff the given polyhedron is the empty polyhedron.
     %
     % NOTE: this only succeeds if the polyhedron is actually *known*
-    % to be empty.  It might fail even when the constraint set is
-    % is inconsistent.  You currently need to call polyhedron.optimize
+    % to be empty. It might fail even when the constraint set is
+    % is inconsistent. You currently need to call polyhedron.optimize
     % to force this to always work.
     %
-:- pred polyhedron.is_empty(polyhedron::in) is semidet.
+:- pred is_empty(polyhedron::in) is semidet.
 
     % Succeeds iff the given polyhedron is the `universe' polyhedron,
     % that is the one whose constraint representation corresponds to `true'.
     % (ie. it is unbounded in all dimensions).
     %
-:- pred polyhedron.is_universe(polyhedron::in) is semidet.
+:- pred is_universe(polyhedron::in) is semidet.
 
     % Optimizes the representation of a polyhedron.
     % At the moment this performs a consistency check and then replaces the
     % polyhedron by empty if necessary.
     %
-:- pred polyhedron.optimize(lp_varset::in, polyhedron::in, polyhedron::out)
-    is det.
+:- pred optimize(lp_varset::in, polyhedron::in, polyhedron::out) is det.
 
-    % polyhedron.intersection(A, B, C).
-    % The polyhedron `C' is the intersection of the the
-    % polyhedra `A' and `B'.
+    % intersection(A, B, C):
+    % The polyhedron `C' is the intersection of  the polyhedra `A' and `B'.
     %
-:- func polyhedron.intersection(polyhedron, polyhedron) = polyhedron.
-:- pred polyhedron.intersection(polyhedron::in, polyhedron::in,
-    polyhedron::out) is det.
+:- func intersection(polyhedron, polyhedron) = polyhedron.
+:- pred intersection(polyhedron::in, polyhedron::in, polyhedron::out) is det.
 
     % Returns a polyhedron that is a closed convex approximation of
     % union of the two polyhedra.
     %
-:- func polyhedron.convex_union(lp_varset, polyhedron, polyhedron) =
-    polyhedron.
-:- pred polyhedron.convex_union(lp_varset::in, polyhedron::in, polyhedron::in,
+:- func convex_union(lp_varset, polyhedron, polyhedron) = polyhedron.
+:- pred convex_union(lp_varset::in, polyhedron::in, polyhedron::in,
     polyhedron::out) is det.
 
     % As above but takes an extra argument that weakens the approximation even
     % further if the size of the internal matrices exceeds the supplied
     % threshold
     %
-:- func polyhedron.convex_union(lp_varset, maybe(int), polyhedron,
-    polyhedron) = polyhedron.
-:- pred polyhedron.convex_union(lp_varset::in, maybe(int)::in, polyhedron::in,
+:- func convex_union(lp_varset, maybe(int), polyhedron, polyhedron)
+    = polyhedron.
+:- pred convex_union(lp_varset::in, maybe(int)::in, polyhedron::in,
     polyhedron::in, polyhedron::out) is det.
 
     % Approximate a (convex) polyhedron by a rectangular region
     % whose sides are parallel to the axes.
     %
-:- func polyhedron.bounding_box(polyhedron, lp_varset) = polyhedron.
+:- func bounding_box(polyhedron, lp_varset) = polyhedron.
 
-    % polyhedron.widen(A, B, Varset) = C.
+    % polyhedron.widen(A, B, VarSet) = C.
     % Remove faces from the polyhedron `A' to form the polyhedron `C'
     % according to the rules that the smallest number of faces
     % should be removed and that `C' must be a superset of `B'.
     % This operation is not commutative.
     %
-:- func polyhedron.widen(polyhedron, polyhedron, lp_varset) = polyhedron.
+:- func widen(polyhedron, polyhedron, lp_varset) = polyhedron.
 
-    % project_all(Varset, Variables, Polyhedra) returns a list
+    % project_all(VarSet, Variables, Polyhedra) returns a list
     % of polyhedra in which the variables listed have been eliminated
     % from each polyhedron.
     %
-:- func polyhedron.project_all(lp_varset, lp_vars, polyhedra) = polyhedra.
+:- func project_all(lp_varset, lp_vars, polyhedra) = polyhedra.
 
-:- func polyhedron.project(lp_vars, lp_varset, polyhedron) = polyhedron.
-:- pred polyhedron.project(lp_vars::in, lp_varset::in, polyhedron::in,
-    polyhedron::out) is det.
+:- pred project_polyhedron(lp_varset::in, lp_vars::in,
+    polyhedron::in, polyhedron::out) is det.
 
     % XXX It might be nicer to think of this as relabelling the axes.
     % Conceptually it alters the names of the variables in the polyhedron
@@ -157,27 +152,24 @@
     % easy to do (at the moment) as the polyhedra are represented as
     % constraints anyway.
     %
-:- func polyhedron.substitute_vars(lp_vars, lp_vars, polyhedron) = polyhedron.
-:- func polyhedron.substitute_vars(map(lp_var, lp_var), polyhedron) =
-    polyhedron.
+:- func substitute_vars(lp_vars, lp_vars, polyhedron) = polyhedron.
+:- func substitute_vars(map(lp_var, lp_var), polyhedron) = polyhedron.
 
     % polyhedron.zero_vars(Set, Polyhedron0) = Polyhedron <=>
     %
     %   Constraints0 = polyhedron.constraints(Polyhedron0),
-    %   Constraints  = lp_rational.set_vars_to_zero(
-    %       Set, Constraints0)
+    %   Constraints  = lp_rational.set_vars_to_zero(Set, Constraints0)
     %   Polyhedron   = polyhedron.from_constraints(Constraints)
     %
     % This is a little more efficient than the above because
     % we don't end up traversing the list of constraints as much.
     %
-:- func polyhedron.zero_vars(set(lp_var), polyhedron) = polyhedron.
+:- func zero_vars(set(lp_var), polyhedron) = polyhedron.
 
-    % Print out the polyhedron using the names of the variables in the
-    % varset.
+    % Print out the polyhedron using the names of the variables in the varset.
     %
-:- pred polyhedron.write_polyhedron(polyhedron::in, lp_varset::in, io::di,
-    io::uo) is det.
+:- pred write_polyhedron(io.text_output_stream::in, lp_varset::in,
+    polyhedron::in, io::di, io::uo) is det.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -204,59 +196,59 @@
 % Creation of polyhedra.
 %
 
-polyhedron.empty = empty_poly.
+empty = empty_poly.
 
-polyhedron.universe = eqns([]).
+universe = eqns([]).
 
     % This does the following:
     %   - checks if the constraint is false.
     %   - simplifies the representation of the constraint.
     %    - calls intersection/3 (which does further simplifications).
     %
-polyhedron.from_constraints([]) = eqns([]).
-polyhedron.from_constraints([C | Cs]) = Polyhedron :-
-    ( lp_rational.is_false(C) ->
+from_constraints([]) = eqns([]).
+from_constraints([C | Cs]) = Polyhedron :-
+    ( if lp_rational.is_false(C) then
         Polyhedron = empty
-    ;
+    else
         Polyhedron0 = polyhedron.from_constraints(Cs),
         Polyhedron = polyhedron.intersection(eqns([C]), Polyhedron0)
     ).
 
-polyhedron.constraints(eqns(Constraints)) = Constraints.
-polyhedron.constraints(empty_poly) = [lp_rational.false_constraint].
+constraints(eqns(Constraints)) = Constraints.
+constraints(empty_poly) = [lp_rational.false_constraint].
 
-polyhedron.non_false_constraints(eqns(Constraints)) = Constraints.
-polyhedron.non_false_constraints(empty_poly) =
-    unexpected($module, $pred, "empty polyhedron").
+non_false_constraints(eqns(Constraints)) = Constraints.
+non_false_constraints(empty_poly) =
+    unexpected($pred, "empty polyhedron").
 
-polyhedron.is_empty(empty_poly).
+is_empty(empty_poly).
 
-polyhedron.is_universe(eqns(Constraints)) :-
+is_universe(eqns(Constraints)) :-
     list.all_true(lp_rational.nonneg_constr, Constraints).
 
-polyhedron.optimize(_, empty_poly, empty_poly).
-polyhedron.optimize(Varset, eqns(Constraints0), Result) :-
+optimize(_, empty_poly, empty_poly).
+optimize(VarSet, eqns(Constraints0), Result) :-
     Constraints = simplify_constraints(Constraints0),
-    ( inconsistent(Varset, Constraints) ->
+    ( if inconsistent(VarSet, Constraints) then
         Result = empty_poly
-    ;
+    else
         Result = eqns(Constraints)
     ).
 
 %-----------------------------------------------------------------------------%
 %
-% Intersection
+% Intersection.
 %
 
-polyhedron.intersection(PolyA, PolyB, polyhedron.intersection(PolyA, PolyB)).
-
-polyhedron.intersection(empty_poly, _) = empty_poly.
-polyhedron.intersection(eqns(_), empty_poly) = empty_poly.
-polyhedron.intersection(eqns(MatrixA), eqns(MatrixB)) = eqns(Constraints) :-
+intersection(empty_poly, _) = empty_poly.
+intersection(eqns(_), empty_poly) = empty_poly.
+intersection(eqns(MatrixA), eqns(MatrixB)) = eqns(Constraints) :-
     Constraints0 = MatrixA ++ MatrixB,
     restore_equalities(Constraints0, Constraints1),
     Constraints = simplify_constraints(Constraints1).
 
+intersection(PolyA, PolyB, polyhedron.intersection(PolyA, PolyB)).
+
 %-----------------------------------------------------------------------------%
 %
 % Convex union.
@@ -267,25 +259,23 @@ polyhedron.intersection(eqns(MatrixA), eqns(MatrixB)) = eqns(Constraints) :-
 % bounding box approximation instead.
 %
 
-polyhedron.convex_union(Varset, PolyhedronA, PolyhedronB) = Polyhedron :-
-    convex_union(Varset, no, PolyhedronA, PolyhedronB, Polyhedron).
+convex_union(VarSet, PolyhedronA, PolyhedronB) = Polyhedron :-
+    convex_union(VarSet, no, PolyhedronA, PolyhedronB, Polyhedron).
 
-polyhedron.convex_union(Varset, PolyhedronA, PolyhedronB, Polyhedron) :-
-    convex_union(Varset, no, PolyhedronA, PolyhedronB, Polyhedron).
+convex_union(VarSet, PolyhedronA, PolyhedronB, Polyhedron) :-
+    convex_union(VarSet, no, PolyhedronA, PolyhedronB, Polyhedron).
 
-polyhedron.convex_union(Varset, MaxMatrixSize, PolyhedronA, PolyhedronB)
-        = Polyhedron :-
-    convex_union(Varset, MaxMatrixSize, PolyhedronA, PolyhedronB,
-        Polyhedron).
+convex_union(VarSet, MaxMatrixSize, PolyhedronA, PolyhedronB) = Polyhedron :-
+    convex_union(VarSet, MaxMatrixSize, PolyhedronA, PolyhedronB, Polyhedron).
 
-polyhedron.convex_union(_, _, empty_poly, empty_poly, empty_poly).
-polyhedron.convex_union(_, _, eqns(Constraints), empty_poly,
+convex_union(_, _, empty_poly, empty_poly, empty_poly).
+convex_union(_, _, eqns(Constraints), empty_poly,
     eqns(Constraints)).
-polyhedron.convex_union(_, _, empty_poly, eqns(Constraints),
+convex_union(_, _, empty_poly, eqns(Constraints),
     eqns(Constraints)).
-polyhedron.convex_union(Varset, MaybeMaxSize, eqns(ConstraintsA),
+convex_union(VarSet, MaybeMaxSize, eqns(ConstraintsA),
         eqns(ConstraintsB), Hull) :-
-    convex_hull([ConstraintsA, ConstraintsB], Hull, MaybeMaxSize, Varset).
+    convex_hull([ConstraintsA, ConstraintsB], Hull, MaybeMaxSize, VarSet).
 
 %-----------------------------------------------------------------------------%
 %
@@ -295,7 +285,7 @@ polyhedron.convex_union(Varset, MaybeMaxSize, eqns(ConstraintsA),
 % The following transformation for computing the convex hull is described in
 % the paper:
 %
-% F. Benoy and A. King.  Inferring Argument Size Relationships with CLPR(R).
+% F. Benoy and A. King. Inferring Argument Size Relationships with CLPR(R).
 % Logic-based Program Synthesis and Transformation,
 % LNCS 1207: pp. 204-223, 1997.
 %
@@ -324,7 +314,7 @@ polyhedron.convex_union(Varset, MaybeMaxSize, eqns(ConstraintsA),
     --->    polyhedra_info(
                 % There is one of these for each polyhedron. It maps the
                 % original variables in the constraints to the temporary
-                % variables introduced by the the transformation.
+                % variables introduced by the transformation.
                 % A variable that occurs in more than one polyhedron
                 % is mapped to a separate temporary variable for each one.
                 var_maps        :: var_maps,
@@ -351,31 +341,29 @@ polyhedron.convex_union(Varset, MaybeMaxSize, eqns(ConstraintsA),
     lp_varset::in) is det.
 
 convex_hull([], _, _, _) :-
-    unexpected($module, $pred, "empty list").
+    unexpected($pred, "empty list").
 convex_hull([Poly], eqns(Poly), _, _).
-convex_hull(Polys @ [_,_|_], ConvexHull, MaybeMaxSize, Varset0) :-
-    %
+convex_hull(Polys @ [_, _ | _], ConvexHull, MaybeMaxSize, VarSet0) :-
     % Perform the matrix transformation from the paper by Benoy and King.
     % Rename variables and add sigma constraints as necessary.
-    %
-    PolyInfo0 = polyhedra_info([], [], Varset0),
+    PolyInfo0 = polyhedra_info([], [], VarSet0),
     transform_polyhedra(Polys, Matrix0, PolyInfo0, PolyInfo),
-    PolyInfo = polyhedra_info(VarMaps, Sigmas, Varset),
+    PolyInfo = polyhedra_info(VarMaps, Sigmas, VarSet),
     add_sigma_constraints(Sigmas, Matrix0, Matrix1),
     Matrix   = add_last_constraints(Matrix1, VarMaps),
-    AppendValues = (func(Map, Varlist0) = Varlist :-
-        Varlist = Varlist0 ++ map.values(Map)
-    ),
+    AppendValues =
+        ( func(Map, Varlist0) = Varlist :-
+            Varlist = Varlist0 ++ map.values(Map)
+        ),
     VarsToEliminate = Sigmas ++ list.foldl(AppendValues, VarMaps, []),
-    %
+
     % Calculate the closure of the convex hull of the original polyhedra by
     % projecting the constraints in the transformed matrix onto the original
-    % variables (ie. eliminate all the sigma and temporary variables).  Since
-    % the resulting matrix tends to contain a large number of redundant
+    % variables (ie. eliminate all the sigma and temporary variables).
+    % Since the resulting matrix tends to contain a large number of redundant
     % constraints, we need to do a redundancy check after this.
-    %
-    lp_rational.project(VarsToEliminate, Varset, MaybeMaxSize, Matrix,
-        ProjectionResult),
+    project_constraints_maybe_size_limit(VarSet, MaybeMaxSize, VarsToEliminate,
+        Matrix, ProjectionResult),
     (
         % XXX We should try using a bounding box first.
         ProjectionResult = pr_res_aborted,
@@ -390,10 +378,9 @@ convex_hull(Polys @ [_,_|_], ConvexHull, MaybeMaxSize, Varset0) :-
             % XXX We should try removing this call to simplify constraints.
             %     It seems unnecessary.
             !:Hull = simplify_constraints(!.Hull),
-            ( remove_some_entailed_constraints(Varset, !Hull) ->
-                ConvexHull = eqns(!.Hull)
-            ;
-                ConvexHull = empty_poly
+            ( if remove_some_entailed_constraints(VarSet, !Hull)
+            then ConvexHull = eqns(!.Hull)
+            else ConvexHull = empty_poly
             )
         )
     ).
@@ -408,48 +395,48 @@ transform_polyhedra(Polys, Eqns, !PolyInfo) :-
     constraints::out, polyhedra_info::in, polyhedra_info::out) is det.
 
 transform_polyhedron(Poly, Polys0, Polys, !PolyInfo) :-
-    some [!Varset] (
-        !.PolyInfo = polyhedra_info(VarMaps, Sigmas, !:Varset),
-        varset.new_var(Sigma, !Varset),
+    some [!VarSet] (
+        !.PolyInfo = polyhedra_info(VarMaps, Sigmas, !:VarSet),
+        varset.new_var(Sigma, !VarSet),
         list.map_foldl2(transform_constraint(Sigma), Poly, NewEqns,
-            map.init, VarMap, !Varset),
+            map.init, VarMap, !VarSet),
         Polys = NewEqns ++ Polys0,
         !:PolyInfo = polyhedra_info([VarMap | VarMaps], [Sigma | Sigmas],
-            !.Varset)
+            !.VarSet)
     ).
 
     % transform_constraint: takes a constraint (with original variables) and
     % the sigma variable to add, and returns the constraint where the original
     % variables are substituted for new ones and where the sigma variable is
-    % included.  The map of old to new variables is updated if necessary.
+    % included. The map of old to new variables is updated if necessary.
     %
 :- pred transform_constraint(lp_var::in, constraint::in, constraint::out,
     var_map::in, var_map::out, lp_varset::in, lp_varset::out) is det.
 
-transform_constraint(Sigma, !Constraint, !VarMap, !Varset) :-
+transform_constraint(Sigma, !Constraint, !VarMap, !VarSet) :-
     some [!Terms] (
         deconstruct_constraint(!.Constraint, !:Terms, Op, Const),
-        list.map_foldl2(change_var, !Terms, !VarMap, !Varset),
+        list.map_foldl2(change_var, !Terms, !VarMap, !VarSet),
         list.cons(Sigma - (-Const), !Terms),
         !:Constraint = construct_constraint(!.Terms, Op, zero)
     ).
 
     % change_var: takes a Var-Num pair with an old variable and returns one
-    % with a new variable which the old variable maps to.  Updates the map of
+    % with a new variable which the old variable maps to. Updates the map of
     % old to new variables if necessary.
     %
 :- pred change_var(lp_term::in, lp_term::out, var_map::in, var_map::out,
     lp_varset::in, lp_varset::out) is det.
 
-change_var(!Term, !VarMap, !Varset) :-
+change_var(!Term, !VarMap, !VarSet) :-
     some [!Var] (
         !.Term = !:Var - Coefficient,
 
         % Have we already mapped this original variable to a new one?
-        ( map.search(!.VarMap, !.Var, !:Var) ->
+        ( if map.search(!.VarMap, !.Var, !:Var) then
             true
-        ;
-            varset.new_var(NewVar, !Varset),
+        else
+            varset.new_var(NewVar, !VarSet),
             map.det_insert(!.Var, NewVar, !VarMap),
             !:Var = NewVar
         ),
@@ -504,63 +491,62 @@ make_last_terms(OriginalVar, VarMap, !Terms) :-
 
 %-----------------------------------------------------------------------------%
 %
-% Approximation of polyhedron by a bounding box
+% Approximation of a polyhedron by a bounding box.
 %
 
-polyhedron.bounding_box(empty_poly, _) = empty_poly.
-polyhedron.bounding_box(eqns(Constraints), Varset) =
-    eqns(lp_rational.bounding_box(Varset, Constraints)).
+bounding_box(empty_poly, _) = empty_poly.
+bounding_box(eqns(Constraints), VarSet) =
+    eqns(lp_rational.bounding_box(VarSet, Constraints)).
 
 %-----------------------------------------------------------------------------%
 %
-% Widening
+% Widening.
 %
 
-polyhedron.widen(empty_poly, empty_poly, _) = empty_poly.
-polyhedron.widen(eqns(_), empty_poly, _) =
-    unexpected($module, $pred, "empty polyhedron").
-polyhedron.widen(empty_poly, eqns(_), _) =
-    unexpected($module, $pred, "empty polyhedron").
-polyhedron.widen(eqns(Poly1), eqns(Poly2), Varset) = eqns(WidenedEqns) :-
-    WidenedEqns = list.filter(entailed(Varset, Poly2), Poly1).
+widen(empty_poly, empty_poly, _) = empty_poly.
+widen(eqns(_), empty_poly, _) =
+    unexpected($pred, "empty polyhedron").
+widen(empty_poly, eqns(_), _) =
+    unexpected($pred, "empty polyhedron").
+widen(eqns(Poly1), eqns(Poly2), VarSet) = eqns(WidenedEqns) :-
+    WidenedEqns = list.filter(entailed(VarSet, Poly2), Poly1).
 
 %-----------------------------------------------------------------------------%
 %
-% Projection
+% Projection.
 %
 
-polyhedron.project_all(Varset, Locals, Polyhedra) =
-    list.map((func(Poly0) = Poly :-
-        (
-            Poly0 = eqns(Constraints0),
-            lp_rational.project(Locals, Varset, Constraints0,
-                ProjectionResult),
+project_all(VarSet, Locals, Polyhedra) =
+    list.map(
+        ( func(Poly0) = Poly :-
             (
-                ProjectionResult = pr_res_aborted,
-                unexpected($module, $pred, "abort from project")
+                Poly0 = eqns(Constraints0),
+                project_constraints(VarSet, Locals, Constraints0,
+                    ProjectionResult),
+                (
+                    ProjectionResult = pr_res_aborted,
+                    unexpected($pred, "abort from project")
+                ;
+                    ProjectionResult = pr_res_inconsistent,
+                    Poly = empty_poly
+                ;
+                    ProjectionResult = pr_res_ok(Constraints1),
+                    restore_equalities(Constraints1, Constraints),
+                    Poly = eqns(Constraints)
+                )
             ;
-                ProjectionResult = pr_res_inconsistent,
-                Poly = empty_poly
-            ;
-                ProjectionResult = pr_res_ok(Constraints1),
-                restore_equalities(Constraints1, Constraints),
-                Poly = eqns(Constraints)
+                Poly0 = empty_poly,
+                Poly  = empty_poly
             )
-        ;
-            Poly0 = empty_poly,
-            Poly  = empty_poly
-        )
-    ), Polyhedra).
+        ), Polyhedra).
 
-polyhedron.project(Vars, Varset, Polyhedron0) = Polyhedron :-
-    polyhedron.project(Vars, Varset, Polyhedron0, Polyhedron).
-
-polyhedron.project(_, _, empty_poly, empty_poly).
-polyhedron.project(Vars, Varset, eqns(Constraints0), Result) :-
-    lp_rational.project(Vars, Varset, Constraints0, ProjectionResult),
+project_polyhedron(_, _, empty_poly, empty_poly).
+project_polyhedron(VarSet, Vars, eqns(Constraints0), Result) :-
+    lp_rational.project_constraints(VarSet, Vars,
+        Constraints0, ProjectionResult),
     (
         ProjectionResult = pr_res_aborted,
-        unexpected($module, $pred, "abort from project")
+        unexpected($pred, "abort from project")
     ;
         ProjectionResult = pr_res_inconsistent,
         Result = empty_poly
@@ -569,41 +555,50 @@ polyhedron.project(Vars, Varset, eqns(Constraints0), Result) :-
         restore_equalities(Constraints1, Constraints),
         Result = eqns(Constraints)
     ).
+
 %-----------------------------------------------------------------------------%
 %
-% Variable substitution
+% Variable substitution.
 %
 
-polyhedron.substitute_vars(OldVars, NewVars, Polyhedron0) = Polyhedron :-
+substitute_vars(OldVars, NewVars, Polyhedron0) = Polyhedron :-
     Constraints0 = polyhedron.non_false_constraints(Polyhedron0),
     Constraints = lp_rational.substitute_vars(OldVars, NewVars, Constraints0),
     Polyhedron = polyhedron.from_constraints(Constraints).
 
-polyhedron.substitute_vars(SubstMap, Polyhedron0) = Polyhedron :-
+substitute_vars(SubstMap, Polyhedron0) = Polyhedron :-
     Constraints0 = polyhedron.non_false_constraints(Polyhedron0),
     Constraints = lp_rational.substitute_vars(SubstMap, Constraints0),
     Polyhedron = polyhedron.from_constraints(Constraints).
 
 %-----------------------------------------------------------------------------%
 %
-% Zeroing out variables
+% Zeroing out variables.
 %
 
-polyhedron.zero_vars(_, empty_poly) = empty_poly.
-polyhedron.zero_vars(Vars, eqns(Constraints0)) = eqns(Constraints) :-
+zero_vars(_, empty_poly) = empty_poly.
+zero_vars(Vars, eqns(Constraints0)) = eqns(Constraints) :-
     Constraints = lp_rational.set_vars_to_zero(Vars, Constraints0).
 
 %-----------------------------------------------------------------------------%
 %
-% Printing
+% Printing.
 %
 
-polyhedron.write_polyhedron(empty_poly, _, !IO) :-
-    io.write_string("\tEmpty\n", !IO).
-polyhedron.write_polyhedron(eqns([]),   _, !IO) :-
-    io.write_string("\tUniverse\n", !IO).
-polyhedron.write_polyhedron(eqns(Constraints @ [_|_]), Varset, !IO) :-
-    lp_rational.write_constraints(Constraints, Varset, !IO).
+write_polyhedron(Stream, VarSet, Polyhedron, !IO) :-
+    (
+        Polyhedron = empty_poly,
+        io.write_string(Stream, "\tEmpty\n", !IO)
+    ;
+        Polyhedron = eqns(Constraints),
+        (
+            Constraints = [],
+            io.write_string(Stream, "\tUniverse\n", !IO)
+        ;
+            Constraints = [_ | _],
+            lp_rational.write_constraints(Stream, VarSet, Constraints, !IO)
+        )
+    ).
 
 %-----------------------------------------------------------------------------%
 :- end_module libs.polyhedron.
