@@ -6,11 +6,13 @@
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
 %
+% Main authors: trd, dgj.
+%
 % This module defines predicate for interfacing with foreign languages.
 % that are necessary for the frontend of the compiler to construct
-% the list of items.  The predicates in this module should not depend
-% on the HLDS in any way.  The predicates for interfacing with foreign
-% languages that do depend on the HLDS are defined in foreign.m.
+% the parse tree. The predicates in this module should not depend on the HLDS
+% in any way. The predicates for interfacing with foreign languages that
+% *do* depend on the HLDS are defined in foreign.m.
 %
 % This module also contains the parts of the name mangler that are used
 % by the frontend of the compiler.
@@ -19,9 +21,6 @@
 % module may also require changes to profiler/demangle.m, util/mdemangle.c and
 % compiler/name_mangle.m.
 %
-% Main authors: trd, dgj.
-% This code was originally part of the foreign module and was moved here.
-%
 %-----------------------------------------------------------------------------%
 
 :- module parse_tree.prog_foreign.
@@ -29,74 +28,71 @@
 
 :- import_module libs.
 :- import_module libs.globals.
+:- import_module parse_tree.file_names.
 :- import_module parse_tree.prog_data.
-:- import_module mdbcomp.prim_data.
+:- import_module parse_tree.prog_data_foreign.
+:- import_module mdbcomp.
+:- import_module mdbcomp.sym_name.
 
 :- import_module bool.
 :- import_module list.
-:- import_module term.
+:- import_module map.
+:- import_module pair.
 
 %-----------------------------------------------------------------------------%
 
-:- type foreign_decl_info == list(foreign_decl_code).
-                                % in reverse order
-:- type foreign_body_info == list(foreign_body_code).
-                                % in reverse order
-
 :- type foreign_decl_code
     --->    foreign_decl_code(
                 fdecl_lang          :: foreign_language,
                 fdecl_is_local      :: foreign_decl_is_local,
-                fdecl_code          :: string,
+                fdecl_code          :: foreign_literal_or_include,
                 fdecl_context       :: prog_context
             ).
 
 :- type foreign_body_code
     --->    foreign_body_code(
                 fbody_lang          :: foreign_language,
-                fbody_code          :: string,
+                fbody_code          :: foreign_literal_or_include,
                 fbody_context       :: prog_context
             ).
 
-:- type foreign_export_defns == list(foreign_export).
+:- type foreign_export_defns == list(foreign_export_defn).
+
+:- type foreign_export_defn
+    --->    foreign_export_defn(
+                % The code for `pragma foreign_export' is generated directly
+                % as strings by export.m.
+                string
+            ).
+
 :- type foreign_export_decls
     --->    foreign_export_decls(
-                fexp_decls_info     :: foreign_decl_info,
+                fexp_decls_codes    :: list(foreign_decl_code),
                 fexp_decls_list     :: list(foreign_export_decl)
             ).
 
 :- type foreign_export_decl
     --->    foreign_export_decl(
+                % Language of the export.
                 fexp_decl_lang      :: foreign_language,
-                                    % Language of the export.
 
+                % Return type.
                 fexp_decl_ret_type  :: string,
-                                    % Return type.
 
+                % Function name.
                 fexp_decl_func_name :: string,
-                                    % Function name.
 
+                % Argument declarations.
                 fexp_decl_arg_decls :: string
-                                    % Argument declarations.
             ).
 
-    % Some code from a `pragma foreign_code' declaration that is not
-    % associated with a given procedure.
-    %
-:- type user_foreign_code
-    --->    user_foreign_code(
-                foreign_language,   % language of this code
-                string,             % code
-                term.context        % source code location
-            ).
-
-    % The code for `pragma foreign_export' is generated directly as strings
-    % by export.m.
-    %
-:- type foreign_export  ==  string.
-
 %-----------------------------------------------------------------------------%
 
+:- pred foreign_decl_code_is_for_lang(foreign_language::in,
+    foreign_decl_code::in) is semidet.
+:- pred foreign_body_code_is_for_lang(foreign_language::in,
+    foreign_body_code::in) is semidet.
+
     % foreign_import_module_name(ForeignImport)
     %
     % returns the module name which represents the ForeignImport.
@@ -106,48 +102,47 @@
     % would return the module_name
     %   unqualified("module__csharp_code")
     %
-:- func foreign_import_module_name(foreign_import_module_info) = module_name.
+:- func fim_spec_module_name(fim_spec) = module_name.
 
     % foreign_import_module_name_from_module(ForeignImport, CurrentModule)
     %
     % returns the module name needed to refer to ForeignImport from the
     % CurrentModule.
     %
-:- func foreign_import_module_name_from_module(foreign_import_module_info,
-    module_name) = module_name.
+:- func fim_spec_module_name_from_module(fim_spec, module_name) = module_name.
 
     % Sub-type of foreign_language for languages for which
     % we generate external files for foreign code.
     %
-:- inst lang_gen_ext_file
+:- inst lang_gen_ext_file for globals.foreign_language/0
     --->    lang_c
     ;       lang_csharp.
 
     % The module name used for this foreign language.
-    % Not all foreign languages generate external modules
+    % Not all foreign languages generate external modules,
     % so this function only succeeds for those that do.
     %
 :- func foreign_language_module_name(module_name, foreign_language) =
     module_name.
-:- mode foreign_language_module_name(in, in) = out is semidet.
 :- mode foreign_language_module_name(in, in(lang_gen_ext_file)) = out is det.
+:- mode foreign_language_module_name(in, in) = out is semidet.
 
     % The file extension used for this foreign language (including the dot).
     % Not all foreign languages generate external files,
     % so this function only succeeds for those that do.
     %
-:- func foreign_language_file_extension(foreign_language) = string.
+    % XXX Actually, all the foreign languages we handle *now* *do*
+    % generate external files. (The exception used to be Erlang.)
+    %
+:- func foreign_language_file_extension(foreign_language) = other_ext.
 :- mode foreign_language_file_extension(in) = out is semidet.
 :- mode foreign_language_file_extension(in(lang_gen_ext_file)) = out is det.
 
     % It is possible that more than one foreign language could be used to
-    % implement a particular piece of code.
-    % Therefore, foreign languages have an order of preference, from most
-    % preferred to least perferred.
-    % prefer_foreign_language(Globals, Target, Lang1, Lang2) returns the
-    % yes if Lang2 is preferred over Lang1.
-    %
-    % Otherwise it will return no.
+    % implement a particular piece of code. Therefore, foreign languages
+    % have an order of preference, from most preferred to least perferred.
+    % prefer_foreign_language(Globals, Target, Lang1, Lang2) returns
+    % `yes' if Lang2 is preferred over Lang1; otherwise, it will return no.
     %
 :- func prefer_foreign_language(globals, compilation_target,
     foreign_language, foreign_language) = bool.
@@ -156,12 +151,13 @@
     %
 :- func all_foreign_languages = list(foreign_language).
 
-:- func foreign_type_language(foreign_language_type) = foreign_language.
+:- func foreign_type_language(generic_language_foreign_type)
+    = foreign_language.
 
 %-----------------------------------------------------------------------------%
 %
 % The following are the parts of the name mangler that are needed by
-% the compiler frontend so that it can write out makefile fragments.
+% the compiler frontend so that it can write out Makefile fragments.
 
     % Returns the name of the initialization function for a given module.
     %
@@ -188,51 +184,75 @@
 
 :- func convert_to_valid_c_identifier(string) = string.
 
+%-----------------------------------------------------------------------------%
+
+    % The exported predicate with the given sym_name and arity is represented
+    % in the target language as the string in the second argument.
+    % The pred_or_func is implicitly pf_predicate.
+    %
+    % Used in the implementation of mutable, initialise and finalise
+    % declarations.
+    %
+:- type pred_target_name == pair(sym_name_arity, string).
+
+    % Maps a mutable, initialise or finalise declaration, whose sequence number
+    % (which must be present and valid) is the integer key, to the
+    % the exported predicate or predicates that the compiler generates
+    % for that declaration.
+    %
+    % We use the sequence number as the key in a map so that we can invoke
+    % the exported initialization predicates in program order, as the
+    % language manual expects us to do.
+    %
+:- type pred_target_names
+    --->    pred_target_names(
+                map(int, list(pred_target_name))
+            ).
+
+    % Add a new initialize or finalize predicate.
+    %
+:- pred new_user_init_pred(module_name::in, int::in, sym_name::in, arity::in,
+    string::out, pred_target_names::in, pred_target_names::out) is det.
+:- pred new_user_final_pred(module_name::in, int::in, sym_name::in, arity::in,
+    string::out, pred_target_names::in, pred_target_names::out) is det.
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
-:- import_module parse_tree.file_names.
-
 :- import_module char.
-:- import_module int.
 :- import_module solutions.
 :- import_module string.
 
 %-----------------------------------------------------------------------------%
 
-foreign_import_module_name(ImportModule) = ModuleName :-
-    ImportModule = foreign_import_module_info(Lang, ForeignImportModule, _),
+foreign_decl_code_is_for_lang(Lang, DeclCode) :-
+    Lang = DeclCode ^ fdecl_lang.
+
+foreign_body_code_is_for_lang(Lang, BodyCode) :-
+    Lang = BodyCode ^ fbody_lang.
+
+fim_spec_module_name(FIMSpec) = ModuleName :-
+    FIMSpec = fim_spec(Lang, ForeignImportModule),
     (
         Lang = lang_c,
         ModuleName = ForeignImportModule
-    ;
-        Lang = lang_il,
-        ModuleName = ForeignImportModule
     ;
         Lang = lang_java,
         ModuleName = ForeignImportModule
-    ;
-        Lang = lang_erlang,
-        ModuleName = ForeignImportModule
     ;
         Lang = lang_csharp,
         ModuleName = foreign_language_module_name(ForeignImportModule, Lang)
     ).
 
-foreign_import_module_name_from_module(ModuleForeignImported, CurrentModule) =
+fim_spec_module_name_from_module(ModuleFIMSpec, CurrentModule) =
         ImportedForeignCodeModuleName :-
-    ModuleForeignImported = foreign_import_module_info(Lang, _, _),
-    ImportedForeignCodeModuleName1 = ModuleForeignImported ^
-        foreign_import_module_name,
+    ModuleFIMSpec = fim_spec(Lang, _),
+    ImportedForeignCodeModuleName1 = fim_spec_module_name(ModuleFIMSpec),
     (
         Lang = lang_c,
         ImportedForeignCodeModuleName = ImportedForeignCodeModuleName1
-    ;
-        Lang = lang_il,
-        ImportedForeignCodeModuleName = handle_std_library(CurrentModule,
-            ImportedForeignCodeModuleName1)
     ;
         Lang = lang_csharp,
         ImportedForeignCodeModuleName = handle_std_library(CurrentModule,
@@ -241,25 +261,22 @@ foreign_import_module_name_from_module(ModuleForeignImported, CurrentModule) =
         Lang = lang_java,
         ImportedForeignCodeModuleName = handle_std_library(CurrentModule,
             ImportedForeignCodeModuleName1)
-    ;
-        Lang = lang_erlang,
-        ImportedForeignCodeModuleName = handle_std_library(CurrentModule,
-            ImportedForeignCodeModuleName1)
     ).
 
     % On the il backend, we need to refer to the module "mercury" when
     % referencing a std library module when we are not actually building
     % the std library.
+    % XXX Obsolete comment.
     %
 :- func handle_std_library(module_name, module_name) = module_name.
 
 handle_std_library(CurrentModule, ModuleName0) = ModuleName :-
-    (
+    ( if
         mercury_std_library_module_name(ModuleName0),
-        \+ mercury_std_library_module_name(CurrentModule)
-    ->
+        not mercury_std_library_module_name(CurrentModule)
+    then
         ModuleName = unqualified("mercury")
-    ;
+    else
         ModuleName = ModuleName0
     ).
 
@@ -280,64 +297,37 @@ foreign_language_module_name(ModuleName, Lang) = FullyQualifiedModuleName :-
 
 %-----------------------------------------------------------------------------%
 
-foreign_language_file_extension(lang_c) = ".c".
-foreign_language_file_extension(lang_csharp) = ".cs".
-foreign_language_file_extension(lang_java) = ".java".
-foreign_language_file_extension(lang_il) = _ :-
-    fail.
+:- pragma no_determinism_warning(func(foreign_language_file_extension/1)).
+
+foreign_language_file_extension(lang_c) = other_ext(".c").
+foreign_language_file_extension(lang_csharp) = other_ext(".cs").
+foreign_language_file_extension(lang_java) = other_ext(".java").
 
 %-----------------------------------------------------------------------------%
 
+prefer_foreign_language(_Globals, Target, Lang1, Lang2) = Prefer :-
     % Currently we don't use the globals to compare foreign language
     % interfaces, but if we added appropriate options we might want
     % to do this later.
-    %
-prefer_foreign_language(_Globals, target_c, Lang1, Lang2) =
-    % When compiling to C, C is always preferred over any other language.
-    ( Lang2 = lang_c, not Lang1 = lang_c ->
-        yes
+    (
+        Target = target_c,
+        % When compiling to C, C is always preferred over any other language.
+        ( if Lang2 = lang_c, not Lang1 = lang_c then
+            Prefer = yes
+        else
+            Prefer = no
+        )
     ;
-        no
-    ).
-
-prefer_foreign_language(_Globals, target_il, Lang1, Lang2) = Comp :-
-    % When compiling to il, first we prefer il, then csharp.
-    % After that we don't care.
-    PreferredList = [lang_il, lang_csharp],
-
-    FindLangPriority = (func(L) = X :-
-        ( list.nth_member_search(PreferredList, L, X0) ->
-            X = X0
-        ;
-            X = list.length(PreferredList) + 1
-        )),
-    N1 = FindLangPriority(Lang1),
-    N2 = FindLangPriority(Lang2),
-    ( N2 < N1 ->
-        Comp = yes
+        Target = target_csharp,
+        Prefer = no
     ;
-        Comp = no
+        Target = target_java,
+        % Nothing useful to do here, but when we add Java as a
+        % foreign language, we should add it here.
+        % XXX Obsolete comment.
+        Prefer = no
     ).
 
-prefer_foreign_language(_Globals, target_csharp, _Lang1, _Lang2) = no.
-
-prefer_foreign_language(_Globals, target_java, _Lang1, _Lang2) = no.
-    % Nothing useful to do here, but when we add Java as a foreign language,
-    % we should add it here.
-
-prefer_foreign_language(_Globals, target_x86_64, Lang1, Lang2) =
-    % When compiling to x86_64 assembler, C is always preferred over any
-    % other language.
-    ( Lang2 = lang_c, not Lang1 = lang_c ->
-        yes
-    ;
-        no
-    ).
-
-prefer_foreign_language(_Globals, target_erlang, _Lang1, _Lang2) = no.
-    % Nothing useful to do here, but when we add Erlang as a foreign language,
-    % we should add it here.
-
 %-----------------------------------------------------------------------------%
 
 all_foreign_languages = Langs :-
@@ -351,16 +341,12 @@ all_foreign_languages = Langs :-
 valid_foreign_language(lang_c).
 valid_foreign_language(lang_java).
 valid_foreign_language(lang_csharp).
-valid_foreign_language(lang_il).
-valid_foreign_language(lang_erlang).
 
 %-----------------------------------------------------------------------------%
 
-foreign_type_language(il(_)) = lang_il.
 foreign_type_language(c(_)) = lang_c.
 foreign_type_language(java(_)) = lang_java.
 foreign_type_language(csharp(_)) = lang_csharp.
-foreign_type_language(erlang(_)) = lang_erlang.
 
 %-----------------------------------------------------------------------------%
 
@@ -386,7 +372,7 @@ name_mangle_2(AllowLeadingDigit, Name) = MangledName :-
     % require changes to profiler/demangle.m, util/mdemangle.c,
     % compiler/name_mangle.m and library/rtti_implementation.m.
 
-    (
+    ( if
         string.is_all_alnum_or_underscore(Name),
         (
             AllowLeadingDigit = yes
@@ -397,17 +383,17 @@ name_mangle_2(AllowLeadingDigit, Name) = MangledName :-
             string.index(Name, 0, FirstChar),
             not char.is_digit(FirstChar)
         )
-    ->
+    then
         % Any names that start with `f_' are changed so that they start with
         % `f__', so that we can use names starting with `f_' (followed by
         % anything except an underscore) without fear of name collisions.
 
-        ( string.append("f_", Suffix, Name) ->
+        ( if string.append("f_", Suffix, Name) then
             MangledName = "f__" ++ Suffix
-        ;
+        else
             MangledName = Name
         )
-    ;
+    else
         MangledName = convert_to_valid_c_identifier(Name)
     ).
 
@@ -415,9 +401,9 @@ qualify_name(Module0, Name0) = Name :-
     string.append_list([Module0, "__", Name0], Name).
 
 convert_to_valid_c_identifier(String) = Name :-
-    ( name_conversion_table(String, Name0) ->
+    ( if name_conversion_table(String, Name0) then
         Name = Name0
-    ;
+    else
         Name = "f" ++ convert_to_valid_c_identifier_2(String)
     ).
 
@@ -460,7 +446,7 @@ name_conversion_table("[]", "f_nil").
 :- func convert_to_valid_c_identifier_2(string) = string.
 
 convert_to_valid_c_identifier_2(String) = Name :-
-    ( string.first_char(String, Char, Rest) ->
+    ( if string.first_char(String, Char, Rest) then
         % XXX This will cause ABI incompatibilities between compilers which are
         % built in grades that have different character representations.
         char.to_int(Char, Code),
@@ -468,11 +454,63 @@ convert_to_valid_c_identifier_2(String) = Name :-
         string.append("_", CodeString, ThisCharString),
         Name0 = convert_to_valid_c_identifier_2(Rest),
         string.append(ThisCharString, Name0, Name)
-    ;
-        % String is the empty string
+    else
+        % String is the empty string.
         Name = String
     ).
 
+%---------------------------------------------------------------------------%
+
+new_user_init_pred(ModuleName, SeqNum, SymName, Arity,
+        CName, !PredTargetNames) :-
+    % XXX There is some debate as to whether duplicate initialise directives
+    % in the same module should constitute an error. Currently it is not, but
+    % we may wish to revisit this code. The reference manual is therefore
+    % deliberately quiet on the subject.
+    new_pred_target_name(ModuleName, "init", SeqNum, SymName, Arity,
+        CName, !PredTargetNames).
+
+new_user_final_pred(ModuleName, SeqNum, SymName, Arity,
+        CName, !PredTargetNames) :-
+    new_pred_target_name(ModuleName, "final", SeqNum, SymName, Arity,
+        CName, !PredTargetNames).
+
+:- pred new_pred_target_name(module_name::in, string::in, int::in,
+    sym_name::in, arity::in, string::out,
+    pred_target_names::in, pred_target_names::out) is det.
+
+new_pred_target_name(ModuleName0, InitOrFinal, SeqNum, SymName, Arity, CName,
+        !PredTargetNames) :-
+    !.PredTargetNames = pred_target_names(PredTargetNameMap0),
+    ( if mercury_std_library_module_name(ModuleName0) then
+        ModuleName = add_outermost_qualifier("mercury", ModuleName0)
+    else
+        ModuleName = ModuleName0
+    ),
+    ModuleNameStr = prog_foreign.sym_name_mangle(ModuleName),
+    ( if map.search(PredTargetNameMap0, SeqNum, SeqNumPredTargetNames0) then
+        % The only situation in which a sequence number will have
+        % more than one entry is when a solver type's representation
+        % involves more than one mutable. The number of these should be
+        % be limited, which is why the quadratic behavior of this code is ok.
+        % We do nevertheless need to include something, such as Suffix,
+        % to distinguish the target names from each other.
+        list.length(SeqNumPredTargetNames0, Suffix),
+        CName = string.format("%s__user_%s_pred_%d_%d",
+            [s(ModuleNameStr), s(InitOrFinal), i(SeqNum), i(Suffix)]),
+        PredTargetName = sym_name_arity(SymName, Arity) - CName,
+        SeqNumPredTargetNames = SeqNumPredTargetNames0 ++ [PredTargetName],
+        map.det_update(SeqNum, SeqNumPredTargetNames,
+            PredTargetNameMap0, PredTargetNameMap)
+    else
+        CName = string.format("%s__user_%s_pred_%d_%d",
+            [s(ModuleNameStr), s(InitOrFinal), i(SeqNum), i(0)]),
+        PredTargetName = sym_name_arity(SymName, Arity) - CName,
+        map.det_insert(SeqNum, [PredTargetName],
+            PredTargetNameMap0, PredTargetNameMap)
+    ),
+    !:PredTargetNames = pred_target_names(PredTargetNameMap).
+
 %-----------------------------------------------------------------------------%
-:- end_module prog_foreign.
+:- end_module parse_tree.prog_foreign.
 %-----------------------------------------------------------------------------%
