@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
 % Copyright (C) 2000-2012 University of Melbourne.
+% Copyright (C) 2015-2018 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -20,11 +21,15 @@
 :- module ll_backend.prog_rep.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.instmap.
+:- import_module hlds.vartypes.
 :- import_module ll_backend.prog_rep_tables.
+:- import_module mdbcomp.
 :- import_module mdbcomp.program_representation.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
@@ -65,11 +70,11 @@
 
 :- type prog_rep_info
     --->    prog_rep_info(
+                pri_module_info         :: module_info,
                 pri_filename            :: string,
                 pri_vartypes            :: vartypes,
                 pri_var_num_map         :: var_num_map,
                 pri_var_num_rep         :: var_num_rep,
-                pri_module_info         :: module_info,
                 pri_flatten_par_conjs   :: flatten_par_conjs
             ).
 
@@ -85,24 +90,26 @@
 
 :- implementation.
 
+:- import_module backend_libs.
 :- import_module backend_libs.proc_label.
-:- import_module check_hlds.inst_match.
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.inst_test.
+:- import_module check_hlds.mode_test.
 :- import_module hlds.code_model.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.special_pred.
-:- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
 :- import_module mdbcomp.rtti_access.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_util.
 :- import_module parse_tree.set_of_var.
 
 :- import_module int.
 :- import_module maybe.
 :- import_module require.
-:- import_module set.
 :- import_module std_util.
 :- import_module string.
 :- import_module term.
@@ -120,9 +127,9 @@ encode_oisu_type_procs(ModuleInfo, [Pair | Pairs], NumOISUTypes, Bytes) :-
         TypeCtorSymName = qualified(TypeCtorModuleName, TypeCtorName)
     ;
         TypeCtorSymName = unqualified(_),
-        unexpected($module, $pred, "unqualified type_ctor name")
+        unexpected($pred, "unqualified type_ctor name")
     ),
-    ( TypeCtorModuleName = ModuleName ->
+    ( if TypeCtorModuleName = ModuleName then
         encode_len_string(TypeCtorName, TypeCtorNameBytes),
         Preds = oisu_preds(CreatorPreds, MutatorPreds, DestructorPreds),
 
@@ -149,7 +156,7 @@ encode_oisu_type_procs(ModuleInfo, [Pair | Pairs], NumOISUTypes, Bytes) :-
 
         NumOISUTypes = 1 + TailNumOISUTypes,
         Bytes = HeadBytes ++ TailBytes
-    ;
+    else
         NumOISUTypes = TailNumOISUTypes,
         Bytes = TailBytes
     ).
@@ -158,13 +165,13 @@ encode_oisu_type_procs(ModuleInfo, [Pair | Pairs], NumOISUTypes, Bytes) :-
 
 encode_oisu_proc(ModuleInfo, PredId, BytesCord) :-
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
-    pred_info_get_procedures(PredInfo, ProcTable),
+    pred_info_get_proc_table(PredInfo, ProcTable),
     map.to_assoc_list(ProcTable, Procs),
-    ( Procs = [ProcId - _ProcInfo] ->
+    ( if Procs = [ProcId - _ProcInfo] then
         ProcLabel = make_proc_label(ModuleInfo, PredId, ProcId),
         encode_string_proc_label(ProcLabel, BytesCord)
-    ;
-        unexpected($module, $pred, "OISU pred should have exactly one proc")
+    else
+        unexpected($pred, "OISU pred should have exactly one proc")
     ).
 
 :- pred encode_string_proc_label(proc_label::in, cord(int)::out) is det.
@@ -237,7 +244,7 @@ represent_proc_as_bytecodes(HeadVars, Goal, InstMap0, VarTypes, VarNumMap,
     represent_var_table_as_bytecode(IncludeVarNameTable, IncludeVarTypes,
         VarTypes, VarNumMap, VarNumRep, VarNameTableBytes,
         !StringTable, !TypeTable),
-    Info = prog_rep_info(FileName, VarTypes, VarNumMap, VarNumRep, ModuleInfo,
+    Info = prog_rep_info(ModuleInfo, FileName, VarTypes, VarNumMap, VarNumRep,
         expect_no_par_conjs),
     InstmapDelta = goal_info_get_instmap_delta(GoalInfo),
 
@@ -270,11 +277,11 @@ represent_proc_as_bytecodes(HeadVars, Goal, InstMap0, VarTypes, VarNumMap,
 represent_var_table_as_bytecode(IncludeVarNameTable, IncludeVarTypes,
         VarTypes, VarNumMap, VarNumRep, Bytes, !StringTable, !TypeTable) :-
     map.foldl(max_var_num, VarNumMap, 0) = MaxVarNum,
-    ( MaxVarNum =< 127 ->
+    ( if MaxVarNum =< 127 then
         VarNumRep = var_num_1_byte
-    ; MaxVarNum =< 32767 ->
+    else if MaxVarNum =< 32767 then
         VarNumRep = var_num_2_bytes
-    ;
+    else
         VarNumRep = var_num_4_bytes
     ),
     var_flag_byte(VarNumRep, IncludeVarNameTable, IncludeVarTypes, FlagByte),
@@ -308,8 +315,8 @@ represent_var_table_as_bytecode(IncludeVarNameTable, IncludeVarTypes,
         Bytes = [FlagByte] ++ NumVarsBytes ++ VarNameTableEntriesBytes
     ;
         IncludeVarNameTable = do_not_include_var_name_table,
-        expect(unify(IncludeVarTypes, do_not_include_var_types),
-            $module, $pred, "IncludeVarTypes but not IncludeVarNameTable"),
+        expect(unify(IncludeVarTypes, do_not_include_var_types), $pred,
+            "IncludeVarTypes but not IncludeVarNameTable"),
         Bytes = [FlagByte]
     ).
 
@@ -334,9 +341,9 @@ max_var_num(_, VarNum1 - _, VarNum2) = Max :-
 
 encode_var_name_table_entry_1_byte(_ProgVar, VarNum - VarName,
         !NumVars, !VarNameTableBytes, !StringTable) :-
-    ( compiler_introduced_varname(VarName) ->
+    ( if compiler_introduced_varname(VarName) then
         true
-    ;
+    else
         !:NumVars = !.NumVars + 1,
         VarBytes = [VarNum],
         encode_string_as_table_offset(VarName, VarNameBytes, !StringTable),
@@ -349,9 +356,9 @@ encode_var_name_table_entry_1_byte(_ProgVar, VarNum - VarName,
 
 encode_var_name_table_entry_2_byte(_ProgVar, VarNum - VarName,
         !NumVars, !VarNameTableBytes, !StringTable) :-
-    ( compiler_introduced_varname(VarName) ->
+    ( if compiler_introduced_varname(VarName) then
         true
-    ;
+    else
         !:NumVars = !.NumVars + 1,
         encode_short_det(VarNum, VarBytes),
         encode_string_as_table_offset(VarName, VarNameBytes, !StringTable),
@@ -364,9 +371,9 @@ encode_var_name_table_entry_2_byte(_ProgVar, VarNum - VarName,
 
 encode_var_name_table_entry_4_byte(_ProgVar, VarNum - VarName,
         !NumVars, !VarNameTableBytes, !StringTable) :-
-    ( compiler_introduced_varname(VarName) ->
+    ( if compiler_introduced_varname(VarName) then
         true
-    ;
+    else
         !:NumVars = !.NumVars + 1,
         encode_int32_det(VarNum, VarBytes),
         encode_string_as_table_offset(VarName, VarNameBytes, !StringTable),
@@ -444,7 +451,7 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
         ;
             FlattenParConjs = expect_no_par_conjs,
             Goals = Goals0,
-            expect(unify(ConjType, plain_conj), $module, $pred,
+            expect(unify(ConjType, plain_conj), $pred,
                 "non-plain conjunction and declarative debugging")
         ),
         conj_to_conj_rep(Info, Instmap0, Goals, GoalReps),
@@ -462,7 +469,7 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
         GoalExpr = if_then_else(_, Cond, Then, Else),
         Cond = hlds_goal(_, CondGoalInfo),
         InstmapDelta = goal_info_get_instmap_delta(CondGoalInfo),
-        instmap.apply_instmap_delta(Instmap0, InstmapDelta, InstmapAfterCond),
+        apply_instmap_delta(InstmapDelta, Instmap0, InstmapAfterCond),
         goal_to_goal_rep(Info, Instmap0, Cond, CondRep),
         goal_to_goal_rep(Info, InstmapAfterCond, Then, ThenRep),
         goal_to_goal_rep(Info, Instmap0, Else, ElseRep),
@@ -485,9 +492,9 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
         goal_to_goal_rep(Info, Instmap0, SubGoal, SubGoalRep),
         OuterDetism = goal_info_get_determinism(GoalInfo),
         InnerDetism = goal_info_get_determinism(SubGoalInfo),
-        ( InnerDetism = OuterDetism ->
+        ( if InnerDetism = OuterDetism then
             MaybeCut = scope_is_no_cut
-        ;
+        else
             MaybeCut = scope_is_cut
         ),
         GoalExprRep = scope_rep(SubGoalRep, MaybeCut)
@@ -497,16 +504,19 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
         ; GoalExpr = plain_call(_, _, _, _, _, _)
         ; GoalExpr = call_foreign_proc(_, _, _, _, _, _, _)
         ),
+        goal_info_to_atomic_goal_rep_fields(GoalInfo, Instmap0, Info,
+            FileName, LineNo, BoundVars),
+        BoundVarsRep = map(var_to_var_rep(Info), BoundVars),
         (
-            GoalExpr = unify(_, _, _, Uni, _),
+            GoalExpr = unify(_, _, _, Unification, _),
             (
-                Uni = assign(Target, Source),
+                Unification = assign(Target, Source),
                 AtomicGoalRep = unify_assign_rep(
                     var_to_var_rep(Info, Target),
                     var_to_var_rep(Info, Source))
             ;
-                ( Uni = construct(Var, ConsId, Args, ArgModes, _, _, _)
-                ; Uni = deconstruct(Var, ConsId, Args, ArgModes, _, _)
+                ( Unification = construct(Var, ConsId, Args, ArgModes, _, _, _)
+                ; Unification = deconstruct(Var, ConsId, Args, ArgModes, _, _)
                 ),
                 VarRep = var_to_var_rep(Info, Var),
                 ConsIdRep = cons_id_rep(ConsId),
@@ -514,32 +524,34 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
                 filter_input_args(Info, ArgModes, Args, MaybeArgs),
                 MaybeArgsRep = map(map_maybe(var_to_var_rep(Info)), MaybeArgs),
                 (
-                    Uni = construct(_, _, _, _, _, _, _),
-                    ( list.all_true(lhs_final_is_ground(Info), ArgModes) ->
+                    Unification = construct(_, _, _, _, _, _, _),
+                    ( if
+                        list.all_true(lhs_final_is_ground(Info), ArgModes)
+                    then
                         AtomicGoalRep = unify_construct_rep(VarRep, ConsIdRep,
                             ArgsRep)
-                    ;
+                    else
                         AtomicGoalRep = partial_construct_rep(VarRep,
                             ConsIdRep, MaybeArgsRep)
                     )
                 ;
-                    Uni = deconstruct(_, _, _, _, _, _),
-                    ( list.member(Var, BoundVars) ->
+                    Unification = deconstruct(_, _, _, _, _, _),
+                    ( if list.member(Var, BoundVars) then
                         AtomicGoalRep = partial_deconstruct_rep(VarRep,
                             ConsIdRep, MaybeArgsRep)
-                    ;
+                    else
                         AtomicGoalRep = unify_deconstruct_rep(VarRep,
                             ConsIdRep, ArgsRep)
                     )
                 )
             ;
-                Uni = simple_test(Var1, Var2),
+                Unification = simple_test(Var1, Var2),
                 AtomicGoalRep = unify_simple_test_rep(
                     var_to_var_rep(Info, Var1),
                     var_to_var_rep(Info, Var2))
             ;
-                Uni = complicated_unify(_, _, _),
-                unexpected($module, $pred, "complicated_unify")
+                Unification = complicated_unify(_, _, _),
+                unexpected($pred, "complicated_unify")
             )
         ;
             GoalExpr = generic_call(GenericCall, Args, _, _, _),
@@ -557,10 +569,10 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
                 AtomicGoalRep = event_call_rep(EventName, ArgsRep)
             ;
                 GenericCall = cast(_),
-                ( ArgsRep = [InputArgRep, OutputArgRep] ->
+                ( if ArgsRep = [InputArgRep, OutputArgRep] then
                     AtomicGoalRep = cast_rep(OutputArgRep, InputArgRep)
-                ;
-                    unexpected($module, $pred, "cast arity != 2")
+                else
+                    unexpected($pred, "cast/coerce arity != 2")
                 )
             )
         ;
@@ -576,9 +588,6 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
             ;
                 Builtin = inline_builtin,
                 AtomicGoalRep = builtin_call_rep(ModuleName, PredName, ArgsRep)
-            ;
-                Builtin = out_of_line_builtin,
-                unexpected($module, $pred, "out_of_line_builtin")
             )
         ;
             GoalExpr = call_foreign_proc(_, _PredId, _, Args, _, _, _),
@@ -586,15 +595,12 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
                 compose(var_to_var_rep(Info), foreign_arg_var), Args),
             AtomicGoalRep = pragma_foreign_code_rep(ArgVarsRep)
         ),
-        goal_info_to_atomic_goal_rep_fields(GoalInfo, Instmap0, Info,
-            FileName, LineNo, BoundVars),
-        BoundVarsRep = map(var_to_var_rep(Info), BoundVars),
         GoalExprRep = atomic_goal_rep(FileName, LineNo, BoundVarsRep,
             AtomicGoalRep)
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "unexpected shorthand")
+        unexpected($pred, "unexpected shorthand")
     ).
 
 :- pred conj_to_conj_rep(prog_rep_info::in, instmap::in, list(hlds_goal)::in,
@@ -603,9 +609,9 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
 conj_to_conj_rep(_, _, [], []).
 conj_to_conj_rep(Info, Instmap0, [Conj | Conjs], [ConjRep | ConjReps]) :-
     goal_to_goal_rep(Info, Instmap0, Conj, ConjRep),
-    GoalInfo = Conj ^ hlds_goal_info,
+    GoalInfo = Conj ^ hg_info,
     InstmapDelta = goal_info_get_instmap_delta(GoalInfo),
-    instmap.apply_instmap_delta(Instmap0, InstmapDelta, Instmap1),
+    apply_instmap_delta(InstmapDelta, Instmap0, Instmap1),
     conj_to_conj_rep(Info, Instmap1, Conjs, ConjReps).
 
 :- pred case_to_case_rep(prog_rep_info::in, instmap::in,
@@ -796,32 +802,34 @@ encode_case_rep(Info, Case, Bytes, !StringTable) :-
     Bytes = MainConsIdBytes ++ encode_length_func(OtherConsIds) ++
         list.condense(OtherConsIdsByteLists) ++ GoalBytes.
 
-:- pred lhs_final_is_ground(prog_rep_info::in, uni_mode::in) is semidet.
+:- pred lhs_final_is_ground(prog_rep_info::in, unify_mode::in) is semidet.
 
-lhs_final_is_ground(Info, (_ - _) -> (LHSFinalInst - _)) :-
+lhs_final_is_ground(Info, UnifyMode) :-
+    UnifyMode = unify_modes_li_lf_ri_rf(_, LHSFinalInst, _, _),
     inst_is_ground(Info ^ pri_module_info, LHSFinalInst).
 
-:- pred rhs_is_input(prog_rep_info::in, uni_mode::in) is semidet.
+:- pred rhs_is_input(prog_rep_info::in, unify_mode::in) is semidet.
 
-rhs_is_input(Info, (_ - RHSInitialInst) -> (_ - RHSFinalInst)) :-
-    mode_is_input(Info ^ pri_module_info, RHSInitialInst -> RHSFinalInst).
+rhs_is_input(Info, UnifyMode) :-
+    UnifyMode = unify_modes_li_lf_ri_rf(_, _, RHSInitInst, _),
+    init_inst_is_input(Info ^ pri_module_info, RHSInitInst).
 
-:- pred filter_input_args(prog_rep_info::in, list(uni_mode)::in,
+:- pred filter_input_args(prog_rep_info::in, list(unify_mode)::in,
     list(prog_var)::in, list(maybe(prog_var))::out) is det.
 
 filter_input_args(_, [], [], []).
+filter_input_args(_, [], [_ | _], _) :-
+    unexpected($pred, "mismatched lists").
+filter_input_args(_, [_ | _], [], _) :-
+    unexpected($pred, "mismatched lists").
 filter_input_args(Info, [Mode | Modes], [Var | Vars],
         [MaybeVar | MaybeVars]) :-
-    ( rhs_is_input(Info, Mode) ->
+    ( if rhs_is_input(Info, Mode) then
         MaybeVar = yes(Var)
-    ;
+    else
         MaybeVar = no
     ),
     filter_input_args(Info, Modes, Vars, MaybeVars).
-filter_input_args(_, [], [_ | _], _) :-
-    unexpected($module, $pred, "mismatched lists").
-filter_input_args(_, [_ | _], [], _) :-
-    unexpected($module, $pred, "mismatched lists").
 
 %-----------------------------------------------------------------------------%
 
@@ -832,17 +840,18 @@ goal_info_to_atomic_goal_rep_fields(GoalInfo, Instmap0, Info, FileName, LineNo,
         BoundVars) :-
     Context = goal_info_get_context(GoalInfo),
     term.context_file(Context, FileName0),
-    ( FileName0 = Info ^ pri_filename ->
+    ( if FileName0 = Info ^ pri_filename then
         FileName = ""
-    ;
+    else
         FileName = FileName0
     ),
     term.context_line(Context, LineNo),
     InstmapDelta = goal_info_get_instmap_delta(GoalInfo),
-    instmap.apply_instmap_delta(Instmap0, InstmapDelta, Instmap),
-    instmap_changed_vars(Instmap0, Instmap, Info ^ pri_vartypes,
-        Info ^ pri_module_info, ChangedVars),
-    set_of_var.to_sorted_list(ChangedVars, BoundVars).
+    instmap_delta_changed_vars(InstmapDelta, ChangedVarsSet),
+    set_of_var.to_sorted_list(ChangedVarsSet, ChangedVars),
+    ModuleInfo = Info ^ pri_module_info,
+    list.negated_filter(var_is_ground_in_instmap(ModuleInfo, Instmap0),
+        ChangedVars, BoundVars).
 
 :- pred encode_cons_id_and_arity_rep(cons_id_arity_rep::in, list(int)::out,
     string_table_info::in, string_table_info::out) is det.
@@ -853,22 +862,18 @@ encode_cons_id_and_arity_rep(ConsIdArity, ConsIdBytes, !StringTable) :-
     encode_short_det(Arity, ArityBytes),
     ConsIdBytes = FunctorBytes ++ ArityBytes.
 
-:- pred encode_cons_id(cons_id::in, list(int)::out,
-    string_table_info::in, string_table_info::out) is det.
-
-encode_cons_id(SymName, Bytes, !StringTable) :-
-    encode_string_as_table_offset(cons_id_rep(SymName), Bytes, !StringTable).
-
 :- func cons_id_rep(cons_id) = string.
 
 cons_id_rep(cons(SymName, _, _)) =
     prog_rep.sym_base_name_to_string(SymName).
 cons_id_rep(tuple_cons(_)) = "{}".
-cons_id_rep(int_const(Int)) = string.int_to_string(Int).
+cons_id_rep(some_int_const(IntConst)) = Str :-
+    int_const_to_string_and_suffix(IntConst, Str, _Suffix).
 cons_id_rep(float_const(Float)) = string.float_to_string(Float).
 cons_id_rep(char_const(Char)) = string.char_to_string(Char).
 cons_id_rep(string_const(String)) = """" ++ String ++ """".
-cons_id_rep(impl_defined_const(Name)) = "$" ++ Name.
+cons_id_rep(impl_defined_const(IDCKind)) =
+    impl_defined_const_kind_to_str(IDCKind).
 cons_id_rep(closure_cons(_, _)) = "$closure_cons".
 cons_id_rep(type_ctor_info_const(_, _, _)) = "$type_ctor_info_const".
 cons_id_rep(base_typeclass_info_const(_, _, _, _)) =
@@ -880,7 +885,7 @@ cons_id_rep(type_info_const(_)) = "$type_info_const".
 cons_id_rep(typeclass_info_const(_)) = "$typeclass_info_const".
 cons_id_rep(ground_term_const(_, _)) = "$ground_term_const".
 cons_id_rep(tabling_info_const(_)) = "$tabling_info_const".
-cons_id_rep(table_io_decl(_)) = "$table_io_decl".
+cons_id_rep(table_io_entry_desc(_)) = "$table_io_entry_desc".
 cons_id_rep(deep_profiling_proc_layout(_)) = "$deep_profiling_proc_layout".
 
 :- func sym_base_name_to_string(sym_name) = string.
@@ -977,9 +982,9 @@ encode_head_var_func(Info, InitialInstmap, InstmapDelta, Var) = Bytes :-
     encode_var_rep_func(Info, var_to_var_rep(Info, Var)) = VarBytes,
     ModuleInfo = Info ^ pri_module_info,
     instmap_lookup_var(InitialInstmap, Var, InitialInst),
-    ( instmap_delta_search_var(InstmapDelta, Var, FinalInstPrime) ->
+    ( if instmap_delta_search_var(InstmapDelta, Var, FinalInstPrime) then
         FinalInst = FinalInstPrime
-    ;
+    else
         % If the variable is not in the instmap delta, then its instantiation
         % cannot possibly change.
         FinalInst = InitialInst
@@ -990,17 +995,17 @@ encode_head_var_func(Info, InitialInstmap, InstmapDelta, Var) = Bytes :-
 :- func inst_to_byte(module_info, mer_inst) = int.
 
 inst_to_byte(ModuleInfo, MerInst) = Byte :-
-    (
+    ( if
         ( MerInst = free
         ; MerInst = free(_)
         )
-    ->
+    then
         InstRep = ir_free_rep
-    ;
+    else if
         inst_is_ground(ModuleInfo, MerInst)
-    ->
+    then
         InstRep = ir_ground_rep
-    ;
+    else
         InstRep = ir_other_rep
     ),
     inst_representation(InstRep, Byte).
