@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2003-2012 The University of Melbourne.
+% Copyright (C) 2015 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -25,14 +26,14 @@
 % calculate the size when the term is being constructed, which requires
 % traversing it anyway. In term profiling grades, we reserve an extra word at
 % the start of every memory cell (with a few exceptions explained below) that
-% stores the size of the whole term the memory cell is the top of.  The size
+% stores the size of the whole term the memory cell is the top of. The size
 % is defined as the memory words in the term or the number of memory cells in
 % the term, depending on the grade.
 %
 % The main job of this module is to annotate every construction unification
 % with the information that the code generator needs to fill in the term size
 % slot. In order to do this, it must be able to find out the sizes of the
-% arguments, which in turn requires knowing the arguments' types.  (Without
+% arguments, which in turn requires knowing the arguments' types. (Without
 % type information, we cannot distinguish a pointer from an integer.) Most of
 % the code in this module is concerned with adding code to the procedure being
 % transformed to find or construct the typeinfos we need in order to find out
@@ -59,7 +60,7 @@
 % need to.
 %
 % We do not associate sizes with the memory cells of a small set of types,
-% including type_infos, type_class_infos, closures and boxed floats.  The two
+% including type_infos, type_class_infos, closures and boxed floats. The two
 % reasons for this are that (1) the sizes of values of these types practically
 % never control the complexity of a procedure, so there is no need for their
 % sizes, and (2) this allows us to create e.g. static type_info structures
@@ -83,6 +84,7 @@
 :- module transform_hlds.size_prof.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 
@@ -104,19 +106,28 @@
 
 :- implementation.
 
-:- import_module check_hlds.inst_match.
-:- import_module check_hlds.mode_util.
-:- import_module check_hlds.polymorphism.
+:- import_module check_hlds.
+:- import_module check_hlds.inst_test.
+:- import_module check_hlds.polymorphism_type_info.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module check_hlds.simplify.
+:- import_module check_hlds.simplify.simplify_proc.
+:- import_module check_hlds.simplify.simplify_tasks.
 :- import_module check_hlds.type_util.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.instmap.
+:- import_module hlds.make_goal.
 :- import_module hlds.passes_aux.
 :- import_module hlds.pred_table.
 :- import_module hlds.quantification.
+:- import_module hlds.vartypes.
+:- import_module mdbcomp.
+:- import_module mdbcomp.builtin_modules.
 :- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_type.
@@ -194,7 +205,7 @@
 :- type rev_type_ctor_map   == map(prog_var, type_ctor).
 :- type known_size_map      == map(prog_var, int).
 
-:- type size_prof.info
+:- type size_prof_info
     --->    size_prof_info(
                 spi_type_ctor_map           :: type_ctor_map,
                 spi_type_info_map           :: type_info_map,
@@ -211,14 +222,10 @@
 
 size_prof_process_proc_msg(Transform, PredProcId, !ProcInfo, !ModuleInfo) :-
     trace [io(!IO)] (
-        write_proc_progress_message("% Adding typeinfos in ",
-            PredProcId, !.ModuleInfo, !IO)
+        write_proc_progress_message(!.ModuleInfo,
+            "Size profiling", PredProcId, !IO)
     ),
-    size_prof_process_proc(Transform, PredProcId, !ProcInfo, !ModuleInfo),
-    trace [io(!IO)] (
-        write_proc_progress_message("done.\n",
-            PredProcId, !.ModuleInfo, !IO)
-    ).
+    size_prof_process_proc(Transform, PredProcId, !ProcInfo, !ModuleInfo).
 
 :- pred size_prof_process_proc(construct_transform::in, pred_proc_id::in,
     proc_info::in, proc_info::out,
@@ -226,14 +233,14 @@ size_prof_process_proc_msg(Transform, PredProcId, !ProcInfo, !ModuleInfo) :-
 
 size_prof_process_proc(Transform, proc(PredId, ProcId), !ProcInfo,
         !ModuleInfo) :-
-    Simplifications = list_to_simplifications([]),
-    simplify_proc_return_msgs(Simplifications, PredId, ProcId,
-        !ModuleInfo, !ProcInfo, _Msgs),
+    module_info_get_globals(!.ModuleInfo, Globals),
+    SimplifyTasks = list_to_simplify_tasks(Globals, []),
+    simplify_proc(SimplifyTasks, PredId, ProcId, !ModuleInfo, !ProcInfo),
 
     proc_info_get_goal(!.ProcInfo, Goal0),
     proc_info_get_varset(!.ProcInfo, VarSet0),
     proc_info_get_vartypes(!.ProcInfo, VarTypes0),
-    proc_info_get_initial_instmap(!.ProcInfo, !.ModuleInfo, InstMap0),
+    proc_info_get_initial_instmap(!.ModuleInfo, !.ProcInfo, InstMap0),
     proc_info_get_rtti_varmaps(!.ProcInfo, RttiVarMaps0),
     % The with_types are needed to avoid a combinatorial explosion
     % of ambiguity in the type checker.
@@ -268,7 +275,7 @@ size_prof_process_proc(Transform, proc(PredId, ProcId), !ProcInfo,
     proc_info_set_rtti_varmaps(RttiVarMaps, !ProcInfo).
 
 :- pred size_prof_process_goal(hlds_goal::in, hlds_goal::out,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 size_prof_process_goal(Goal0, Goal, !Info) :-
     Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
@@ -282,7 +289,7 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
         ;
             Unify0 = deconstruct(Var, ConsId, Args, ArgModes,
                 _CanFail, _CanCGC),
-            (
+            ( if
                 % The following test is an optimization. If
                 % BindingArgModes = [], which is almost 100% likely,
                 % then size_prof_process_deconstruct would return GoalExpr0 as
@@ -290,10 +297,10 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
                 list.filter(binds_arg_in_cell(!.Info), ArgModes,
                     BindingArgModes),
                 BindingArgModes = [_ | _]
-            ->
+            then
                 size_prof_process_deconstruct(Var, ConsId, Args, ArgModes,
                     Goal0, GoalExpr, !Info)
-            ;
+            else
                 GoalExpr = GoalExpr0
             )
         ;
@@ -304,7 +311,7 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
         ;
             Unify0 = complicated_unify(_, _, _),
             % These should have been expanded out by now.
-            unexpected($module, $pred, "complicated_unify")
+            unexpected($pred, "complicated_unify")
         )
     ;
         GoalExpr0 = plain_call(_, _, _, _, _, _),
@@ -375,7 +382,7 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
             Cases = [First | Later]
         ;
             Cases0 = [],
-            unexpected($module, $pred, "empty switch")
+            unexpected($pred, "empty switch")
         ),
         update_rev_maps(!Info),
         update_target_map(!Info),
@@ -475,42 +482,45 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
         % The code inside from_ground_term_construct scopes wants to construct
         % terms statically, but for term size profiling, we need to construct
         % terms dynamically,
-        ( Reason0 = from_ground_term(TermVar, from_ground_term_construct) ->
+        ( if
+            Reason0 = from_ground_term(TermVar, from_ground_term_construct)
+        then
             Reason = from_ground_term(TermVar, from_ground_term_other)
-        ;
+        else
             Reason = Reason0
         ),
         size_prof_process_goal(SubGoal0, SubGoal, !Info),
         GoalExpr = scope(Reason, SubGoal)
     ;
         GoalExpr0 = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ),
     Goal = hlds_goal(GoalExpr, GoalInfo0).
 
 %---------------------------------------------------------------------------%
 
 :- pred size_prof_process_conj(list(hlds_goal)::in, list(hlds_goal)::out,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 size_prof_process_conj([], [], !Info).
 size_prof_process_conj([Goal0 | Goals0], Conj, !Info) :-
     size_prof_process_goal(Goal0, Goal, !Info),
     size_prof_process_conj(Goals0, Goals, !Info),
-    ( Goal = hlds_goal(conj(plain_conj, SubConj), _) ->
+    ( if Goal = hlds_goal(conj(plain_conj, SubConj), _) then
         % Flatten out any conjunction introduced by size_prof_process_goal.
         % We never create conjunctions more than one level deep,
         % so this single test is sufficient to ensure that we never
         % leave conjunctions nested more deeply than the input goal.
         Conj = list.append(SubConj, Goals)
-    ;
+    else
         Conj = [Goal | Goals]
     ).
 
 %---------------------------------------------------------------------------%
 
 :- pred size_prof_process_par_conj(list(hlds_goal)::in, list(hlds_goal)::out,
-    info::in, info::out, type_info_map::in, type_info_map::in,
+    size_prof_info::in, size_prof_info::out,
+    type_info_map::in, type_info_map::in,
     type_ctor_map::in, known_size_map::in) is det.
 
 size_prof_process_par_conj([], [], !Info, _, _, _, _).
@@ -527,7 +537,8 @@ size_prof_process_par_conj([Goal0 | Goals0], [Goal | Goals], !Info,
 %---------------------------------------------------------------------------%
 
 :- pred size_prof_process_disj(hlds_goal::in, hlds_goal::out,
-    list(hlds_goal)::in, list(hlds_goal)::out, info::in, info::out,
+    list(hlds_goal)::in, list(hlds_goal)::out,
+    size_prof_info::in, size_prof_info::out,
     type_info_map::in, type_info_map::in, rev_type_info_map::in,
     type_ctor_map::in, rev_type_ctor_map::in,
     type_info_map::out, known_size_map::in, known_size_map::out) is det.
@@ -565,7 +576,7 @@ size_prof_process_disj(First0, First, Later0, Later, !Info, TargetTypeInfoMap,
 %---------------------------------------------------------------------------%
 
 :- pred size_prof_process_switch(case::in, case::out,
-    list(case)::in, list(case)::out, info::in, info::out,
+    list(case)::in, list(case)::out, size_prof_info::in, size_prof_info::out,
     type_info_map::in, type_info_map::in, rev_type_info_map::in,
     type_ctor_map::in, rev_type_ctor_map::in,
     type_info_map::out, known_size_map::in, known_size_map::out) is det.
@@ -607,9 +618,9 @@ size_prof_process_switch(First0, First, Later0, Later, !Info,
 
 :- pred size_prof_process_construct(prog_var::in, unify_rhs::in,
     unify_mode::in, unify_context::in, prog_var::in, cons_id::in,
-    list(prog_var)::in, list(uni_mode)::in, how_to_construct::in,
+    list(prog_var)::in, list(unify_mode)::in, how_to_construct::in,
     cell_is_unique::in, hlds_goal_info::in, hlds_goal_expr::out,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 size_prof_process_construct(LHS, RHS, UniMode, UnifyContext, Var, ConsId,
         Args, ArgModes, How, Unique, GoalInfo, GoalExpr, !Info) :-
@@ -617,46 +628,46 @@ size_prof_process_construct(LHS, RHS, UniMode, UnifyContext, Var, ConsId,
     type_to_ctor_det(VarType, VarTypeCtor),
     type_ctor_module_name_arity(VarTypeCtor,
         VarTypeCtorModule, VarTypeCtorName, _),
-    (
+    ( if
         ctor_is_type_info_related(VarTypeCtorModule, VarTypeCtorName)
-    ->
-        ( VarTypeCtorName = "type_info" ->
-            (
+    then
+        ( if VarTypeCtorName = "type_info" then
+            ( if
                 ConsId = type_info_cell_constructor(_),
                 Args = [TypeCtorInfoVar | ArgTypeInfoVars]
-            ->
+            then
                 record_known_type_info(Var, TypeCtorInfoVar, ArgTypeInfoVars,
                     !Info)
-            ;
+            else if
                 ConsId = type_ctor_info_const(M, N, A)
-            ->
+            then
                 % When type specialization creates a procedure with a type
                 % substitution such as K=int, it leaves the type of
                 % TypeInfo_for_K as type_info, not type_ctor_info.
                 record_known_type_ctor_info(Var, M, N, A, !Info)
-            ;
-                unexpected($module, $pred, "bad type_info")
+            else
+                unexpected($pred, "bad type_info")
             )
-        ; VarTypeCtorName = "type_ctor_info" ->
-            ( ConsId = type_ctor_info_const(M, N, A) ->
+        else if VarTypeCtorName = "type_ctor_info" then
+            ( if ConsId = type_ctor_info_const(M, N, A) then
                 record_known_type_ctor_info(Var, M, N, A, !Info)
-            ;
-                unexpected($module, $pred, "bad type_ctor_info")
+            else
+                unexpected($pred, "bad type_ctor_info")
             )
-        ;
+        else
             !:Info = !.Info
         ),
         Unification = construct(Var, ConsId, Args, ArgModes, How, Unique,
             no_construct_sub_info),
         GoalExpr = unify(LHS, RHS, UniMode, Unification, UnifyContext)
-    ;
+    else if
         ConsId = cons(_Name, _Arity, _TypeCtor),
         Args = [_ | _]
-    ->
+    then
         size_prof_process_cons_construct(LHS, RHS, UniMode, UnifyContext,
             Var, VarType, ConsId, Args, ArgModes, How, Unique,
             GoalInfo, GoalExpr, !Info)
-    ;
+    else
         % All ConsIds other than cons/2 with at least one argument
         % construct terms that we consider zero-sized.
         record_known_size(Var, 0, !Info),
@@ -668,8 +679,8 @@ size_prof_process_construct(LHS, RHS, UniMode, UnifyContext, Var, ConsId,
 %-----------------------------------------------------------------------------%
 
 :- pred size_prof_process_deconstruct(prog_var::in, cons_id::in,
-    list(prog_var)::in, list(uni_mode)::in, hlds_goal::in, hlds_goal_expr::out,
-    info::in, info::out) is det.
+    list(prog_var)::in, list(unify_mode)::in, hlds_goal::in,
+    hlds_goal_expr::out, size_prof_info::in, size_prof_info::out) is det.
 
 size_prof_process_deconstruct(Var, ConsId, Args, ArgModes, Goal0, GoalExpr,
         !Info) :-
@@ -677,19 +688,19 @@ size_prof_process_deconstruct(Var, ConsId, Args, ArgModes, Goal0, GoalExpr,
     type_to_ctor_det(VarType, VarTypeCtor),
     type_ctor_module_name_arity(VarTypeCtor, VarTypeCtorModule,
         VarTypeCtorName, _),
-    (
+    ( if
         ctor_is_type_info_related(VarTypeCtorModule, VarTypeCtorName)
-    ->
+    then
         Goal0 = hlds_goal(GoalExpr, _)
-    ;
+    else if
         ( ConsId = cons(_Name, _Arity, _TypeCtor)
         ; ConsId = tuple_cons(_Arity)
         ),
         Args = [_ | _]
-    ->
+    then
         size_prof_process_cons_deconstruct(Var, Args, ArgModes, Goal0,
             GoalExpr, !Info)
-    ;
+    else
         % All ConsIds other than cons/2 deconstruct terms that we
         % consider zero-sized.
         record_known_size(Var, 0, !Info),
@@ -700,9 +711,9 @@ size_prof_process_deconstruct(Var, ConsId, Args, ArgModes, Goal0, GoalExpr,
 
 :- pred size_prof_process_cons_construct(prog_var::in, unify_rhs::in,
     unify_mode::in, unify_context::in, prog_var::in, mer_type::in,
-    cons_id::in, list(prog_var)::in, list(uni_mode)::in,
+    cons_id::in, list(prog_var)::in, list(unify_mode)::in,
     how_to_construct::in, cell_is_unique::in, hlds_goal_info::in,
-    hlds_goal_expr::out, info::in, info::out) is det.
+    hlds_goal_expr::out, size_prof_info::in, size_prof_info::out) is det.
 
 size_prof_process_cons_construct(LHS, RHS, UniMode, UnifyContext, Var, _Type,
         ConsId, Args, ArgModes, How, Unique, GoalInfo0, GoalExpr, !Info) :-
@@ -713,7 +724,7 @@ size_prof_process_cons_construct(LHS, RHS, UniMode, UnifyContext, Var, _Type,
         no, MaybeDynamicSizeVar, Context, ArgGoals, !Info),
     (
         MaybeDynamicSizeVar = no,
-        expect(unify(ArgGoals, []), $module, $pred, "nonempty ArgGoals"),
+        expect(unify(ArgGoals, []), $pred, "nonempty ArgGoals"),
         (
             NonDefinedArgs = [],
             record_known_size(Var, KnownSize, !Info)
@@ -744,8 +755,8 @@ size_prof_process_cons_construct(LHS, RHS, UniMode, UnifyContext, Var, _Type,
 %-----------------------------------------------------------------------------%
 
 :- pred size_prof_process_cons_deconstruct(prog_var::in, list(prog_var)::in,
-    list(uni_mode)::in, hlds_goal::in, hlds_goal_expr::out,
-    info::in, info::out) is det.
+    list(unify_mode)::in, hlds_goal::in, hlds_goal_expr::out,
+    size_prof_info::in, size_prof_info::out) is det.
 
 size_prof_process_cons_deconstruct(Var, Args, ArgModes, UnifyGoal, GoalExpr,
         !Info) :-
@@ -756,7 +767,7 @@ size_prof_process_cons_deconstruct(Var, Args, ArgModes, UnifyGoal, GoalExpr,
         no, MaybeDynamicSizeVar, Context, ArgGoals, !Info),
     (
         MaybeDynamicSizeVar = no,
-        expect(unify(ArgGoals, []), $module, $pred, "nonempty ArgGoals"),
+        expect(unify(ArgGoals, []), $pred, "nonempty ArgGoals"),
         GoalExpr = GoalExpr0
     ;
         MaybeDynamicSizeVar = yes(SizeVar0),
@@ -765,10 +776,10 @@ size_prof_process_cons_deconstruct(Var, Args, ArgModes, UnifyGoal, GoalExpr,
         % The increment_size primitive doesn't need Var's type_info,
         % so we make it a no_type_info_builtin.
         TermSizeProfBuiltin = mercury_term_size_prof_builtin_module,
-        goal_util.generate_simple_call(TermSizeProfBuiltin,
-            "increment_size", pf_predicate, only_mode, detism_det,
-            purity_impure, [Var, SizeVar], [], instmap_delta_bind_no_var,
-            !.Info ^ spi_module_info, Context, UpdateGoal),
+        generate_simple_call(!.Info ^ spi_module_info,
+            TermSizeProfBuiltin, "increment_size", pf_predicate, only_mode,
+            detism_det, purity_impure, [Var, SizeVar], [],
+            instmap_delta_bind_no_var, Context, UpdateGoal),
         % Put UnifyGoal first in case it fails.
         Goals = [UnifyGoal] ++ ArgGoals ++ SizeGoals ++ [UpdateGoal],
         GoalExpr = conj(plain_conj, Goals)
@@ -787,18 +798,18 @@ size_prof_process_cons_deconstruct(Var, Args, ArgModes, UnifyGoal, GoalExpr,
     %
 :- pred size_prof_process_args(list(prog_var)::in, int::in, int::out,
     maybe(prog_var)::in, maybe(prog_var)::out, prog_context::in,
-    list(hlds_goal)::out, info::in, info::out) is det.
+    list(hlds_goal)::out, size_prof_info::in, size_prof_info::out) is det.
 
 size_prof_process_args([], !KnownSize, !MaybeSizeVar, _, [], !Info).
 size_prof_process_args([Arg | Args], !KnownSize, !MaybeSizeVar, Context, Goals,
         !Info) :-
     lookup_var_type(!.Info ^ spi_vartypes, Arg, Type),
-    ( map.search(!.Info ^ spi_known_size_map, Arg, ArgSize) ->
+    ( if map.search(!.Info ^ spi_known_size_map, Arg, ArgSize) then
         !:KnownSize = !.KnownSize + ArgSize,
         ArgGoals = []
-    ; zero_size_type(!.Info ^ spi_module_info, Type) ->
+    else if zero_size_type(!.Info ^ spi_module_info, Type) then
         ArgGoals = []
-    ;
+    else
         make_type_info(Context, Type, TypeInfoVar, TypeInfoGoals, !Info),
         make_size_goal(TypeInfoVar, Arg, Context, SizeGoal, !MaybeSizeVar,
             !Info),
@@ -816,13 +827,14 @@ size_prof_process_args([Arg | Args], !KnownSize, !MaybeSizeVar, Context, Goals,
     % compute it.
     %
 :- pred generate_size_var(prog_var::in, int::in, prog_context::in,
-    prog_var::out, list(hlds_goal)::out, info::in, info::out) is det.
+    prog_var::out, list(hlds_goal)::out,
+    size_prof_info::in, size_prof_info::out) is det.
 
 generate_size_var(SizeVar0, KnownSize, Context, SizeVar, Goals, !Info) :-
-    ( KnownSize = 0 ->
+    ( if KnownSize = 0 then
         SizeVar = SizeVar0,
         Goals = []
-    ;
+    else
         VarSet0 = !.Info ^ spi_varset,
         VarTypes0 = !.Info ^ spi_vartypes,
         make_int_const_construction_alloc(KnownSize,
@@ -832,11 +844,10 @@ generate_size_var(SizeVar0, KnownSize, Context, SizeVar, Goals, !Info) :-
         !Info ^ spi_vartypes := VarTypes1,
         get_new_var(int_type, "FinalSizeVar", SizeVar, !Info),
         TermSizeProfModule = mercury_term_size_prof_builtin_module,
-        goal_util.generate_simple_call(TermSizeProfModule,
-            "term_size_plus", pf_function, mode_no(0), detism_det, purity_pure,
-            [SizeVar0, KnownSizeVar, SizeVar], [],
-            instmap_delta_bind_var(SizeVar),
-            !.Info ^ spi_module_info, Context, AddGoal),
+        generate_simple_call(!.Info ^ spi_module_info,
+            TermSizeProfModule, "term_size_plus", pf_function, mode_no(0),
+            detism_det, purity_pure, [SizeVar0, KnownSizeVar, SizeVar], [],
+            instmap_delta_bind_var(SizeVar), Context, AddGoal),
         Goals = [KnownSizeGoal, AddGoal]
     ).
 
@@ -848,16 +859,16 @@ generate_size_var(SizeVar0, KnownSize, Context, SizeVar, Goals, !Info) :-
     % TypeInfoVar, and the goals needed to create it in TypeInfoGoals.
 
 :- pred make_type_info(prog_context::in, mer_type::in, prog_var::out,
-    list(hlds_goal)::out, info::in, info::out) is det.
+    list(hlds_goal)::out, size_prof_info::in, size_prof_info::out) is det.
 
 make_type_info(Context, Type, TypeInfoVar, TypeInfoGoals, !Info) :-
-    ( map.search(!.Info ^ spi_type_info_map, Type, TypeInfoVarPrime) ->
+    ( if map.search(!.Info ^ spi_type_info_map, Type, TypeInfoVarPrime) then
         TypeInfoVar = TypeInfoVarPrime,
         TypeInfoGoals = []
-    ; type_has_variable_arity_ctor(Type, TypeCtor, ArgTypes) ->
+    else if type_has_variable_arity_ctor(Type, TypeCtor, ArgTypes) then
         construct_type_info(Context, Type, TypeCtor, ArgTypes, yes,
             TypeInfoVar, TypeInfoGoals, !Info)
-    ; type_to_ctor_and_args(Type, TypeCtor, ArgTypes) ->
+    else if type_to_ctor_and_args(Type, TypeCtor, ArgTypes) then
         (
             ArgTypes = [],
             make_type_ctor_info(TypeCtor, [], TypeCtorVar, TypeCtorGoals,
@@ -869,7 +880,7 @@ make_type_info(Context, Type, TypeInfoVar, TypeInfoGoals, !Info) :-
             construct_type_info(Context, Type, TypeCtor, ArgTypes,
                 no, TypeInfoVar, TypeInfoGoals, !Info)
         )
-    ; Type = type_variable(TVar, _) ->
+    else if Type = type_variable(TVar, _) then
         rtti_lookup_type_info_locn(!.Info ^ spi_rtti_varmaps, TVar, TVarLocn),
         (
             TVarLocn = type_info(TypeInfoVar),
@@ -879,13 +890,13 @@ make_type_info(Context, Type, TypeInfoVar, TypeInfoGoals, !Info) :-
             TargetTypeInfoMap = !.Info ^ spi_target_type_info_map,
             VarSet0 = !.Info ^ spi_varset,
             VarTypes0 = !.Info ^ spi_vartypes,
-            ( map.search(TargetTypeInfoMap, Type, TargetVar) ->
+            ( if map.search(TargetTypeInfoMap, Type, TargetVar) then
                 TypeInfoVar = TargetVar,
                 VarSet1 = VarSet0,
                 VarTypes1 = VarTypes0
-            ;
+            else
                 RttiVarMaps0 = !.Info ^ spi_rtti_varmaps,
-                polymorphism.new_type_info_var_raw(Type, type_info,
+                new_type_info_var_raw(Type, type_info,
                     TypeInfoVar, VarSet0, VarSet1, VarTypes0, VarTypes1,
                     RttiVarMaps0, RttiVarMaps),
                 !Info ^ spi_rtti_varmaps := RttiVarMaps
@@ -895,20 +906,19 @@ make_type_info(Context, Type, TypeInfoVar, TypeInfoGoals, !Info) :-
             !Info ^ spi_varset := VarSet,
             !Info ^ spi_vartypes := VarTypes,
             PrivateBuiltin = mercury_private_builtin_module,
-            goal_util.generate_simple_call(PrivateBuiltin,
-                "type_info_from_typeclass_info", pf_predicate, only_mode,
-                detism_det, purity_pure,
+            generate_simple_call(!.Info ^ spi_module_info,
+                PrivateBuiltin, "type_info_from_typeclass_info", pf_predicate,
+                only_mode, detism_det, purity_pure,
                 [TypeClassInfoVar, SlotVar, TypeInfoVar], [],
-                instmap_delta_bind_var(TypeInfoVar),
-                !.Info ^ spi_module_info, Context, ExtractGoal),
+                instmap_delta_bind_var(TypeInfoVar), Context, ExtractGoal),
             record_type_info_var(Type, TypeInfoVar, !Info),
             TypeInfoGoals = [SlotGoal, ExtractGoal]
         )
-    ;
+    else
         % Type_to_ctor_and_args can fail only if Type is a type variable,
         % or acts like one. The tests in our callers should have filtered
         % out both cases.
-        unexpected($module, $pred, "cannot happen")
+        unexpected($pred, "cannot happen")
     ).
 
     % Construct a type_info for Type = TypeCtor(ArgTypes), given that we know
@@ -918,7 +928,7 @@ make_type_info(Context, Type, TypeInfoVar, TypeInfoGoals, !Info) :-
     %
 :- pred construct_type_info(prog_context::in, mer_type::in, type_ctor::in,
     list(mer_type)::in, bool::in, prog_var::out, list(hlds_goal)::out,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 construct_type_info(Context, Type, TypeCtor, ArgTypes, CtorIsVarArity,
         TypeInfoVar, TypeInfoGoals, !Info) :-
@@ -946,12 +956,12 @@ construct_type_info(Context, Type, TypeCtor, ArgTypes, CtorIsVarArity,
     VarTypes2 = !.Info ^ spi_vartypes,
     RttiVarMaps0 = !.Info ^ spi_rtti_varmaps,
     TargetTypeInfoMap = !.Info ^ spi_target_type_info_map,
-    ( map.search(TargetTypeInfoMap, Type, PrefTIVar) ->
+    ( if map.search(TargetTypeInfoMap, Type, PrefTIVar) then
         MaybePreferredVar = yes(PrefTIVar)
-    ;
+    else
         MaybePreferredVar = no
     ),
-    polymorphism.init_type_info_var(Type, ArgVars, MaybePreferredVar,
+    init_type_info_var(Type, ArgVars, MaybePreferredVar,
         TypeInfoVar, TypeInfoGoal, VarSet2, VarSet, VarTypes2, VarTypes,
         RttiVarMaps0, RttiVarMaps),
     !Info ^ spi_varset := VarSet,
@@ -965,25 +975,27 @@ construct_type_info(Context, Type, TypeCtor, ArgTypes, CtorIsVarArity,
     % TypeCtorVar, and the goals needed to create it in TypeCtorGoals.
     %
 :- pred make_type_ctor_info(type_ctor::in, list(mer_type)::in, prog_var::out,
-    list(hlds_goal)::out, info::in, info::out) is det.
+    list(hlds_goal)::out, size_prof_info::in, size_prof_info::out) is det.
 
 make_type_ctor_info(TypeCtor, TypeArgs, TypeCtorVar, TypeCtorGoals, !Info) :-
-    ( map.search(!.Info ^ spi_type_ctor_map, TypeCtor, TypeCtorVarPrime) ->
+    ( if
+        map.search(!.Info ^ spi_type_ctor_map, TypeCtor, TypeCtorVarPrime)
+    then
         TypeCtorVar = TypeCtorVarPrime,
         TypeCtorGoals = []
-    ;
-        (
+    else
+        ( if
             type_ctor_is_higher_order(TypeCtor, Purity, PredOrFunc, EvalMethod)
-        ->
+        then
             construct_higher_order_type(Purity, PredOrFunc, EvalMethod,
                 TypeArgs, Type)
-        ;
+        else
             construct_type(TypeCtor, [], Type)
         ),
         VarSet0 = !.Info ^ spi_varset,
         VarTypes0 = !.Info ^ spi_vartypes,
         RttiVarMaps0 = !.Info ^ spi_rtti_varmaps,
-        polymorphism.init_const_type_ctor_info_var(Type, TypeCtor,
+        init_const_type_ctor_info_var(Type, TypeCtor,
             TypeCtorVar, _, TypeCtorGoal, VarSet0, VarSet, VarTypes0, VarTypes,
             RttiVarMaps0, RttiVarMaps),
         TypeCtorGoals = [TypeCtorGoal],
@@ -1005,7 +1017,7 @@ make_type_ctor_info(TypeCtor, TypeArgs, TypeCtorVar, TypeCtorGoals, !Info) :-
     %
 :- pred make_size_goal(prog_var::in, prog_var::in, prog_context::in,
     hlds_goal::out, maybe(prog_var)::in, maybe(prog_var)::out,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 make_size_goal(TypeInfoVar, Arg, Context, SizeGoal,
         MaybeSizeVar0, MaybeSizeVar, !Info) :-
@@ -1020,10 +1032,9 @@ make_size_goal(TypeInfoVar, Arg, Context, SizeGoal,
         Args = [TypeInfoVar, Arg, SizeVar]
     ),
     TermSizeProfBuiltin = mercury_term_size_prof_builtin_module,
-    goal_util.generate_simple_call(TermSizeProfBuiltin, Pred, pf_predicate,
-        only_mode, detism_det, purity_pure, Args,
-        [], instmap_delta_bind_var(SizeVar),
-        !.Info ^ spi_module_info, Context, SizeGoal),
+    generate_simple_call(!.Info ^ spi_module_info, TermSizeProfBuiltin, Pred,
+        pf_predicate, only_mode, detism_det, purity_pure, Args, [],
+        instmap_delta_bind_var(SizeVar), Context, SizeGoal),
     MaybeSizeVar = yes(SizeVar).
 
 %---------------------------------------------------------------------------%
@@ -1032,7 +1043,7 @@ make_size_goal(TypeInfoVar, Arg, Context, SizeGoal,
     % variable number.
     %
 :- pred get_new_var(mer_type::in, string::in, prog_var::out,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 get_new_var(Type, Prefix, Var, !Info) :-
     VarSet0 = !.Info ^ spi_varset,
@@ -1067,7 +1078,7 @@ get_new_var(Type, Prefix, Var, !Info) :-
     % various branches to be used when processing the following code.
     %
 :- pred record_known_type_ctor_info(prog_var::in, module_name::in, string::in,
-    int::in, info::in, info::out) is det.
+    int::in, size_prof_info::in, size_prof_info::out) is det.
 
 record_known_type_ctor_info(Var, TypeCtorModule, TypeCtorName, TypeCtorArity,
         !Info) :-
@@ -1081,23 +1092,25 @@ record_known_type_ctor_info(Var, TypeCtorModule, TypeCtorName, TypeCtorArity,
     !Info ^ spi_rev_type_ctor_map := RevTypeCtorMap.
 
 :- pred record_known_type_info(prog_var::in, prog_var::in, list(prog_var)::in,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 record_known_type_info(Var, TypeCtorInfoVar, ArgTypeInfoVars, !Info) :-
     RevTypeCtorMap0 = !.Info ^ spi_rev_type_ctor_map,
-    ( map.search(RevTypeCtorMap0, TypeCtorInfoVar, TypeCtor0) ->
+    ( if map.search(RevTypeCtorMap0, TypeCtorInfoVar, TypeCtor0) then
         RevTypeInfoMap0 = !.Info ^ spi_rev_type_info_map,
-        ( list.map(map.search(RevTypeInfoMap0), ArgTypeInfoVars, ArgTypes) ->
+        ( if
+            list.map(map.search(RevTypeInfoMap0), ArgTypeInfoVars, ArgTypes)
+        then
             list.length(ArgTypes, Arity),
             % Just in case TypeCtorInfo0 has fake arity, e.g. if it is a tuple.
             TypeCtor0 = type_ctor(SymName, _DeclArity),
             TypeCtor1 = type_ctor(SymName, Arity),
             construct_type(TypeCtor1, ArgTypes, Type),
             record_type_info_var(Type, Var, !Info)
-        ;
+        else
             true
         )
-    ;
+    else
         % We don't know what type is being constructed, because we have
         % forgotten what type constructor the type constructor variable
         % stands for due to an intervening call. This cannot happen
@@ -1107,25 +1120,26 @@ record_known_type_info(Var, TypeCtorInfoVar, ArgTypeInfoVars, !Info) :-
         true
     ).
 
-:- pred record_type_info_var(mer_type::in, prog_var::in, info::in, info::out)
-    is det.
+:- pred record_type_info_var(mer_type::in, prog_var::in,
+    size_prof_info::in, size_prof_info::out) is det.
 
 record_type_info_var(Type, Var, !Info) :-
     RevTypeInfoMap0 = !.Info ^ spi_rev_type_info_map,
     TypeInfoMap0 = !.Info ^ spi_type_info_map,
     map.set(Type, Var, TypeInfoMap0, TypeInfoMap),
-    ( map.insert(Var, Type, RevTypeInfoMap0, RevTypeInfoMap1) ->
+    ( if map.insert(Var, Type, RevTypeInfoMap0, RevTypeInfoMap1) then
         RevTypeInfoMap = RevTypeInfoMap1
-    ;
-        % This can happen because inlining XXX can leave a
-        % type_info_varmap saying that one type_info variable
-        % holds the typeinfo for more than one type.
+    else
+        % This can happen because inlining XXX can leave a type_info_varmap
+        % saying that one type_info variable holds the typeinfo for
+        % more than one type.
         RevTypeInfoMap = RevTypeInfoMap0
     ),
     !Info ^ spi_type_info_map := TypeInfoMap,
     !Info ^ spi_rev_type_info_map := RevTypeInfoMap.
 
-:- pred record_known_size(prog_var::in, int::in, info::in, info::out) is det.
+:- pred record_known_size(prog_var::in, int::in,
+    size_prof_info::in, size_prof_info::out) is det.
 
 record_known_size(Var, KnownSize, !Info) :-
     KnownSizeMap0 = !.Info ^ spi_known_size_map,
@@ -1133,7 +1147,7 @@ record_known_size(Var, KnownSize, !Info) :-
     !Info ^ spi_known_size_map := KnownSizeMap.
 
 :- pred record_typeinfo_in_type_info_varmap(rtti_varmaps::in, tvar::in,
-    info::in, info::out) is det.
+    size_prof_info::in, size_prof_info::out) is det.
 
 record_typeinfo_in_type_info_varmap(RttiVarMaps, TVar, !Info) :-
     rtti_lookup_type_info_locn(RttiVarMaps, TVar, TypeInfoLocn),
@@ -1169,7 +1183,7 @@ record_typeinfo_in_type_info_varmap(RttiVarMaps, TVar, !Info) :-
     %     (since a forward map can say that e.g. both the type int and the type
     %     constructor int/0 have their typeinfo in the same variable).
     %
-:- pred update_rev_maps(info::in, info::out) is det.
+:- pred update_rev_maps(size_prof_info::in, size_prof_info::out) is det.
 
 update_rev_maps(!Info) :-
     map.to_sorted_assoc_list(!.Info ^ spi_type_info_map, TypeInfoList),
@@ -1187,9 +1201,9 @@ update_rev_maps(!Info) :-
 
 count_appearances([], !VarCounts).
 count_appearances([_ - Var | AssocList], !VarCounts) :-
-    ( map.search(!.VarCounts, Var, Count) ->
+    ( if map.search(!.VarCounts, Var, Count) then
         map.det_update(Var, Count + 1, !VarCounts)
-    ;
+    else
         map.det_insert(Var, 1, !VarCounts)
     ),
     count_appearances(AssocList, !VarCounts).
@@ -1201,9 +1215,9 @@ count_appearances([_ - Var | AssocList], !VarCounts) :-
 construct_rev_map([], _, !RevMap).
 construct_rev_map([T - Var | AssocList], VarCounts, !RevMap) :-
     map.lookup(VarCounts, Var, Count),
-    ( Count = 1 ->
+    ( if Count = 1 then
         map.det_insert(Var, T, !RevMap)
-    ;
+    else
         true
     ),
     construct_rev_map(AssocList, VarCounts, !RevMap).
@@ -1221,7 +1235,7 @@ construct_rev_map([T - Var | AssocList], VarCounts, !RevMap) :-
     % for later code. We must therefore remove the variable from the target
     % type_info map used by later code.
     %
-:- pred update_target_map(info::in, info::out) is det.
+:- pred update_target_map(size_prof_info::in, size_prof_info::out) is det.
 
 update_target_map(!Info) :-
     TargetTypeInfoMap0 = !.Info ^ spi_target_type_info_map,
@@ -1235,15 +1249,15 @@ update_target_map(!Info) :-
     type_info_map::in, type_info_map::out) is det.
 
 include_in_target_map(TypeInfoMap, Type - TypeInfoVar, !TargetTypeInfoMap) :-
-    ( map.search(TypeInfoMap, Type, TypeInfoVar) ->
+    ( if map.search(TypeInfoMap, Type, TypeInfoVar) then
         map.det_insert(Type, TypeInfoVar, !TargetTypeInfoMap)
-    ;
+    else
         true
     ).
 
 %---------------------------------------------------------------------------%
 
-:- func compute_functor_size(list(prog_var), info) = int.
+:- func compute_functor_size(list(prog_var), size_prof_info) = int.
 
 compute_functor_size(Args, Info) = FunctorSize :-
     TransformOp = Info ^ spi_transform_op,
@@ -1255,8 +1269,8 @@ compute_functor_size(Args, Info) = FunctorSize :-
         FunctorSize = list.length(Args)
     ).
 
-:- pred find_defined_args(list(prog_var)::in, list(uni_mode)::in,
-    list(prog_var)::out, list(prog_var)::out, info::in) is det.
+:- pred find_defined_args(list(prog_var)::in, list(unify_mode)::in,
+    list(prog_var)::out, list(prog_var)::out, size_prof_info::in) is det.
 
 find_defined_args(Args, Modes, DefinedArgs, NonDefinedArgs, Info) :-
     (
@@ -1267,30 +1281,30 @@ find_defined_args(Args, Modes, DefinedArgs, NonDefinedArgs, Info) :-
     ;
         Args = [],
         Modes = [_ | _],
-        unexpected($module, $pred, "length mismatch")
+        unexpected($pred, "length mismatch")
     ;
         Args = [_ | _],
         Modes = [],
-        unexpected($module, $pred, "length mismatch")
+        unexpected($pred, "length mismatch")
     ;
         Args = [FirstArg | LaterArgs],
         Modes = [FirstMode | LaterModes],
         find_defined_args(LaterArgs, LaterModes, LaterDefinedArgs,
             LaterNonDefinedArgs, Info),
-        ( binds_arg_in_cell(Info, FirstMode) ->
+        ( if binds_arg_in_cell(Info, FirstMode) then
             DefinedArgs = [FirstArg | LaterDefinedArgs],
             NonDefinedArgs = LaterNonDefinedArgs
-        ;
+        else
             DefinedArgs = LaterDefinedArgs,
             NonDefinedArgs = [FirstArg | LaterNonDefinedArgs]
         )
     ).
 
-:- pred binds_arg_in_cell(info::in, uni_mode::in) is semidet.
+:- pred binds_arg_in_cell(size_prof_info::in, unify_mode::in) is semidet.
 
-binds_arg_in_cell(Info,
-        (CellInitInst - _ArgInitInst) -> (CellFinalInst - _ArgFinalInst)) :-
+binds_arg_in_cell(Info, UnifyMode) :-
     ModuleInfo = Info ^ spi_module_info,
+    UnifyMode = unify_modes_li_lf_ri_rf(CellInitInst, CellFinalInst, _, _),
     inst_is_free(ModuleInfo, CellInitInst),
     inst_is_bound(ModuleInfo, CellFinalInst).
 
