@@ -71,6 +71,7 @@
 :- module ll_backend.stack_opt.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 
@@ -84,10 +85,14 @@
 
 :- implementation.
 
+:- import_module backend_libs.
 :- import_module backend_libs.interval.
 :- import_module backend_libs.matching.
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module check_hlds.simplify.
+:- import_module check_hlds.simplify.simplify_proc.
+:- import_module check_hlds.simplify.simplify_tasks.
 :- import_module check_hlds.type_util.
 :- import_module hlds.arg_info.
 :- import_module hlds.goal_path.
@@ -97,14 +102,23 @@
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_goal.
 :- import_module hlds.hlds_out.hlds_out_util.
+:- import_module hlds.passes_aux.
 :- import_module hlds.quantification.
+:- import_module hlds.vartypes.
+:- import_module libs.
 :- import_module libs.globals.
+:- import_module libs.optimization_options.
 :- import_module libs.options.
 :- import_module ll_backend.live_vars.
 :- import_module ll_backend.liveness.
 :- import_module ll_backend.store_alloc.
+:- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
+:- import_module parse_tree.parse_tree_out_info.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_type.
 :- import_module parse_tree.set_of_var.
 
@@ -119,6 +133,7 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
+:- import_module string.
 :- import_module term.
 
 %-----------------------------------------------------------------------------%
@@ -132,7 +147,7 @@
 :- type opt_stack_alloc
     --->    opt_stack_alloc(
                 % XXX: this is an over-simplification, it gives stack slots to
-                % variables that may not need them.  For example, vars local to
+                % variables that may not need them. For example, vars local to
                 % loop control scopes and parallel conjunctions, And vars used
                 % after a loop control scope but before a recursive call don't
                 % need to be placed here.
@@ -141,11 +156,16 @@
 
 :- type stack_opt_params
     --->    stack_opt_params(
+                % These two fields are used to find the appropriate
+                % debug output stream.
+                sop_globals             :: globals,
+                sop_module_name         :: module_name,
+
                 sop_matching_params     :: matching_params,
                 sop_all_path_node_ratio :: int,
-                sop_fixpoint_loop       :: bool,
-                sop_full_path           :: bool,
-                sop_on_stack            :: bool,
+                sop_fixpoint_loop       :: maybe_opt_svcell_loop,
+                sop_full_path           :: maybe_opt_svcell_full_path,
+                sop_on_stack            :: maybe_opt_svcell_on_stack,
                 sop_non_candidate_vars  :: set_of_progvar
             ).
 
@@ -174,12 +194,12 @@ stack_opt_cell(PredProcId, !ProcInfo, !ModuleInfo) :-
     % This simplication is necessary to fix some bad inputs from
     % getting to the liveness computation.
     % (see tests/valid/stack_opt_simplify.m)
-    Simplications = list_to_simplifications([]),
-    simplify_proc(Simplications, PredId, ProcId, !ModuleInfo, !ProcInfo),
-    detect_liveness_proc(!.ModuleInfo, PredProcId, !ProcInfo),
-    initial_liveness(!.ProcInfo, PredId, !.ModuleInfo, Liveness0),
     module_info_get_globals(!.ModuleInfo, Globals),
+    SimplifyTasks = list_to_simplify_tasks(Globals, []),
+    simplify_proc(SimplifyTasks, PredId, ProcId, !ModuleInfo, !ProcInfo),
+    detect_liveness_proc(!.ModuleInfo, PredProcId, !ProcInfo),
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
+    initial_liveness(!.ModuleInfo, PredInfo, !.ProcInfo, Liveness0),
     body_should_use_typeinfo_liveness(PredInfo, Globals, TypeInfoLiveness),
     globals.lookup_bool_option(Globals, opt_no_return_calls,
         OptNoReturnCalls),
@@ -246,14 +266,14 @@ optimize_live_sets(ModuleInfo, OptAlloc, !ProcInfo, Changed, DebugStackOpt,
         InputArgs, OutputArgs, UnusedArgs),
     HeadVars = set.union_list([InputArgs, OutputArgs, UnusedArgs]),
     module_info_get_globals(ModuleInfo, Globals),
-    globals.lookup_bool_option(Globals,
-        optimize_saved_vars_cell_candidate_headvars, CandHeadvars),
+    globals.get_opt_tuple(Globals, OptTuple),
+    CandHeadvars = OptTuple ^ ot_opt_svcell_candidate_headvars,
     (
-        CandHeadvars = no,
+        CandHeadvars = do_not_opt_svcell_candidate_headvars,
         set_of_var.union(set_to_bitset(HeadVars), ParConjOwnSlot,
             NonCandidateVars)
     ;
-        CandHeadvars = yes,
+        CandHeadvars = opt_svcell_candidate_headvars,
         NonCandidateVars = ParConjOwnSlot
     ),
     Counter0 = counter.init(1),
@@ -264,33 +284,20 @@ optimize_live_sets(ModuleInfo, OptAlloc, !ProcInfo, Changed, DebugStackOpt,
     map.init(StartMap0),
     SuccMap0 = map.singleton(CurIntervalId, []),
     VarsMap0 = map.singleton(CurIntervalId, set_to_bitset(OutputArgs)),
-    globals.lookup_int_option(Globals,
-        optimize_saved_vars_cell_cv_store_cost, CellVarStoreCost),
-    globals.lookup_int_option(Globals,
-        optimize_saved_vars_cell_cv_load_cost, CellVarLoadCost),
-    globals.lookup_int_option(Globals,
-        optimize_saved_vars_cell_fv_store_cost, FieldVarStoreCost),
-    globals.lookup_int_option(Globals,
-        optimize_saved_vars_cell_fv_load_cost, FieldVarLoadCost),
-    globals.lookup_int_option(Globals,
-        optimize_saved_vars_cell_op_ratio, OpRatio),
-    globals.lookup_int_option(Globals,
-        optimize_saved_vars_cell_node_ratio, NodeRatio),
-    globals.lookup_bool_option(Globals,
-        optimize_saved_vars_cell_include_all_candidates, InclAllCand),
+    CellVarStoreCost = OptTuple ^ ot_opt_svcell_cv_store_cost,
+    CellVarLoadCost = OptTuple ^ ot_opt_svcell_cv_load_cost,
+    FieldVarStoreCost = OptTuple ^ ot_opt_svcell_fv_store_cost,
+    FieldVarLoadCost = OptTuple ^ ot_opt_svcell_fv_load_cost,
+    OpRatio = OptTuple ^ ot_opt_svcell_op_ratio,
+    NodeRatio = OptTuple ^ ot_opt_svcell_node_ratio,
+    InclAllCand = OptTuple ^ ot_opt_svcell_all_candidates,
     MatchingParams = matching_params(CellVarStoreCost, CellVarLoadCost,
         FieldVarStoreCost, FieldVarLoadCost, OpRatio, NodeRatio, InclAllCand),
-    globals.lookup_int_option(Globals,
-        optimize_saved_vars_cell_all_path_node_ratio,
-        AllPathNodeRatio),
-    globals.lookup_bool_option(Globals,
-        optimize_saved_vars_cell_loop, FixpointLoop),
-    globals.lookup_bool_option(Globals,
-        optimize_saved_vars_cell_full_path, FullPath),
-    globals.lookup_bool_option(Globals,
-        optimize_saved_vars_cell_on_stack, OnStack),
-    globals.lookup_bool_option(Globals,
-        opt_no_return_calls, OptNoReturnCalls),
+    AllPathNodeRatio = OptTuple ^ ot_opt_svcell_all_path_node_ratio,
+    FixpointLoop = OptTuple ^ ot_opt_svcell_loop,
+    FullPath = OptTuple ^ ot_opt_svcell_full_path,
+    OnStack = OptTuple ^ ot_opt_svcell_on_stack,
+    globals.lookup_bool_option(Globals, opt_no_return_calls, OptNoReturnCalls),
     IntParams = interval_params(ModuleInfo, VarTypes0, OptNoReturnCalls),
     IntervalInfo0 = interval_info(IntParams,
         set_of_var.init, set_to_bitset(OutputArgs),
@@ -298,23 +305,25 @@ optimize_live_sets(ModuleInfo, OptAlloc, !ProcInfo, Changed, DebugStackOpt,
         set.make_singleton_set(CurIntervalId),
         map.init, set.init, StartMap0, EndMap0,
         SuccMap0, VarsMap0, map.init),
-    StackOptParams = stack_opt_params(MatchingParams, AllPathNodeRatio,
-        FixpointLoop, FullPath, OnStack, NonCandidateVars),
+    module_info_get_name(ModuleInfo, ModuleName),
+    StackOptParams = stack_opt_params(Globals, ModuleName, MatchingParams,
+        AllPathNodeRatio, FixpointLoop, FullPath, OnStack, NonCandidateVars),
     StackOptInfo0 = stack_opt_info(StackOptParams, InsertMap0, []),
     build_interval_info_in_goal(Goal0, IntervalInfo0, IntervalInfo,
         StackOptInfo0, StackOptInfo),
-    ( DebugStackOpt = PredIdInt ->
+    ( if DebugStackOpt = PredIdInt then
         trace [io(!IO)] (
-            dump_interval_info(IntervalInfo, !IO),
-            dump_stack_opt_info(StackOptInfo, !IO)
+            get_debug_output_stream(ModuleInfo, DebugStream, !IO),
+            dump_interval_info(DebugStream, IntervalInfo, !IO),
+            dump_stack_opt_info(DebugStream, StackOptInfo, !IO)
         )
-    ;
+    else
         true
     ),
     InsertMap = StackOptInfo ^ soi_left_anchor_inserts,
-    ( map.is_empty(InsertMap) ->
+    ( if map.is_empty(InsertMap) then
         Changed = no
-    ;
+    else
         record_decisions_in_goal(Goal0, Goal1, VarSet0, VarSet,
             VarTypes0, VarTypes, map.init, RenameMap,
             InsertMap, yes(feature_stack_opt)),
@@ -415,37 +424,33 @@ use_cell(CellVar, FieldVarList, ConsId, Goal, !IntervalInfo, !StackOptInfo) :-
     set_of_var.intersect(FieldVars, FlushedLater, FlushedLaterFieldVars),
     set_of_var.difference(FlushedLaterFieldVars, NonCandidateVars,
         CandidateArgVars0),
-    (
+    ( if
         set_of_var.is_empty(CandidateArgVars0)
-    ->
+    then
         true
-    ;
+    else if
         ConsId = cons(_Name, _Arity, _TypeCtor),
         IntParams = !.IntervalInfo ^ ii_interval_params,
         VarTypes = IntParams ^ ip_var_types,
         lookup_var_type(VarTypes, CellVar, Type),
-        (
+        ( if
             type_is_tuple(Type, _)
-        ->
+        then
             FreeOfCost = no
-        ;
-            type_to_ctor(Type, TypeCtor),
+        else if
             ModuleInfo = IntParams ^ ip_module_info,
-            module_info_get_type_table(ModuleInfo, TypeTable),
-            lookup_type_ctor_defn(TypeTable, TypeCtor, TypeDefn),
-            hlds_data.get_type_defn_body(TypeDefn, TypeBody),
-            ConsTable = TypeBody ^ du_type_cons_tag_values
-        ->
-            map.lookup(ConsTable, ConsId, ConsTag),
-            ( ConsTag = no_tag ->
+            get_cons_repn_defn(ModuleInfo, ConsId, ConsRepn)
+        then
+            ConsTag = ConsRepn ^ cr_tag,
+            ( if ConsTag = no_tag then
                 FreeOfCost = yes
-            ;
+            else
                 FreeOfCost = no
             )
-        ;
+        else
             fail
         )
-    ->
+    then
         set_of_var.insert(CellVar, FieldVars, RelevantVars),
         find_all_branches_from_cur_interval(RelevantVars, MatchInfo,
             !.IntervalInfo, !.StackOptInfo),
@@ -466,22 +471,22 @@ use_cell(CellVar, FieldVarList, ConsId, Goal, !IntervalInfo, !StackOptInfo) :-
                 set_of_var.difference(CandidateArgVars0, RelevantAfterVars,
                     CandidateArgVars),
                 (
-                    OnStack = yes,
-                    ( set_of_var.member(FlushedLater, CellVar) ->
+                    OnStack = opt_svcell_on_stack,
+                    ( if set_of_var.member(FlushedLater, CellVar) then
                         CellVarFlushedLater = yes
-                    ;
+                    else
                         CellVarFlushedLater = no
                     )
                 ;
-                    OnStack = no,
-                    (
+                    OnStack = do_not_opt_svcell_on_stack,
+                    ( if
                         list.member(PathInfo, PathsInfo),
                         PathInfo = match_path_info(_, Segments),
                         list.member(Segment, Segments),
                         set_of_var.member(Segment, CellVar)
-                    ->
+                    then
                         CellVarFlushedLater = yes
-                    ;
+                    else
                         CellVarFlushedLater = no
                     )
                 ),
@@ -494,7 +499,7 @@ use_cell(CellVar, FieldVarList, ConsId, Goal, !IntervalInfo, !StackOptInfo) :-
                 AfterModelNon = yes
             )
         )
-    ;
+    else
         true
     ).
 
@@ -512,9 +517,9 @@ apply_matching(CellVar, CellVarFlushedLater, IntParams, StackOptParams,
     set.count(BenefitNodes, NumBenefitNodes),
     set.count(CostNodes, NumCostNodes),
     AllPathNodeRatio = StackOptParams ^ sop_all_path_node_ratio,
-    ( NumBenefitNodes * 100 >= NumCostNodes * AllPathNodeRatio ->
+    ( if NumBenefitNodes * 100 >= NumCostNodes * AllPathNodeRatio then
         ViaCellVars = ViaCellVars0
-    ;
+    else
         ViaCellVars = set_of_var.init
     ).
 
@@ -526,10 +531,11 @@ apply_matching(CellVar, CellVarFlushedLater, IntParams, StackOptParams,
 apply_matching_loop(CellVar, CellVarFlushedLater, IntParams, StackOptParams,
         PathInfos, CandidateArgVars0, BenefitNodeSets, CostNodeSets,
         ViaCellVars) :-
-    list.map3(apply_matching_for_path(CellVar, CellVarFlushedLater,
-        StackOptParams, CandidateArgVars0), PathInfos,
-        BenefitNodeSets0, CostNodeSets0, PathViaCellVars),
-    ( list.all_same(PathViaCellVars) ->
+    list.map3(
+        apply_matching_for_path(StackOptParams,
+            CellVar, CellVarFlushedLater, CandidateArgVars0),
+        PathInfos, BenefitNodeSets0, CostNodeSets0, PathViaCellVars),
+    ( if list.all_same(PathViaCellVars) then
         BenefitNodeSets = BenefitNodeSets0,
         CostNodeSets = CostNodeSets0,
         (
@@ -538,37 +544,39 @@ apply_matching_loop(CellVar, CellVarFlushedLater, IntParams, StackOptParams,
             PathViaCellVars = [],
             ViaCellVars = set_of_var.init
         )
-    ;
+    else
         CandidateArgVars1 = set_of_var.intersect_list(PathViaCellVars),
         FixpointLoop = StackOptParams ^ sop_fixpoint_loop,
         (
-            FixpointLoop = no,
+            FixpointLoop = do_not_opt_svcell_loop,
             BenefitNodeSets = BenefitNodeSets0,
             CostNodeSets = CostNodeSets0,
             ViaCellVars = CandidateArgVars1
         ;
-            FixpointLoop = yes,
+            FixpointLoop = opt_svcell_loop,
             apply_matching_loop(CellVar, CellVarFlushedLater,
                 IntParams, StackOptParams, PathInfos, CandidateArgVars1,
                 BenefitNodeSets, CostNodeSets, ViaCellVars)
         )
     ).
 
-:- pred apply_matching_for_path(prog_var::in, bool::in, stack_opt_params::in,
+:- pred apply_matching_for_path(stack_opt_params::in, prog_var::in, bool::in,
     set_of_progvar::in, match_path_info::in,
     set(benefit_node)::out, set(cost_node)::out, set_of_progvar::out) is det.
 
-apply_matching_for_path(CellVar, CellVarFlushedLater, StackOptParams,
+apply_matching_for_path(StackOptParams, CellVar, CellVarFlushedLater,
         CandidateArgVars, PathInfo, BenefitNodes, CostNodes, ViaCellVars) :-
-    ( set_of_var.is_empty(CandidateArgVars) ->
+    ( if set_of_var.is_empty(CandidateArgVars) then
         BenefitNodes = set.init,
         CostNodes = set.init,
         ViaCellVars = set_of_var.init
-    ;
+    else
         PathInfo = match_path_info(FirstSegment, LaterSegments),
+        Globals = StackOptParams ^ sop_globals,
+        ModuleName = StackOptParams ^ sop_module_name,
         MatchingParams = StackOptParams ^ sop_matching_params,
-        find_via_cell_vars(CellVar, CandidateArgVars, CellVarFlushedLater,
-            FirstSegment, LaterSegments, MatchingParams,
+        find_via_cell_vars(Globals, ModuleName, MatchingParams, CellVar,
+            CandidateArgVars, CellVarFlushedLater, FirstSegment, LaterSegments,
             BenefitNodes, CostNodes, ViaCellVars)
     ).
 
@@ -579,9 +587,9 @@ apply_matching_for_path(CellVar, CellVarFlushedLater, StackOptParams,
 
 record_matching_result(CellVar, ConsId, ArgVars, ViaCellVars, Goal,
         PotentialAnchors, PotentialIntervals, !IntervalInfo, !StackOptInfo) :-
-    ( set_of_var.is_empty(ViaCellVars) ->
+    ( if set_of_var.is_empty(ViaCellVars) then
         true
-    ;
+    else
         set.to_sorted_list(PotentialIntervals, PotentialIntervalList),
         set.to_sorted_list(PotentialAnchors, PotentialAnchorList),
         list.foldl3(record_cell_var_for_interval(CellVar, ViaCellVars),
@@ -610,9 +618,9 @@ record_cell_var_for_interval(CellVar, ViaCellVars, IntervalId,
         !IntervalInfo, !StackOptInfo, !InsertIntervals) :-
     record_interval_vars(IntervalId, [CellVar], !IntervalInfo),
     delete_interval_vars(IntervalId, ViaCellVars, DeletedVars, !IntervalInfo),
-    ( set_of_var.is_non_empty(DeletedVars) ->
+    ( if set_of_var.is_non_empty(DeletedVars) then
         set.insert(IntervalId, !InsertIntervals)
-    ;
+    else
         true
     ).
 
@@ -627,19 +635,19 @@ add_anchor_inserts(Goal, ArgVarsViaCellVar, InsertIntervals, Anchor,
     AnchorFollow = anchor_follow_info(_, AnchorIntervals),
     set.intersect(AnchorIntervals, InsertIntervals,
         AnchorInsertIntervals),
-    ( set.non_empty(AnchorInsertIntervals) ->
+    ( if set.is_non_empty(AnchorInsertIntervals) then
         Insert = insert_spec(Goal, ArgVarsViaCellVar),
         InsertMap0 = !.StackOptInfo ^ soi_left_anchor_inserts,
-        ( map.search(InsertMap0, Anchor, Inserts0) ->
+        ( if map.search(InsertMap0, Anchor, Inserts0) then
             Inserts = [Insert | Inserts0],
             map.det_update(Anchor, Inserts, InsertMap0, InsertMap)
-        ;
+        else
             Inserts = [Insert],
             map.det_insert(Anchor, Inserts, InsertMap0, InsertMap)
         ),
         !StackOptInfo ^ soi_left_anchor_inserts := InsertMap,
         set.insert(Anchor, !InsertAnchors)
-    ;
+    else
         true
     ).
 
@@ -691,16 +699,16 @@ close_path(Path0) = Path :-
         FlushAnchors, IntervalIds),
     (
         FlushState = current_is_before_first_flush,
-        expect(set_of_var.is_empty(FirstSegment0), $module, $pred,
+        expect(set_of_var.is_empty(FirstSegment0), $pred,
             "FirstSegment0 not empty"),
         FirstSegment = CurSegment,
         OtherSegments = OtherSegments0
     ;
         FlushState = current_is_after_first_flush,
-        ( set_of_var.is_empty(CurSegment) ->
+        ( if set_of_var.is_empty(CurSegment) then
             FirstSegment = FirstSegment0,
             OtherSegments = OtherSegments0
-        ;
+        else
             FirstSegment = FirstSegment0,
             OtherSegments = [CurSegment | OtherSegments0]
         )
@@ -711,9 +719,9 @@ close_path(Path0) = Path :-
 :- func add_interval_to_path(interval_id, set_of_progvar, path) = path.
 
 add_interval_to_path(IntervalId, Vars, !.Path) = !:Path :-
-    ( set_of_var.is_empty(Vars) ->
+    ( if set_of_var.is_empty(Vars) then
         true
-    ;
+    else
         CurSegment0 = !.Path ^ current_segment,
         CurSegment = set_of_var.union(Vars, CurSegment0),
         OccurringIntervals0 = !.Path ^ occurring_intervals,
@@ -829,28 +837,28 @@ find_all_branches(RelevantVars, IntervalId, MaybeSearchAnchor0,
     map.lookup(IntervalInfo ^ ii_interval_succ, IntervalId, SuccessorIds),
     (
         SuccessorIds = [],
-        expect(unify(may_have_no_successor(End), yes), $module, $pred,
+        expect(unify(may_have_no_successor(End), yes), $pred,
             "unexpected no successor")
-        % expect(unify(MaybeSearchAnchor0, no), $module, $pred,
+        % expect(unify(MaybeSearchAnchor0, no), $pred,
         %   "find_all_branches: no successor while in search"),
         % that test may fail if we come to a call that cannot succeed
     ;
         SuccessorIds = [SuccessorId | MoreSuccessorIds],
         (
             MoreSuccessorIds = [],
-            expect(unify(may_have_one_successor(End), yes), $module, $pred,
+            expect(unify(may_have_one_successor(End), yes), $pred,
                 "unexpected one successor")
         ;
             MoreSuccessorIds = [_ | _],
-            expect(unify(may_have_more_successors(End), yes), $module, $pred,
+            expect(unify(may_have_more_successors(End), yes), $pred,
                 "unexpected more successors")
         ),
-        (
+        ( if
             MaybeSearchAnchor0 = yes(SearchAnchor0),
             End = SearchAnchor0
-        ->
+        then
             !AllPaths ^ used_after_scope := set_of_var.init
-        ;
+        else if
             End = anchor_branch_end(_, EndGoalId),
             map.lookup(IntervalInfo ^ ii_branch_end_map, EndGoalId,
                 BranchEndInfo),
@@ -861,9 +869,9 @@ find_all_branches(RelevantVars, IntervalId, MaybeSearchAnchor0,
             RelevantAfter = set_of_var.intersect(RelevantVars,
                 NeededAfterBranch),
             set_of_var.is_non_empty(RelevantAfter)
-        ->
+        then
             !AllPaths ^ used_after_scope := RelevantAfter
-        ;
+        else
             find_all_branches_from(End, RelevantVars,
                 MaybeSearchAnchor0, IntervalInfo, StackOptInfo,
                 [SuccessorId | MoreSuccessorIds], !AllPaths)
@@ -887,10 +895,10 @@ find_all_branches_from(End, RelevantVars, MaybeSearchAnchor0, IntervalInfo,
     ),
     StackOptParams = StackOptInfo ^ soi_stack_opt_params,
     FullPath = StackOptParams ^ sop_full_path,
-    (
-        FullPath = yes,
+    ( if
+        FullPath = opt_svcell_full_path,
         End = anchor_branch_start(branch_disj, EndGoalId)
-    ->
+    then
         MaybeSearchAnchor1 = yes(anchor_branch_end(branch_disj, EndGoalId)),
         one_after_another(RelevantVars, MaybeSearchAnchor1,
             IntervalInfo, StackOptInfo, SuccessorIds, !AllPaths),
@@ -899,15 +907,15 @@ find_all_branches_from(End, RelevantVars, MaybeSearchAnchor0, IntervalInfo,
         ContinueId = BranchEndInfo ^ interval_after_branch,
         apply_interval_find_all_branches(RelevantVars, MaybeSearchAnchor0,
             IntervalInfo, StackOptInfo, ContinueId, !AllPaths)
-    ;
-        FullPath = yes,
+    else if
+        FullPath = opt_svcell_full_path,
         End = anchor_branch_start(branch_ite, EndGoalId)
-    ->
-        ( SuccessorIds = [ElseStartIdPrime, CondStartIdPrime] ->
+    then
+        ( if SuccessorIds = [ElseStartIdPrime, CondStartIdPrime] then
             ElseStartId = ElseStartIdPrime,
             CondStartId = CondStartIdPrime
-        ;
-            unexpected($module, $pred, "ite not else, cond")
+        else
+            unexpected($pred, "ite not else, cond")
         ),
         MaybeSearchAnchorCond = yes(anchor_cond_then(EndGoalId)),
         apply_interval_find_all_branches(RelevantVars,
@@ -923,9 +931,9 @@ find_all_branches_from(End, RelevantVars, MaybeSearchAnchor0, IntervalInfo,
         ContinueId = BranchEndInfo ^ interval_after_branch,
         apply_interval_find_all_branches(RelevantVars, MaybeSearchAnchor0,
             IntervalInfo, StackOptInfo, ContinueId, !AllPaths)
-    ;
+    else if
         End = anchor_branch_start(BranchType, EndGoalId)
-    ->
+    then
         MaybeSearchAnchor1 = yes(anchor_branch_end(BranchType, EndGoalId)),
         list.map(apply_interval_find_all_branches_map(RelevantVars,
             MaybeSearchAnchor1, IntervalInfo, StackOptInfo, !.AllPaths),
@@ -936,13 +944,13 @@ find_all_branches_from(End, RelevantVars, MaybeSearchAnchor0, IntervalInfo,
         ContinueId = BranchEndInfo ^ interval_after_branch,
         apply_interval_find_all_branches(RelevantVars, MaybeSearchAnchor0,
             IntervalInfo, StackOptInfo, ContinueId, !AllPaths)
-    ;
-        ( SuccessorIds = [SuccessorId] ->
+    else
+        ( if SuccessorIds = [SuccessorId] then
             apply_interval_find_all_branches(RelevantVars,
                 MaybeSearchAnchor0, IntervalInfo,
                 StackOptInfo, SuccessorId, !AllPaths)
-        ;
-            unexpected($module, $pred, "more successor ids")
+        else
+            unexpected($pred, "more successor ids")
         )
     ).
 
@@ -982,7 +990,7 @@ apply_interval_find_all_branches(RelevantVars, MaybeSearchAnchor0,
     Paths1 = set.map(add_interval_to_path(IntervalId, RelevantIntervalVars),
         Paths0),
     map.lookup(IntervalInfo ^ ii_interval_start, IntervalId, Start),
-    (
+    ( if
         % Check if intervals starting at Start use any RelevantVars.
         ( Start = anchor_call_site(_)
         ; Start = anchor_branch_end(_, _)
@@ -992,14 +1000,14 @@ apply_interval_find_all_branches(RelevantVars, MaybeSearchAnchor0,
         StartInfo = anchor_follow_info(AnchorFollowVars, _),
         set_of_var.intersect(RelevantVars, AnchorFollowVars, NeededVars),
         set_of_var.is_non_empty(NeededVars)
-    ->
+    then
         Paths2 = set.map(add_anchor_to_path(Start), Paths1)
-    ;
+    else
         Paths2 = Paths1
     ),
-    ( set.member(Start, IntervalInfo ^ ii_model_non_anchors) ->
+    ( if set.member(Start, IntervalInfo ^ ii_model_non_anchors) then
         AfterModelNon = yes
-    ;
+    else
         AfterModelNon = AfterModelNon0
     ),
     !:AllPaths = all_paths(Paths2, AfterModelNon, RelevantAfter),
@@ -1009,7 +1017,7 @@ apply_interval_find_all_branches(RelevantVars, MaybeSearchAnchor0,
 :- pred consolidate_after_join(list(all_paths)::in, all_paths::out) is det.
 
 consolidate_after_join([], _) :-
-    unexpected($module, $pred, "no paths to join").
+    unexpected($pred, "no paths to join").
 consolidate_after_join([First | Rest], AllPaths) :-
     PathsList = list.map(project_paths_from_all_paths, [First | Rest]),
     Paths0 = set.union_list(PathsList),
@@ -1043,16 +1051,18 @@ compress_paths(Paths) = Paths.
 
 maybe_write_progress_message(Message, DebugStackOpt, PredIdInt, ProcInfo,
         ModuleInfo, !IO) :-
-    ( DebugStackOpt = PredIdInt ->
-        io.write_string(Message, !IO),
-        io.write_string(":\n", !IO),
+    ( if DebugStackOpt = PredIdInt then
         proc_info_get_goal(ProcInfo, Goal),
         proc_info_get_varset(ProcInfo, VarSet),
         module_info_get_globals(ModuleInfo, Globals),
-        OutInfo = init_hlds_out_info(Globals),
-        write_goal(OutInfo, Goal, ModuleInfo, VarSet, yes, 0, "\n", !IO),
-        io.write_string("\n", !IO)
-    ;
+        io.output_stream(Stream, !IO),
+        io.write_string(Stream, Message, !IO),
+        io.write_string(Stream, ":\n", !IO),
+        OutInfo = init_hlds_out_info(Globals, output_debug),
+        write_goal(OutInfo, Stream, ModuleInfo, VarSet, print_name_and_num,
+            0, "\n", Goal, !IO),
+        io.write_string(Stream, "\n", !IO)
+    else
         true
     ).
 
@@ -1061,89 +1071,84 @@ maybe_write_progress_message(Message, DebugStackOpt, PredIdInt, ProcInfo,
     % This predicate (along with dump_interval_info) can help debug the
     % performance of the transformation.
     %
-:- pred dump_stack_opt_info(stack_opt_info::in, io::di, io::uo) is det.
-
-dump_stack_opt_info(StackOptInfo, !IO) :-
-    map.to_assoc_list(StackOptInfo ^ soi_left_anchor_inserts, Inserts),
-    io.write_string("\nANCHOR INSERT:\n", !IO),
-    list.foldl(dump_anchor_inserts, Inserts, !IO),
-
-    io.write_string("\nMATCHING RESULTS:\n", !IO),
-    list.foldl(dump_matching_result, StackOptInfo ^ soi_matching_results, !IO),
-    io.write_string("\n", !IO).
-
-:- pred dump_anchor_inserts(pair(anchor, list(insert_spec))::in,
+:- pred dump_stack_opt_info(io.text_output_stream::in, stack_opt_info::in,
     io::di, io::uo) is det.
 
-dump_anchor_inserts(Anchor - InsertSpecs, !IO) :-
-    io.write_string("\ninsertions after ", !IO),
-    io.write(Anchor, !IO),
-    io.write_string(":\n", !IO),
-    list.foldl(dump_insert, InsertSpecs, !IO).
+dump_stack_opt_info(Stream, StackOptInfo, !IO) :-
+    map.to_assoc_list(StackOptInfo ^ soi_left_anchor_inserts, Inserts),
+    io.write_string(Stream, "\nANCHOR INSERT:\n", !IO),
+    list.foldl(dump_anchor_inserts(Stream), Inserts, !IO),
 
-:- pred dump_insert(insert_spec::in, io::di, io::uo) is det.
+    io.write_string(Stream, "\nMATCHING RESULTS:\n", !IO),
+    list.foldl(dump_matching_result(Stream),
+        StackOptInfo ^ soi_matching_results, !IO),
+    io.write_string(Stream, "\n", !IO).
 
-dump_insert(insert_spec(Goal, Vars), !IO) :-
+:- pred dump_anchor_inserts(io.text_output_stream::in,
+    pair(anchor, list(insert_spec))::in, io::di, io::uo) is det.
+
+dump_anchor_inserts(Stream, Anchor - InsertSpecs, !IO) :-
+    io.write_string(Stream, "\ninsertions after ", !IO),
+    io.write(Stream, Anchor, !IO),
+    io.write_string(Stream, ":\n", !IO),
+    list.foldl(dump_insert(Stream), InsertSpecs, !IO).
+
+:- pred dump_insert(io.text_output_stream::in, insert_spec::in,
+    io::di, io::uo) is det.
+
+dump_insert(Stream, insert_spec(Goal, Vars), !IO) :-
     list.map(term.var_to_int, set_of_var.to_sorted_list(Vars), VarNums),
-    io.write_string("vars [", !IO),
-    write_int_list(VarNums, !IO),
-    io.write_string("]: ", !IO),
-    (
+    io.format(Stream, "vars [%s]:", [s(int_list_to_string(VarNums))], !IO),
+    ( if
         Goal = hlds_goal(unify(_, _, _, Unification, _), _),
         Unification = deconstruct(CellVar, ConsId, ArgVars, _,_,_)
-    ->
+    then
         term.var_to_int(CellVar, CellVarNum),
-        io.write_int(CellVarNum, !IO),
-        io.write_string(" => ", !IO),
-        write_cons_id_and_arity(ConsId, !IO),
-        io.write_string("(", !IO),
         list.map(term.var_to_int, ArgVars, ArgVarNums),
-        write_int_list(ArgVarNums, !IO),
-        io.write_string(")\n", !IO)
-    ;
-        io.write_string("BAD INSERT GOAL\n", !IO)
+        io.format(Stream, "%d => %s(%s)\n",
+            [i(CellVarNum), s(cons_id_and_arity_to_string(ConsId)),
+            s(int_list_to_string(ArgVarNums))], !IO)
+    else
+        io.write_string(Stream, "BAD INSERT GOAL\n", !IO)
     ).
 
-:- pred dump_matching_result(matching_result::in,
+:- pred dump_matching_result(io.text_output_stream::in, matching_result::in,
     io::di, io::uo) is det.
 
-dump_matching_result(MatchingResult, !IO) :-
+dump_matching_result(Stream, MatchingResult, !IO) :-
     MatchingResult = matching_result(CellVar, ConsId, ArgVars, ViaCellVars,
         GoalId, PotentialIntervals, InsertIntervals,
         PotentialAnchors, InsertAnchors),
-    io.write_string("\nmatching result at ", !IO),
-    io.write(GoalId, !IO),
-    io.write_string("\n", !IO),
+    io.write_string(Stream, "\nmatching result at ", !IO),
+    io.write_line(Stream, GoalId, !IO),
     term.var_to_int(CellVar, CellVarNum),
     list.map(term.var_to_int, ArgVars, ArgVarNums),
     list.map(term.var_to_int, set_of_var.to_sorted_list(ViaCellVars),
         ViaCellVarNums),
-    io.write_int(CellVarNum, !IO),
-    io.write_string(" => ", !IO),
-    write_cons_id_and_arity(ConsId, !IO),
-    io.write_string("(", !IO),
-    write_int_list(ArgVarNums, !IO),
-    io.write_string("): via cell ", !IO),
-    write_int_list(ViaCellVarNums, !IO),
-    io.write_string("\n", !IO),
+    io.format(Stream, "%d => %s(%s): via cell %s\n",
+        [i(CellVarNum), s(cons_id_and_arity_to_string(ConsId)),
+        s(int_list_to_string(ArgVarNums)),
+        s(int_list_to_string(ViaCellVarNums))], !IO),
 
-    io.write_string("potential intervals: ", !IO),
     PotentialIntervalNums = list.map(interval_id_to_int,
         set.to_sorted_list(PotentialIntervals)),
-    write_int_list(PotentialIntervalNums, !IO),
-    io.write_string("\n", !IO),
-    io.write_string("insert intervals: ", !IO),
     InsertIntervalNums = list.map(interval_id_to_int,
         set.to_sorted_list(InsertIntervals)),
-    write_int_list(InsertIntervalNums, !IO),
-    io.write_string("\n", !IO),
+    io.format(Stream, "potential intervals: %s\n",
+        [s(int_list_to_string(PotentialIntervalNums))], !IO),
+    io.format(Stream, "insert intervals: %s\n",
+        [s(int_list_to_string(InsertIntervalNums))], !IO),
 
-    io.write_string("potential anchors: ", !IO),
-    io.write_list(set.to_sorted_list(PotentialAnchors), " ", io.write, !IO),
-    io.write_string("\n", !IO),
-    io.write_string("insert anchors: ", !IO),
-    io.write_list(set.to_sorted_list(InsertAnchors), " ", io.write, !IO),
-    io.write_string("\n", !IO).
+    PotentialAnchorStrs = list.map(string.string,
+        set.to_sorted_list(PotentialAnchors)),
+    InsertAnchorStrs = list.map(string.string,
+        set.to_sorted_list(InsertAnchors)),
+    PotentialAnchorsStr = string.join_list(" ", PotentialAnchorStrs),
+    InsertAnchorsStr = string.join_list(" ", InsertAnchorStrs),
+    io.format(Stream, "potential anchors: %s\n",
+        [s(PotentialAnchorsStr)], !IO),
+    io.format(Stream, "insert anchors: %s\n",
+        [s(InsertAnchorsStr)], !IO).
 
 %-----------------------------------------------------------------------------%
 :- end_module ll_backend.stack_opt.
