@@ -12,7 +12,7 @@
 % This module analyses a SCC of the call-graph and tries to prove that
 % it terminates.
 %
-% XXX This version is just a place-holder.  It attempts a very simple
+% XXX This version is just a place-holder. It attempts a very simple
 % proof method which is essentially what the existing termination analyser
 % does.
 %
@@ -21,12 +21,10 @@
 :- module transform_hlds.term_constr_pass2.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
-:- import_module transform_hlds.term_constr_main.
-
-:- import_module io.
-:- import_module list.
+:- import_module transform_hlds.term_constr_main_types.
 
 %-----------------------------------------------------------------------------%
 
@@ -35,28 +33,28 @@
 :- type pass2_options.
 
     % pass2_options_init(MaxMatrixSize).
-    % Initialise the pass2_options structure.  `MaxMatrixSize' specifies
+    % Initialise the pass2_options structure. `MaxMatrixSize' specifies
     % the maximum number of constraints we allow a matrix to grow to
     % before we abort and try other approximations.
     %
 :- func pass2_options_init(int) = pass2_options.
 
-:- pred prove_termination_in_scc(pass2_options::in, list(pred_proc_id)::in,
-    module_info::in, constr_termination_info::out, io::di, io::uo) is det.
+:- pred prove_termination_in_scc(pass2_options::in, scc::in,
+    module_info::in, constr_termination_info::out) is det.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
-:- import_module hlds.hlds_module.
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_util.
-:- import_module hlds.hlds_pred.
+:- import_module libs.
 :- import_module libs.lp_rational.
 :- import_module libs.polyhedron.
 :- import_module libs.rat.
-:- import_module parse_tree.prog_data.
+:- import_module parse_tree.
+:- import_module parse_tree.prog_data_pragma.
 :- import_module transform_hlds.term_constr_data.
 :- import_module transform_hlds.term_constr_errors.
 :- import_module transform_hlds.term_constr_util.
@@ -65,11 +63,14 @@
 :- import_module bimap.
 :- import_module bool.
 :- import_module int.
+:- import_module io.
+:- import_module list.
 :- import_module map.
 :- import_module maybe.
 :- import_module pair.
 :- import_module require.
 :- import_module set.
+:- import_module string.
 :- import_module term.
 :- import_module varset.
 
@@ -87,7 +88,7 @@ pass2_options_init(MaxSize) = pass2_options(MaxSize).
 
 %-----------------------------------------------------------------------------%
 
-:- type scc == list(abstract_ppid).
+:- type abstract_ppids == list(abstract_ppid).
 
     % Each edge in the call-graph represents a single call site.
     %
@@ -130,9 +131,9 @@ pass2_options_init(MaxSize) = pass2_options(MaxSize).
 
 :- type cycles == list(cycle).
 
-    % A c_cycle, or collapsed cycle, is an elmentary cycle from the
-    % call-graph where we have picked a starting vertex and travelled
-    % around the cycle conjoining all the labels (constraints) as we go.
+    % A c_cycle, or collapsed cycle, is an elementary cycle from the call-graph
+    % where we have picked a starting vertex and travelled around the cycle
+    % conjoining all the labels (constraints) as we go.
     %
 :- type cycle_set
     --->    term_cg_cycle_set(
@@ -142,20 +143,25 @@ pass2_options_init(MaxSize) = pass2_options(MaxSize).
 
 %-----------------------------------------------------------------------------%
 
-prove_termination_in_scc(_, [], _, cannot_loop(term_reason_analysis), !IO).
-prove_termination_in_scc(Options, SCC0 @ [_ | _], ModuleInfo, Result, !IO) :-
-    AbstractSCC = get_abstract_scc(ModuleInfo, SCC0),
-    % XXX Pass 1 should really set this up.
-    SCC = list.map((func(A) = real(A)), SCC0),
-    ( scc_contains_recursion(AbstractSCC) ->
-        SizeVarSet = size_varset_from_abstract_scc(AbstractSCC),
-        Edges  = label_edges_in_scc(AbstractSCC, ModuleInfo,
-            Options ^ max_matrix_size),
-        Cycles    = find_elementary_cycles_in_scc(SCC, Edges),
-        CycleSets = partition_cycles(SCC, Cycles),
-        prove_termination(CycleSets, AbstractSCC, SizeVarSet, Result)
-    ;
+prove_termination_in_scc(Options, SCC0, ModuleInfo, Result) :-
+    ( if set.is_empty(SCC0) then
         Result = cannot_loop(term_reason_analysis)
+    else
+        AbstractSCC = get_abstract_scc(ModuleInfo, SCC0),
+        ( if scc_contains_recursion(AbstractSCC) then
+            % XXX Pass 1 should really set this up.
+            PPIdSCC = set.map((func(A) = real(A)), SCC0),
+            set.to_sorted_list(PPIdSCC, PPIds),
+
+            SizeVarSet = size_varset_from_abstract_scc(AbstractSCC),
+            Edges = label_edges_in_scc(AbstractSCC, ModuleInfo,
+                Options ^ max_matrix_size),
+            Cycles = find_elementary_cycles_in_scc(PPIds, Edges),
+            CycleSets = partition_cycles(PPIds, Cycles),
+            prove_termination(CycleSets, AbstractSCC, SizeVarSet, Result)
+        else
+            Result = cannot_loop(term_reason_analysis)
+        )
     ).
 
 %-----------------------------------------------------------------------------%
@@ -164,22 +170,23 @@ prove_termination_in_scc(Options, SCC0 @ [_ | _], ModuleInfo, Result, !IO) :-
 %
 
 % Work out what the constraints are between each procedure head and each
-% call for every call in the SCC.  This information is implicit in the
+% call for every call in the SCC. This information is implicit in the
 % AR, so we traverse the AR building up a list of labelled edges as
 % we go - this is similar to the fixpoint calculation we performed in pass 1
-% except that we can stop after we have examined the last call.  This often
+% except that we can stop after we have examined the last call. This often
 % means that we can avoid performing unnecessary convex hull operations.
 
 :- func label_edges_in_scc(abstract_scc, module_info, int) = edges.
 
-label_edges_in_scc(Procs, ModuleInfo, MaxMatrixSize) = Edges :-
-    FindEdges = (pred(Proc::in, !.Edges::in, !:Edges::out) is det :-
-        find_edges_in_goal(Proc, Procs, ModuleInfo, MaxMatrixSize,
-            Proc ^ ap_body, 1, _, polyhedron.universe, _, [],
-            ProcEdges, yes, _),
-        list.append(ProcEdges, !Edges)
-    ),
-    list.foldl(FindEdges, Procs, [], Edges).
+label_edges_in_scc(AbstractSCC, ModuleInfo, MaxMatrixSize) = Edges :-
+    FindEdges =
+        ( pred(Proc::in, !.Edges::in, !:Edges::out) is det :-
+            find_edges_in_goal(Proc, AbstractSCC, ModuleInfo, MaxMatrixSize,
+                Proc ^ ap_body, 1, _, polyhedron.universe, _, [],
+                ProcEdges, yes, _),
+            list.append(ProcEdges, !Edges)
+        ),
+    set.foldl(FindEdges, AbstractSCC, [], Edges).
 
     % The four accumulators here are for:
     % (1) the number of calls seen so far
@@ -200,7 +207,7 @@ find_edges_in_goal(Proc, AbstractSCC, ModuleInfo, MaxMatrixSize,
             !.Continue = yes,
 
             % XXX We may be able to prove termination in more cases if we pass
-            % it !.Polyhedron instead of of polyhedron.universe ... although
+            % it !.Polyhedron instead of polyhedron.universe ... although
             % I don't think it is a major concern at the moment.
             find_edges_in_disj(Proc, AbstractSCC, ModuleInfo,
                 MaxMatrixSize, polyhedron.universe, Goals, !Calls,
@@ -231,8 +238,9 @@ find_edges_in_goal(Proc, AbstractSCC, ModuleInfo, MaxMatrixSize,
                     MaxMatrixSize),
                 Goals, !Calls, !Polyhedron, !Edges, !Continue),
             (
-                !.Continue = yes, polyhedron.project(Locals,
-                    Proc ^ ap_size_varset, !Polyhedron)
+                !.Continue = yes,
+                polyhedron.project_polyhedron(Proc ^ ap_size_varset, Locals,
+                    !Polyhedron)
             ;
                 !.Continue = no
             )
@@ -241,7 +249,7 @@ find_edges_in_goal(Proc, AbstractSCC, ModuleInfo, MaxMatrixSize,
         )
     ;
         Goal = term_call(CallPPId0, _, CallVars, ZeroVars, _, _, _),
-        % Having found a call we now need to construct a label for that edge
+        % Having found a call, we now need to construct a label for that edge
         % and then continue looking for more edges.
         Edge = term_cg_edge(Proc ^ ap_ppid, CallPPId0,
             Proc ^ ap_head_vars, CallVars, Proc ^ ap_zeros, !.Polyhedron),
@@ -250,9 +258,9 @@ find_edges_in_goal(Proc, AbstractSCC, ModuleInfo, MaxMatrixSize,
         % Update the call count and maybe stop processing
         % if that was the last call.
         !:Calls = !.Calls + 1,
-        ( !.Calls > Proc ^ ap_num_calls ->
+        ( if !.Calls > Proc ^ ap_num_calls then
             !:Continue = no
-        ;
+        else
             true
         ),
         (
@@ -261,26 +269,25 @@ find_edges_in_goal(Proc, AbstractSCC, ModuleInfo, MaxMatrixSize,
             !.Continue = yes,
             CallPPId0 = real(CallPPId),
             module_info_pred_proc_info(ModuleInfo, CallPPId,  _, CallProcInfo),
-            proc_info_get_termination2_info(CallProcInfo, CallTermInfo),
-            MaybeArgSizeInfo = CallTermInfo ^ success_constrs,
+            proc_info_get_termination2_info(CallProcInfo, CallTerm2Info),
+            MaybeArgSizeInfo = term2_info_get_success_constrs(CallTerm2Info),
             (
                 MaybeArgSizeInfo = no,
-                unexpected($module, $pred,
-                    "proc with no arg size info in pass 2")
+                unexpected($pred, "proc with no arg size info in pass 2")
             ;
                 MaybeArgSizeInfo = yes(ArgSizePolyhedron0),
-                ( polyhedron.is_universe(ArgSizePolyhedron0) ->
-                    % If the polyhedron is universe then there is no point
+                ( if polyhedron.is_universe(ArgSizePolyhedron0) then
+                    % If the polyhedron is universe, then there is no point
                     % in running the substitution.
                     true
-                ;
-                    MaybeCallProc = CallTermInfo ^ abstract_rep,
+                else
+                    MaybeCallProc = term2_info_get_abstract_rep(CallTerm2Info),
                     (
                         MaybeCallProc = yes(CallProc0),
                         CallProc = CallProc0
                     ;
                         MaybeCallProc = no,
-                        unexpected($module, $pred,
+                        unexpected($pred,
                             "no abstract representation for proc")
                     ),
                     HeadVars = CallProc ^ ap_head_vars,
@@ -339,10 +346,10 @@ fix_edges(Poly, Edge0) = Edge :-
 % Cycle detection.
 %
 
-% To find the elementary cycles of this SCC we perform a DFS of the
-% call-graph.  Since the call-graph is technically a pseudograph (ie. it
-% admits parallel edges and self-loops), we first of all strip out any
-% self-loops to make things easier.
+% To find the elementary cycles of this SCC we perform a DFS of the call-graph.
+% Since the call-graph is technically a pseudograph (i.e. it admits parallel
+% edges and self-loops), we first of all strip out any self-loops
+% to make things easier.
 
 :- func find_elementary_cycles_in_scc(list(abstract_ppid), edges) = cycles.
 
@@ -379,24 +386,25 @@ partition_edges([], _) = map.init.
 partition_edges([ProcId | SCC], Edges0) = Map :-
     Map0 = partition_edges(SCC, Edges0),
     Edges = list.filter(
-        (pred(Edge::in) is semidet :- ProcId = Edge ^ tcge_caller),
-        Edges0),
-    Map = map.det_insert(Map0, ProcId, Edges).
+        ( pred(Edge::in) is semidet :-
+            ProcId = Edge ^ tcge_caller
+        ), Edges0),
+    map.det_insert(ProcId, Edges, Map0, Map).
 
 :- func search_for_cycles(list(abstract_ppid), map(abstract_ppid, edges))
     = cycles.
 
 search_for_cycles([], _) = [].
-search_for_cycles([Start | Rest], Map0) = Cycles :-
-    Cycles0 = search_for_cycles_2(Start, Map0),
-    Map = map.delete(Map0, Start),
-    Cycles1 = search_for_cycles(Rest, Map),
-    Cycles = Cycles0 ++ Cycles1.
+search_for_cycles([HeadPPId | TailPPId], Map0) = Cycles :-
+    HeadCycles = search_for_cycles_2(HeadPPId, Map0),
+    map.delete(HeadPPId, Map0, Map1),
+    TailCycles = search_for_cycles(TailPPId, Map1),
+    Cycles = HeadCycles ++ TailCycles.
 
 :- func search_for_cycles_2(abstract_ppid, map(abstract_ppid, edges)) = cycles.
 
 search_for_cycles_2(StartPPId, Map) = Cycles :-
-    InitialEdges = Map ^ det_elem(StartPPId),
+    map.lookup(Map, StartPPId, InitialEdges),
     list.foldl(search_for_cycles_3(StartPPId, [], Map, []), InitialEdges,
         [], Cycles).
 
@@ -405,20 +413,21 @@ search_for_cycles_2(StartPPId, Map) = Cycles :-
     cycles::in, cycles::out) is det.
 
 search_for_cycles_3(Start, SoFar, Map, Visited, Edge, !Cycles) :-
-    ( Start = Edge ^ tcge_callee ->
+    ( if Start = Edge ^ tcge_callee then
         Cycle = term_cg_cycle([Edge ^ tcge_caller | Visited], [Edge | SoFar]),
         list.cons(Cycle, !Cycles)
-    ;
-        ( MoreEdges0 = Map ^ elem(Edge ^ tcge_callee) ->
-            NotVisited = (pred(E::in) is semidet :-
-                not list.member(E ^ tcge_caller, Visited)
-            ),
+    else
+        ( if map.search(Map, Edge ^ tcge_callee, MoreEdges0) then
+            NotVisited =
+                ( pred(E::in) is semidet :-
+                    not list.member(E ^ tcge_caller, Visited)
+                ),
             MoreEdges = list.filter(NotVisited, MoreEdges0),
             list.foldl(
                 search_for_cycles_3(Start, [Edge | SoFar], Map,
                     [Edge ^ tcge_caller | Visited]),
                 MoreEdges, !Cycles)
-        ;
+        else
             true
         )
     ).
@@ -428,13 +437,13 @@ search_for_cycles_3(Start, SoFar, Map, Visited, Edge, !Cycles) :-
 % Partitioning sets of cycles.
 %
 
-:- func partition_cycles(scc, cycles) = list(cycle_set).
+:- func partition_cycles(abstract_ppids, cycles) = list(cycle_set).
 
 partition_cycles([], _) = [].
 partition_cycles([Proc | Procs], Cycles0) = CycleSets :-
     list.filter(cycle_contains_proc(Proc), Cycles0, PCycles, Cycles1),
     CycleSets0 = partition_cycles(Procs, Cycles1),
-    PEdges = list.map(collapse_cycle(Proc), PCycles),
+    PEdges = collapse_cycles(Proc, PCycles),
     (
         PEdges = [],
         CycleSets = CycleSets0
@@ -447,11 +456,11 @@ partition_cycles([Proc | Procs], Cycles0) = CycleSets :-
     = abstract_proc.
 
 get_proc_from_abstract_scc([], _) = _ :-
-    unexpected($module, $pred, "cannot find proc").
+    unexpected($pred, "cannot find proc").
 get_proc_from_abstract_scc([Proc | Procs], PPId) =
-    ( Proc ^ ap_ppid = PPId ->
+    ( if Proc ^ ap_ppid = PPId then
         Proc
-    ;
+    else
         get_proc_from_abstract_scc(Procs, PPId)
     ).
 
@@ -460,19 +469,19 @@ get_proc_from_abstract_scc([Proc | Procs], PPId) =
 % Termination checking.
 %
 
-% This approach is very crude.  It just checks that the sum of all
+% This approach is very crude. It just checks that the sum of all
 % the non-zero arguments is decreasing around all the elementary cycles.
 
 :- pred prove_termination(list(cycle_set)::in, abstract_scc::in,
     size_varset::in, constr_termination_info::out) is det.
 
 prove_termination(Cycles, AbstractSCC, SizeVarSet, Result) :-
-    ( total_sum_decrease(AbstractSCC, SizeVarSet, Cycles) ->
+    ( if total_sum_decrease(AbstractSCC, SizeVarSet, Cycles) then
         Result = cannot_loop(term_reason_analysis)
-    ;
+    else
         % NOTE: The context here will never be used, in any case
         % it is not clear what it should be.
-        Error = term.context_init - cond_not_satisfied,
+        Error = term2_error(term.context_init, cond_not_satisfied),
         Result = can_loop([Error])
     ).
 
@@ -503,19 +512,20 @@ total_sum_decrease_2(AbstractSCC, SizeVarSet, PPId, Loops @ [_ | _]) :-
     abstract_ppid::in, edge::in) is semidet.
 
 strict_decrease_around_loop(AbstractSCC, SizeVarSet, PPId, Loop) :-
-    (
+    ( if
         ( PPId \= Loop ^ tcge_caller
         ; PPId \= Loop ^ tcge_callee
         )
-    ->
-        unexpected($module, $pred, "badly formed loop")
-    ;
+    then
+        unexpected($pred, "badly formed loop")
+    else
         true
     ),
-    IsActive = (func(Var::in, Input::in) = (Var::out) is semidet :-
-        Input = yes
-    ),
-    Proc = get_proc_from_abstract_scc(AbstractSCC, PPId),
+    IsActive =
+        ( func(Var::in, Input::in) = (Var::out) is semidet :-
+            Input = yes
+        ),
+    Proc = get_proc_from_abstract_scc(set.to_sorted_list(AbstractSCC), PPId),
     Inputs = Proc ^ ap_inputs,
     HeadArgs = list.filter_map_corresponding(IsActive, Loop ^ tcge_head_args,
         Inputs),
@@ -555,7 +565,7 @@ collapse_cycle(StartPPId, Cycle) = CollapsedCycle :-
     Cycle = term_cg_cycle(_, Edges0),
     (
         Edges0 = [],
-        unexpected($module, $pred, "trying to collapse a cycle with no edges")
+        unexpected($pred, "trying to collapse a cycle with no edges")
     ;
         Edges0 = [Edge],
         CollapsedCycle = Edge
@@ -572,7 +582,7 @@ collapse_cycle(StartPPId, Cycle) = CollapsedCycle :-
                 HeadVars, CallVars, Zeros, Polyhedron)
         ;
             Edges = [],
-            unexpected($module, $pred, "error while collapsing cycles")
+            unexpected($pred, "error while collapsing cycles")
         )
     ).
 
@@ -608,16 +618,16 @@ collapse_cycle_2([Edge | Edges], !Zeros, !CallVars, !Polyhedron) :-
 
 order_nodes(StartPPId, Edges0, [Edge | Edges]) :-
     EdgeMap = build_edge_map(Edges0),
-    Edge = EdgeMap ^ det_elem(StartPPId),
+    map.lookup(EdgeMap, StartPPId, Edge),
     order_nodes_2(StartPPId, Edge ^ tcge_callee, EdgeMap, Edges).
 
 :- pred order_nodes_2(abstract_ppid::in, abstract_ppid::in,
     map(abstract_ppid, edge)::in, edges::out) is det.
 
 order_nodes_2(StartPPId, CurrPPId, Map, Edges) :-
-    ( StartPPId = CurrPPId ->
+    ( if StartPPId = CurrPPId then
         Edges = []
-    ;
+    else
         map.lookup(Map, CurrPPId, Edge),
         order_nodes_2(StartPPId, Edge ^ tcge_callee, Map, Edges0),
         Edges = [Edge | Edges0]
@@ -651,48 +661,45 @@ subst_size_var(Map, Old) = (if bimap.search(Map, Old, New) then New else Old).
 % Predicates for printing out debugging traces.
 %
 
-:- pred write_cycles(cycles::in, module_info::in, size_varset::in,
-    io::di, io::uo) is det.
+:- pred write_cycles(io.text_output_stream::in, module_info::in,
+    size_varset::in, cycles::in, io::di, io::uo) is det.
+:- pragma consider_used(pred(write_cycles/6)).
 
-write_cycles([], _, _, !IO).
-write_cycles([Cycle | Cycles], ModuleInfo, SizeVarSet, !IO) :-
-    io.write_string("Cycle in SCC:\n", !IO),
-    write_cycle(Cycle ^ tcgc_nodes, ModuleInfo, !IO),
-    io.write_list(Cycle ^ tcgc_edges, "\n",
-        write_edge(ModuleInfo, SizeVarSet), !IO),
-    io.nl(!IO),
-    write_cycles(Cycles, ModuleInfo, SizeVarSet, !IO).
+write_cycles(_, _, _, [], !IO).
+write_cycles(Stream, ModuleInfo, SizeVarSet, [Cycle | Cycles], !IO) :-
+    io.write_string(Stream, "Cycle in SCC:\n", !IO),
+    write_cycle(Stream, ModuleInfo, Cycle ^ tcgc_nodes, !IO),
+    list.foldl(write_edge(Stream, ModuleInfo, SizeVarSet),
+        Cycle ^ tcgc_edges, !IO),
+    io.nl(Stream, !IO),
+    write_cycles(Stream, ModuleInfo, SizeVarSet, Cycles, !IO).
 
-:- pred write_cycle(list(abstract_ppid)::in, module_info::in, io::di, io::uo)
-    is det.
+:- pred write_cycle(io.text_output_stream::in, module_info::in,
+    list(abstract_ppid)::in, io::di, io::uo) is det.
 
-write_cycle([], _, !IO).
-write_cycle([Proc | Procs], ModuleInfo, !IO) :-
-    io.write_string("\t- ", !IO),
+write_cycle(_, _, [], !IO).
+write_cycle(Stream, ModuleInfo, [Proc | Procs], !IO) :-
     Proc = real(PredProcId),
-    write_pred_proc_id(ModuleInfo, PredProcId, !IO),
-    io.nl(!IO),
-    write_cycle(Procs, ModuleInfo, !IO).
+    io.format(Stream, "\t- %s\n",
+        [s(pred_proc_id_to_string(ModuleInfo, PredProcId))], !IO),
+    write_cycle(Stream, ModuleInfo, Procs, !IO).
 
-:- pred write_edge(module_info::in, size_varset::in, edge::in,
-    io::di, io::uo) is det.
+:- pred write_edge(io.text_output_stream::in, module_info::in,
+    size_varset::in, edge::in, io::di, io::uo) is det.
 
-write_edge(ModuleInfo, SizeVarSet, Edge, !IO) :-
-    io.write_string("Edge is:\n\tHead: ", !IO),
+write_edge(Stream, ModuleInfo, SizeVarSet, Edge, !IO) :-
     Edge ^ tcge_caller = real(PredProcId),
-    write_pred_proc_id(ModuleInfo, PredProcId, !IO),
-    io.write_string(" : ", !IO),
-    write_size_vars(SizeVarSet, Edge ^ tcge_head_args, !IO),
-    io.write_string(" :- \n", !IO),
-    io.write_string("\tConstraints are:  \n", !IO),
-    write_polyhedron(Edge ^ tcge_label, SizeVarSet, !IO),
-    io.write_string("\n\tCall is:  ", !IO),
     Edge ^ tcge_callee = real(CallPredProcId),
-    write_pred_proc_id(ModuleInfo, CallPredProcId, !IO),
-    io.write_string(" : ", !IO),
-    write_size_vars(SizeVarSet, Edge ^ tcge_call_args, !IO),
-    io.write_string(" :- \n", !IO),
-    io.nl(!IO).
+    io.format(Stream, "Edge is:\n\tHead: %s : ",
+        [s(pred_proc_id_to_string(ModuleInfo, PredProcId))], !IO),
+    write_size_vars(Stream, SizeVarSet, Edge ^ tcge_head_args, !IO),
+    io.write_string(Stream, " :- \n", !IO),
+    io.write_string(Stream, "\tConstraints are:  \n", !IO),
+    write_polyhedron(Stream, SizeVarSet, Edge ^ tcge_label, !IO),
+    io.format(Stream, "\n\tCall is:  %s : ",
+        [s(pred_proc_id_to_string(ModuleInfo, CallPredProcId))], !IO),
+    write_size_vars(Stream, SizeVarSet, Edge ^ tcge_call_args, !IO),
+    io.write_string(Stream, " :- \n\n", !IO).
 
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.term_constr_pass2.
