@@ -16,9 +16,12 @@
 :- module transform_hlds.term_norm.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
+:- import_module libs.
 :- import_module libs.globals.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module list.
@@ -42,21 +45,21 @@
     % NOTE: the list of arguments and the list of modes must be the same
     % length. They must also *not* contain any typeinfo related arguments as
     % this may cause an exception to be thrown when using the
-    % `--num-data-elems' norm.  (This is because the weight table doesn't keep
-    % track of typeinfo related variables - it used to but intervening
-    % compiler passes tend to do things to the code in the mean time so the
-    % whole lot becomes inconsistent - in the end it's just easier to ignore
+    % `--num-data-elems' norm. (This is because the weight table does not
+    % keep track of typeinfo related variables - it used to, but intervening
+    % compiler passes tend to do things to the code in the mean time, so the
+    % whole lot becomes inconsistent - in the end it is just easier to ignore
     % them).
     %
 :- pred functor_norm(module_info::in, functor_info::in, type_ctor::in,
     cons_id::in, int::out, list(prog_var)::in, list(prog_var)::out,
-    list(uni_mode)::in, list(uni_mode)::out) is det.
+    list(unify_mode)::in, list(unify_mode)::out) is det.
 
-    % This function computes a lower bound on the weight of a fuctor.  If the
-    % lower bound is zero then the weight of that functor is also zero.  If
-    % the lower bound is non-zero then there may be no upper bound on the size
-    % of the functor.  (And if there were this function wouldn't tell you about
-    % it anyhow).
+    % This function computes a lower bound on the weight of a functor.
+    % If the lower bound is zero then the weight of that functor is also zero.
+    % If the lower bound is non-zero then there may be no upper bound
+    % on the size of the functor. (And if there were, this function
+    % would not tell you about it anyhow).
     %
 :- func functor_lower_bound(module_info, functor_info, type_ctor, cons_id)
     = int.
@@ -70,6 +73,7 @@
 
 :- implementation.
 
+:- import_module check_hlds.
 :- import_module check_hlds.type_util.
 :- import_module hlds.const_struct.
 :- import_module hlds.hlds_data.
@@ -78,6 +82,7 @@
 :- import_module bool.
 :- import_module int.
 :- import_module map.
+:- import_module one_or_more.
 :- import_module pair.
 :- import_module require.
 
@@ -90,7 +95,7 @@
 % where i is an element of a set I, and I is a subset of {1, ... n}
 %
 % We currently support four kinds of semilinear norms.
-% XXX Actually we currently only use three of them.  `use_map/1' is unused.
+% XXX Actually we currently only use three of them. `use_map/1' is unused.
 
 :- type functor_info
     --->    simple
@@ -147,10 +152,10 @@ find_weights(ModuleInfo, Weights) :-
 find_weights_for_type(TypeCtor - TypeDefn, !Weights) :-
     hlds_data.get_type_defn_body(TypeDefn, TypeBody),
     (
-        TypeBody = hlds_du_type(Constructors, _, _, _, _, _, _, _, _),
+        TypeBody = hlds_du_type(type_body_du(Constructors, _, _, _, _)),
         hlds_data.get_type_defn_tparams(TypeDefn, TypeParams),
         list.foldl(find_weights_for_cons(TypeCtor, TypeParams),
-            Constructors, !Weights)
+            one_or_more_to_list(Constructors), !Weights)
     ;
         % This type does not introduce any functors.
         TypeBody = hlds_eqv_type(_)
@@ -163,32 +168,31 @@ find_weights_for_type(TypeCtor - TypeDefn, !Weights) :-
         TypeBody = hlds_foreign_type(_)
     ;
         % This type does not introduce any functors.
-        TypeBody = hlds_solver_type(_, _)
+        TypeBody = hlds_solver_type(_)
     ).
 
 :- pred find_weights_for_cons(type_ctor::in, list(type_param)::in,
     constructor::in, weight_table::in, weight_table::out) is det.
 
-% For existentially typed data items the compiler may insert some typeinfo
-% related arguments into the functor.  We ignore these arguments when
-% calculating the weight of a functor and we do not include them in the list
-% of counted arguments.
-
 find_weights_for_cons(TypeCtor, Params, Ctor, !Weights) :-
-    Ctor = ctor(_ExistQVars, _Constraints, SymName, Args, _),
-    list.length(Args, Arity),
-    ( Arity > 0 ->
+    % For existentially typed data items the compiler may insert some
+    % typeinfo related arguments into the functor. We ignore these arguments
+    % when calculating the weight of a functor and we do not include them
+    % in the list of counted arguments.
+    Ctor = ctor(_Ordinal, _MaybeExistConstraints, SymName, Args, Arity,
+        _Context),
+    ( if Arity > 0 then
         find_and_count_nonrec_args(Args, TypeCtor, Params,
             NumNonRec, ArgInfos0),
-        ( NumNonRec = 0 ->
+        ( if NumNonRec = 0 then
             Weight = 1,
             list.duplicate(Arity, yes, ArgInfos)
-        ;
+        else
             Weight = NumNonRec,
             ArgInfos = ArgInfos0
         ),
         WeightInfo = weight(Weight, ArgInfos)
-    ;
+    else
         WeightInfo = weight(0, [])
     ),
     ConsId = cons(SymName, Arity, TypeCtor),
@@ -208,10 +212,10 @@ find_weights_for_tuple(Arity, weight(Weight, ArgInfos)) :-
 find_and_count_nonrec_args([], _, _, 0, []).
 find_and_count_nonrec_args([Arg | Args], Id, Params, NonRecArgs, ArgInfo) :-
     find_and_count_nonrec_args(Args, Id, Params, NonRecArgs0, ArgInfo0),
-    ( is_arg_recursive(Arg, Id, Params) ->
+    ( if is_arg_recursive(Arg, Id, Params) then
         NonRecArgs = NonRecArgs0,
         ArgInfo = [yes | ArgInfo0]
-    ;
+    else
         NonRecArgs = NonRecArgs0 + 1,
         ArgInfo = [no | ArgInfo0]
     ).
@@ -230,12 +234,12 @@ is_arg_recursive(Arg, TypeCtor, Params) :-
     weight_info::out) is semidet.
 
 search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) :-
-    ( map.search(WeightMap, TypeCtor - ConsId, WeightInfo0) ->
+    ( if map.search(WeightMap, TypeCtor - ConsId, WeightInfo0) then
         WeightInfo = WeightInfo0
-    ; type_ctor_is_tuple(TypeCtor) ->
+    else if type_ctor_is_tuple(TypeCtor) then
         TypeCtor = type_ctor(_, Arity),
         find_weights_for_tuple(Arity, WeightInfo)
-    ;
+    else
         fail
     ).
 
@@ -256,50 +260,50 @@ functor_norm(ModuleInfo, FunctorInfo, TypeCtor, ConsId, Gamma,
         !Args, !Modes) :-
     (
         FunctorInfo = simple,
-        (
+        ( if
             ConsId = cons(_, Arity, _),
             Arity \= 0
-        ->
+        then
             Gamma = 1
-        ;
+        else if
             ConsId = ground_term_const(ConstNum, _)
-        ->
+        then
             module_info_get_const_struct_db(ModuleInfo, ConstStructDb),
             const_struct_count_cells(ConstStructDb, ConstNum, 0, Gamma)
-        ;
+        else
             Gamma = 0
         )
     ;
         FunctorInfo = total,
-        ( ConsId = cons(_, Arity, _) ->
+        ( if ConsId = cons(_, Arity, _) then
             Gamma = Arity
-        ; ConsId = ground_term_const(ConstNum, _) ->
+        else if ConsId = ground_term_const(ConstNum, _) then
             module_info_get_const_struct_db(ModuleInfo, ConstStructDb),
             const_struct_count_cell_arities(ConstStructDb, ConstNum, 0, Gamma)
-        ;
+        else
             Gamma = 0
         )
     ;
         FunctorInfo = use_map(WeightMap),
-        ( search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) ->
+        ( if search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) then
             WeightInfo = weight(Gamma, _)
-        ; ConsId = ground_term_const(ConstNum, _) ->
+        else if ConsId = ground_term_const(ConstNum, _) then
             module_info_get_const_struct_db(ModuleInfo, ConstStructDb),
             const_struct_count_cell_weights(ConstStructDb, WeightMap,
                 ConstNum, 0, Gamma)
-        ;
+        else
             Gamma = 0
         )
     ;
         FunctorInfo = use_map_and_args(WeightMap),
-        ( search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) ->
+        ( if search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) then
             WeightInfo = weight(Gamma, UseArgList),
-            ( functor_norm_filter_args(UseArgList, !Args, !Modes) ->
+            ( if functor_norm_filter_args(UseArgList, !Args, !Modes) then
                 true
-            ;
-                unexpected($module, $pred, "unmatched lists")
+            else
+                unexpected($pred, "unmatched lists")
             )
-        ; ConsId = ground_term_const(ConstNum, _) ->
+        else if ConsId = ground_term_const(ConstNum, _) then
             % XXX Since ground_term_consts have no argument variables,
             % we cannot filter those argument variables. I (zs) *think* that
             % returning the !.Args and !.Modes (which should both be empty
@@ -307,7 +311,7 @@ functor_norm(ModuleInfo, FunctorInfo, TypeCtor, ConsId, Gamma,
             module_info_get_const_struct_db(ModuleInfo, ConstStructDb),
             const_struct_count_cell_filtered_weights(ConstStructDb, WeightMap,
                 ConstNum, 0, Gamma)
-        ;
+        else
             Gamma = 0
         )
     ).
@@ -316,7 +320,7 @@ functor_norm(ModuleInfo, FunctorInfo, TypeCtor, ConsId, Gamma,
     % matched.
     %
 :- pred functor_norm_filter_args(list(bool)::in, list(prog_var)::in,
-    list(prog_var)::out, list(uni_mode)::in, list(uni_mode)::out)
+    list(prog_var)::out, list(unify_mode)::in, list(unify_mode)::out)
     is semidet.
 
 functor_norm_filter_args([], [], [], [], []).
@@ -334,7 +338,7 @@ functor_norm_filter_args([no | Bools], [_Arg0 | Args0], Args,
 
 const_struct_count_cells(ConstStructDb, ConstNum, !Gamma) :-
     lookup_const_struct_num(ConstStructDb, ConstNum, ConstStruct),
-    ConstStruct = const_struct(_ConsId, Args, _, _),
+    ConstStruct = const_struct(_ConsId, Args, _, _, _),
     !:Gamma = !.Gamma + 1,
     const_struct_count_cells_args(ConstStructDb, Args, !Gamma).
 
@@ -356,7 +360,7 @@ const_struct_count_cells_args(ConstStructDb, [Arg | Args], !Gamma) :-
 
 const_struct_count_cell_arities(ConstStructDb, ConstNum, !Gamma) :-
     lookup_const_struct_num(ConstStructDb, ConstNum, ConstStruct),
-    ConstStruct = const_struct(_ConsId, Args, _, _),
+    ConstStruct = const_struct(_ConsId, Args, _, _, _),
     !:Gamma = !.Gamma + list.length(Args),
     const_struct_count_cell_arities_args(ConstStructDb, Args, !Gamma).
 
@@ -378,14 +382,14 @@ const_struct_count_cell_arities_args(ConstStructDb, [Arg | Args], !Gamma) :-
 
 const_struct_count_cell_weights(ConstStructDb, WeightMap, ConstNum, !Gamma) :-
     lookup_const_struct_num(ConstStructDb, ConstNum, ConstStruct),
-    ConstStruct = const_struct(ConsId, Args, Type, _),
+    ConstStruct = const_struct(ConsId, Args, Type, _, _),
     type_to_ctor_det(Type, TypeCtor),
-    ( search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) ->
+    ( if search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) then
         WeightInfo = weight(ConsIdGamma, _),
         !:Gamma = !.Gamma + ConsIdGamma,
         const_struct_count_cell_weights_args(ConstStructDb, WeightMap,
             Args, !Gamma)
-    ;
+    else
         true
     ).
 
@@ -398,10 +402,10 @@ const_struct_count_cell_weights_args(ConstStructDb, WeightMap,
     (
         Arg = csa_constant(ConsId, Type),
         type_to_ctor_det(Type, TypeCtor),
-        ( search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) ->
+        ( if search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) then
             WeightInfo = weight(ConsIdGamma, _),
             !:Gamma = !.Gamma + ConsIdGamma
-        ;
+        else
             true
         )
     ;
@@ -418,14 +422,14 @@ const_struct_count_cell_weights_args(ConstStructDb, WeightMap,
 const_struct_count_cell_filtered_weights(ConstStructDb, WeightMap,
         ConstNum, !Gamma) :-
     lookup_const_struct_num(ConstStructDb, ConstNum, ConstStruct),
-    ConstStruct = const_struct(ConsId, Args, Type, _),
+    ConstStruct = const_struct(ConsId, Args, Type, _, _),
     type_to_ctor_det(Type, TypeCtor),
-    ( search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) ->
+    ( if search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) then
         WeightInfo = weight(ConsIdGamma, UseArgs),
         !:Gamma = !.Gamma + ConsIdGamma,
         const_struct_count_cell_filtered_weights_args(ConstStructDb, WeightMap,
             Args, UseArgs, !Gamma)
-    ;
+    else
         true
     ).
 
@@ -437,10 +441,10 @@ const_struct_count_cell_filtered_weights_args(_ConstStructDb, _WeightMap,
         [], [], !Gamma).
 const_struct_count_cell_filtered_weights_args(_ConstStructDb, _WeightMap,
         [], [_ | _], !Gamma) :-
-    unexpected($module, $pred, "mismatched lists").
+    unexpected($pred, "mismatched lists").
 const_struct_count_cell_filtered_weights_args(_ConstStructDb, _WeightMap,
         [_ | _], [], !Gamma) :-
-    unexpected($module, $pred, "mismatched lists").
+    unexpected($pred, "mismatched lists").
 const_struct_count_cell_filtered_weights_args(ConstStructDb, WeightMap,
         [Arg | Args], [UseArg | UseArgs], !Gamma) :-
     (
@@ -450,10 +454,12 @@ const_struct_count_cell_filtered_weights_args(ConstStructDb, WeightMap,
         (
             Arg = csa_constant(ConsId, Type),
             type_to_ctor_det(Type, TypeCtor),
-            ( search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) ->
+            ( if
+                search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo)
+            then
                 WeightInfo = weight(ConsIdGamma, _),
                 !:Gamma = !.Gamma + ConsIdGamma
-            ;
+            else
                 true
             )
         ;
@@ -476,15 +482,17 @@ functor_lower_bound(_ModuleInfo, FunctorInfo, TypeCtor, ConsId) = Weight :-
         Weight = ( if ConsId = cons(_, Arity, _) then Arity else 0 )
     ;
         FunctorInfo = use_map(WeightMap),
-        ( if    search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo)
-          then  WeightInfo = weight(Weight, _)
-          else  Weight = 0
+        ( if search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) then
+            WeightInfo = weight(Weight, _)
+        else
+            Weight = 0
         )
     ;
         FunctorInfo = use_map_and_args(WeightMap),
-        ( if    search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo)
-          then  WeightInfo = weight(Weight, _)
-          else  Weight = 0
+        ( if search_weight_table(WeightMap, TypeCtor, ConsId, WeightInfo) then
+            WeightInfo = weight(Weight, _)
+        else
+            Weight = 0
         )
     ).
 
@@ -500,6 +508,7 @@ zero_size_type_category(CtorCat, ZeroSize) :-
     (
         ( CtorCat = ctor_cat_builtin(_)
         ; CtorCat = ctor_cat_user(cat_user_direct_dummy)
+        ; CtorCat = ctor_cat_user(cat_user_abstract_dummy)
         ; CtorCat = ctor_cat_void
         ; CtorCat = ctor_cat_system(_)
         ; CtorCat = ctor_cat_higher_order
@@ -509,6 +518,7 @@ zero_size_type_category(CtorCat, ZeroSize) :-
         ZeroSize = yes
     ;
         ( CtorCat = ctor_cat_user(cat_user_notag)
+        ; CtorCat = ctor_cat_user(cat_user_abstract_notag)
         ; CtorCat = ctor_cat_user(cat_user_general)
         ; CtorCat = ctor_cat_tuple
         ; CtorCat = ctor_cat_variable
