@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % Copyright (C) 2006 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% Copyright (C) 2015-2016, 2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %------------------------------------------------------------------------------%
 /****M* libs/log4m/log4m.m/log4m
  * NAME
@@ -27,7 +27,7 @@
  * NAME
  *    Type: id
  * DESCRIPTION
- *    An id for identifying a logger.  The id's are hierachical.
+ *    An id for identifying a logger.  The id's are hierarchical.
  *    The root logger is [].
  *    A child of the root logger is ["Performance"]
  *    A child of the previous logger is ["Child", "Performance"]
@@ -259,7 +259,7 @@
  *    unsafe_log_f
  * SYNOPSIS
  *    Gets a fake io__state and calls log_f.
- *    Unsafe because the compiler might decide to elimiate this call on you.
+ *    Unsafe because the compiler might decide to eliminate this call on you.
  * SOURCE
  */
 :- pred unsafe_log_f(id::in, level::in, ((func) = string)::in) is det.
@@ -661,7 +661,7 @@ set_log(Log, !IO) :-
 
 write_format(A, Id, Level, _Message, date, !IO) :-
     time(Time, !IO),
-    TM = localtime(Time),
+    localtime(Time, TM, !IO),
 	TM = tm(Yr, Mnt, MD, Hrs, Min, Sec, _YD, _WD, _DST),
     Date = string__format("%4d-%02d-%02d %02d:%02d:%02d",
             [i(Yr+1900), i(Mnt+1), i(MD), i(Hrs), i(Min), i(Sec)]),
