@@ -1,17 +1,17 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2002-2007 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
-%-----------------------------------------------------------------------------%
-% 
+% Copyright (C) 2014-2015, 2017-2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
+%---------------------------------------------------------------------------%
+%
 % File: deconstruct.m.
 % Main author: zs.
 % Stability: low.
-% 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module deconstruct.
 :- interface.
@@ -21,7 +21,7 @@
 :- import_module maybe.
 :- import_module univ.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Values of type noncanon_handling are intended to control how
     % predicates that deconstruct terms behave when they find that
@@ -30,7 +30,7 @@
     % concrete representation.
     %
     % The value `do_not_allow' means that in such circumstances the
-    % predicate should abort.
+    % predicate should cause a runtime abort.
     %
     % The value `canonicalize' means that in such circumstances the
     % predicate should return a constant giving the identity of the type,
@@ -45,13 +45,16 @@
     ;       canonicalize
     ;       include_details_cc.
 
-:- inst do_not_allow ---> do_not_allow.
-:- inst canonicalize ---> canonicalize.
-:- inst include_details_cc ---> include_details_cc.
-:- inst canonicalize_or_do_not_allow
+:- inst do_not_allow for noncanon_handling/0
+    --->    do_not_allow.
+:- inst canonicalize for noncanon_handling/0
+    --->    canonicalize.
+:- inst include_details_cc for noncanon_handling/0
+    --->    include_details_cc.
+:- inst canonicalize_or_do_not_allow for noncanon_handling/0
     --->    do_not_allow
     ;       canonicalize.
-:- inst do_not_allow_or_include_details_cc
+:- inst do_not_allow_or_include_details_cc for noncanon_handling/0
     --->    do_not_allow
     ;       include_details_cc.
 
@@ -71,10 +74,15 @@
     %     handled as if it had standard equality.
     %   - for integers, the string is a base 10 number;
     %     positive integers have no sign.
-    %   - for floats, the string is a floating point, base 10 number;
-    %     positive floating point numbers have no sign.
-    %   - for strings, the string, inside double quotation marks
-    %   - for characters, the character inside single quotation marks
+    %   - for finite floats, the string is a base 10 floating point number;
+    %     positive floating point numbers have no sign;
+    %     for infinite floats, the string "infinity" or "-infinity".
+    %   - for strings, the string, inside double quotation marks using
+    %     backslash escapes if necessary and backslash or octal escapes for
+    %     all characters for which char.is_control/1 is true.
+    %   - for characters, the character inside single quotation marks using
+    %     a backslash escape if necessary and a backslash or octal escape for
+    %     for all characters for which char.is_control/1 is true.
     %   - for predicates, the string <<predicate>>, and for functions,
     %     the string <<function>>, except with include_details_cc,
     %     in which case it will be the predicate or function name.
@@ -82,9 +90,12 @@
     %     predicate and function values created by lambda expressions.)
     %   - for tuples, the string {}.
     %   - for arrays, the string <<array>>.
-    %   - for c_pointers, the string ptr(0xXXXX) where XXXX is the
+    %   - for c_pointers, the string c_pointer(0xXXXX) where XXXX is the
     %     hexadecimal representation of the pointer.
-    %   - for bitmaps, the bitmap converted to a a length and a
+    %   - for foreign types, a string of the form <<foreign(Name, Rep)>> where
+    %     Name is the type's Mercury name and Rep is a target language specific
+    %     representation of the term's value.
+    %   - for bitmaps, the bitmap converted to a length and a
     %     hexadecimal string inside angle brackets and quotes of the
     %     form """<[0-9]:[0-9A-F]*>""".
     %
@@ -105,6 +116,7 @@
     %   - for tuples, the number of elements in the tuple.
     %   - for arrays, the number of elements in the array.
     %   - for c_pointers, zero.
+    %   - for foreign types, zero.
     %   - for bitmaps, zero.
     %
     % Note that in the current University of Melbourne implementation,
@@ -130,7 +142,7 @@
     % Given a data item, return the number of the functor,
     % suitable for use by construct.construct, and the arity.
     % Fail if the item does not have a discriminated union type.
-    % Abort if the type has user-defined equality.
+    % Cause a runtime abort if the type has user-defined equality.
     %
 :- pred functor_number(T::in, functor_number_lex::out, int::out) is semidet.
 
@@ -139,10 +151,10 @@
     % Given a data item, return the number of the functor,
     % suitable for use by construct.construct, and the arity.
     % Fail if the item does not have a discriminated union type.
-    % Don't abort if the type has user-defined equality.
+    % Do not cause a runtime abort if the type has user-defined equality.
     %
 :- pred functor_number_cc(T::in, functor_number_lex::out,
-    int::out) is cc_nondet. 
+    int::out) is cc_nondet.
 
     % arg(Data, NonCanon, Index, Argument)
     %
@@ -190,7 +202,7 @@
     % det_arg(Data, NonCanon, Index, Argument)
     %
     % Same as arg/4, except that for cases where arg/4 would fail,
-    % det_arg/4 will abort.
+    % det_arg/4 will throw an exception.
     %
 :- some [ArgT] pred det_arg(T, noncanon_handling, int, ArgT).
 :- mode det_arg(in, in(do_not_allow), in, out) is det.
@@ -201,7 +213,7 @@
     % det_named_arg(Data, NonCanon, Name, Argument)
     %
     % Same as named_arg/4, except that for cases where named_arg/4 would fail,
-    % det_named_arg/4 will abort.
+    % det_named_arg/4 will throw an exception.
     %
 :- some [ArgT] pred det_named_arg(T, noncanon_handling, string, ArgT).
 :- mode det_named_arg(in, in(do_not_allow), in, out) is det.
@@ -267,8 +279,8 @@
 :- pred limited_deconstruct_cc(T::in, int::in,
     maybe({string, int, list(univ)})::out) is cc_multi.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -276,11 +288,7 @@
 :- import_module require.
 :- import_module type_desc.
 
-% For use by the Erlang backends.
-%
-:- use_module erlang_rtti_implementation.
-
-% For use by the Java and IL backends.
+% For use by the Java and C# backends.
 %
 :- use_module rtti_implementation.
 
@@ -291,22 +299,22 @@
 
 ").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 % XXX The no-inline pragmas are necessary because when it inlines a predicate
 % defined by foreign_procs, the compiler does not preserve the names of the
 % typeinfo variables. Thus these foreign_proc's references to TypeInfo_for_T
 % will refer to an undefined variable.
 
-:- pragma no_inline(functor/4).
-:- pragma no_inline(functor_number/3).
-:- pragma no_inline(functor_number_cc/3).
-:- pragma no_inline(arg/4).
-:- pragma no_inline(named_arg/4).
-:- pragma no_inline(deconstruct/5).
-:- pragma no_inline(limited_deconstruct/6).
+:- pragma no_inline(pred(functor/4)).
+:- pragma no_inline(pred(functor_number/3)).
+:- pragma no_inline(pred(functor_number_cc/3)).
+:- pragma no_inline(pred(arg/4)).
+:- pragma no_inline(pred(named_arg/4)).
+:- pragma no_inline(pred(deconstruct/5)).
+:- pragma no_inline(pred(limited_deconstruct/6)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 functor(Term, NonCanon, Functor, Arity) :-
     (
@@ -320,178 +328,7 @@ functor(Term, NonCanon, Functor, Arity) :-
         functor_idcc(Term, Functor, Arity)
     ).
 
-arg(Term, NonCanon, Index, Argument) :-
-    (
-        NonCanon = do_not_allow,
-        univ_arg_dna(Term, Index, Univ)
-    ;
-        NonCanon = canonicalize,
-        univ_arg_can(Term, Index, Univ)
-    ;
-        NonCanon = include_details_cc,
-        error("deconstruct.arg called with include_details_cc")
-    ),
-    Argument = univ_value(Univ).
-
-arg_cc(Term, Index, MaybeArg) :-
-    univ_arg_idcc(Term, Index, dummy_univ, Univ, Success),
-    ( Success \= 0 ->
-        MaybeArg = 'new arg'(univ_value(Univ))
-    ;
-        MaybeArg = no_arg
-    ).
-
-named_arg(Term, NonCanon, Name, Argument) :-
-    (
-        NonCanon = do_not_allow,
-        univ_named_arg_dna(Term, Name, Univ)
-    ;
-        NonCanon = canonicalize,
-        univ_named_arg_can(Term, Name, Univ)
-    ;
-        NonCanon = include_details_cc,
-        error("deconstruct.named_arg called with include_details_cc")
-    ),
-    Argument = univ_value(Univ).
-
-named_arg_cc(Term, Name, MaybeArg) :-
-    univ_named_arg_idcc(Term, Name, dummy_univ, Univ, Success),
-    ( Success \= 0 ->
-        MaybeArg = 'new arg'(univ_value(Univ))
-    ;
-        MaybeArg = no_arg
-    ).
-
-    % This is a dummy value of type `univ'. It is used only to ensure that
-    % the C interface procedure univ_named_arg_idcc doesn't return an
-    % uninitialized (or otherwise bogus) univ value.
-    %
-:- func dummy_univ = univ.
-
-dummy_univ = univ(0).
-
-det_arg(Term, NonCanon, Index, Argument) :-
-    (
-        NonCanon = do_not_allow,
-        ( univ_arg_dna(Term, Index, Univ0) ->
-            Univ = Univ0
-        ;
-            error("det_arg: argument number out of range")
-        )
-    ;
-        NonCanon = canonicalize,
-        ( univ_arg_can(Term, Index, Univ0) ->
-            Univ = Univ0
-        ;
-            error("det_arg: argument number out of range")
-        )
-    ;
-        NonCanon = include_details_cc,
-        univ_arg_idcc(Term, Index, dummy_univ, Univ0, Success),
-        ( Success \= 0 ->
-            Univ = Univ0
-        ;
-            error("det_arg: argument number out of range")
-        )
-    ),
-    Argument = univ_value(Univ).
-
-det_named_arg(Term, NonCanon, Name, Argument) :-
-    (
-        (
-            NonCanon = do_not_allow,
-            univ_named_arg_dna(Term, Name, Univ)
-        ;
-            NonCanon = canonicalize,
-            univ_named_arg_can(Term, Name, Univ)
-        ;
-            NonCanon = include_details_cc,
-            univ_named_arg_idcc(Term, Name, dummy_univ, Univ0, Success),
-            ( Success \= 0 ->
-                Univ = Univ0
-            ;
-                error("det_named_arg: no argument with that name")
-            )
-        )
-    ->
-        Argument = univ_value(Univ)
-    ;
-        error("det_named_arg: no argument with that name")
-    ).
-
-deconstruct(Term, NonCanon, Functor, Arity, Arguments) :-
-    (
-        NonCanon = do_not_allow,
-        deconstruct_dna(Term, Functor, _, Arity, Arguments)
-    ;
-        NonCanon = canonicalize,
-        deconstruct_can(Term, Functor, Arity, Arguments)
-    ;
-        NonCanon = include_details_cc,
-        deconstruct_idcc(Term, Functor, _, Arity, Arguments)
-    ).
-
-deconstruct_du(Term, NonCanon, FunctorNumber, Arity, Arguments) :-
-    ( erlang_rtti_implementation.is_erlang_backend ->
-        erlang_rtti_implementation.deconstruct_du(Term, NonCanon,
-            FunctorNumber, Arity, Arguments)
-    ;
-        deconstruct_du_2(Term, NonCanon, FunctorNumber, Arity, Arguments)
-    ).
-
-:- pred deconstruct_du_2(T, noncanon_handling, functor_number_lex,
-    int, list(univ)).
-:- mode deconstruct_du_2(in, in(do_not_allow), out, out, out) is semidet.
-:- mode deconstruct_du_2(in, in(include_details_cc), out, out, out)
-    is cc_nondet.
-:- mode deconstruct_du_2(in, in, out, out, out) is cc_nondet.
-
-deconstruct_du_2(Term, NonCanon, FunctorNumber, Arity, Arguments) :-
-    ( _ = construct.num_functors(type_of(Term)) ->
-        (
-            NonCanon = do_not_allow,
-            deconstruct_dna(Term, _, FunctorNumber, Arity, Arguments)
-        ;
-            NonCanon = canonicalize,
-            error("deconstruct_du: canonicalize not supported")
-        ;
-            NonCanon = include_details_cc,
-            deconstruct_idcc(Term, _, FunctorNumber, Arity, Arguments)
-        ),
-        ( FunctorNumber >= 0 ->
-            true
-        ;
-            error("deconstruct_du: internal error (recompile needed?)")
-        )
-    ;
-        fail
-    ).
-
-limited_deconstruct(Term, NonCanon, MaxArity, Functor, Arity, Arguments) :-
-    (
-        NonCanon = do_not_allow,
-        limited_deconstruct_dna(Term, MaxArity, Functor, Arity, Arguments)
-    ;
-        NonCanon = canonicalize,
-        limited_deconstruct_can(Term, MaxArity, Functor, Arity, Arguments)
-    ;
-        NonCanon = include_details_cc,
-        error("limited_deconstruct called with include_details_cc")
-    ).
-
-limited_deconstruct_cc(Term, MaxArity, MaybeResult) :-
-    limited_deconstruct_idcc(Term, MaxArity, Functor, Arity, Arguments),
-    ( Arity =< MaxArity ->
-        MaybeResult = yes({Functor, Arity, Arguments})
-    ;
-        MaybeResult = no
-    ).
-
-%-----------------------------------------------------------------------------%
-
 :- pred functor_dna(T::in, string::out, int::out) is det.
-:- pred functor_can(T::in, string::out, int::out) is det.
-:- pred functor_idcc(T::in, string::out, int::out) is cc_multi.
 
 :- pragma foreign_proc("C",
     functor_dna(Term::in, Functor::out, Arity::out),
@@ -510,6 +347,11 @@ limited_deconstruct_cc(Term, MaxArity, MaybeResult) :-
 #undef  NONCANON
 }").
 
+functor_dna(Term, Functor, Arity) :-
+    local_deconstruct(Term, do_not_allow, Functor, _, Arity, _Arguments).
+
+:- pred functor_can(T::in, string::out, int::out) is det.
+
 :- pragma foreign_proc("C",
     functor_can(Term::in, Functor::out, Arity::out),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -527,6 +369,11 @@ limited_deconstruct_cc(Term, MaxArity, MaybeResult) :-
 #undef  NONCANON
 }").
 
+functor_can(Term, Functor, Arity) :-
+    local_deconstruct(Term, canonicalize, Functor, _, Arity, _Arguments).
+
+:- pred functor_idcc(T::in, string::out, int::out) is cc_multi.
+
 :- pragma foreign_proc("C",
     functor_idcc(Term::in, Functor::out, Arity::out),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -544,16 +391,10 @@ limited_deconstruct_cc(Term, MaxArity, MaybeResult) :-
 #undef  NONCANON
 }").
 
-functor_dna(Term, Functor, Arity) :-
-    local_deconstruct(Term, do_not_allow, Functor, _, Arity, _Arguments).
-
-functor_can(Term, Functor, Arity) :-
-    local_deconstruct(Term, canonicalize, Functor, _, Arity, _Arguments).
-
 functor_idcc(Term, Functor, Arity) :-
     local_deconstruct(Term, include_details_cc, Functor, _, Arity, _Arguments).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_proc("C",
     functor_number(Term::in, FunctorNumber::out, Arity::out),
@@ -575,6 +416,9 @@ functor_idcc(Term, Functor, Arity) :-
 SUCCESS_INDICATOR = (FunctorNumber >= 0);
 }").
 
+functor_number(_Term, _FunctorNumber, _Arity) :-
+    private_builtin.sorry("deconstruct.functor_number").
+
 :- pragma foreign_proc("C",
     functor_number_cc(Term::in, FunctorNumber::out, Arity::out),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -595,22 +439,166 @@ SUCCESS_INDICATOR = (FunctorNumber >= 0);
 SUCCESS_INDICATOR = (FunctorNumber >= 0);
 }").
 
-functor_number(Term, FunctorNumber, Arity) :-
-    ( erlang_rtti_implementation.is_erlang_backend ->
-        erlang_rtti_implementation.functor_number(Term, FunctorNumber, Arity)
-    ;
-        private_builtin.sorry("deconstruct.functor_number")
-    ).
-
 functor_number_cc(Term, FunctorNumber, Arity) :-
-    ( erlang_rtti_implementation.is_erlang_backend ->
-        erlang_rtti_implementation.functor_number_cc(Term, FunctorNumber,
-            Arity)
+    rtti_implementation.functor_number_cc(Term, FunctorNumber, Arity).
+
+%---------------------------------------------------------------------------%
+
+arg(Term, NonCanon, Index, Argument) :-
+    (
+        NonCanon = do_not_allow,
+        univ_arg_dna(Term, Index, Univ)
     ;
-        rtti_implementation.functor_number_cc(Term, FunctorNumber, Arity)
+        NonCanon = canonicalize,
+        univ_arg_can(Term, Index, Univ)
+    ;
+        NonCanon = include_details_cc,
+        unexpected($pred, "called with include_details_cc")
+    ),
+    Argument = univ_value(Univ).
+
+arg_cc(Term, Index, MaybeArg) :-
+    univ_arg_idcc(Term, Index, dummy_univ, Univ, Success),
+    ( if Success = 0 then
+        MaybeArg = no_arg
+    else
+        MaybeArg = 'new arg'(univ_value(Univ))
     ).
 
-%-----------------------------------------------------------------------------%
+named_arg(Term, NonCanon, Name, Argument) :-
+    (
+        NonCanon = do_not_allow,
+        univ_named_arg_dna(Term, Name, Univ)
+    ;
+        NonCanon = canonicalize,
+        univ_named_arg_can(Term, Name, Univ)
+    ;
+        NonCanon = include_details_cc,
+        unexpected($pred, "called with include_details_cc")
+    ),
+    Argument = univ_value(Univ).
+
+named_arg_cc(Term, Name, MaybeArg) :-
+    univ_named_arg_idcc(Term, Name, dummy_univ, Univ, Success),
+    ( if Success = 0 then
+        MaybeArg = no_arg
+    else
+        MaybeArg = 'new arg'(univ_value(Univ))
+    ).
+
+    % This is a dummy value of type `univ'. It is used only to ensure that
+    % the C interface procedure univ_named_arg_idcc doesn't return an
+    % uninitialized (or otherwise bogus) univ value.
+    %
+:- func dummy_univ = univ.
+
+dummy_univ = univ(0).
+
+det_arg(Term, NonCanon, Index, Argument) :-
+    (
+        NonCanon = do_not_allow,
+        ( if univ_arg_dna(Term, Index, Univ0) then
+            Univ = Univ0
+        else
+            unexpected($pred, "argument number out of range")
+        )
+    ;
+        NonCanon = canonicalize,
+        ( if univ_arg_can(Term, Index, Univ0) then
+            Univ = Univ0
+        else
+            unexpected($pred, "argument number out of range")
+        )
+    ;
+        NonCanon = include_details_cc,
+        univ_arg_idcc(Term, Index, dummy_univ, Univ0, Success),
+        ( if Success = 0 then
+            unexpected($pred, "argument number out of range")
+        else
+            Univ = Univ0
+        )
+    ),
+    Argument = univ_value(Univ).
+
+det_named_arg(Term, NonCanon, Name, Argument) :-
+    ( if
+        (
+            NonCanon = do_not_allow,
+            univ_named_arg_dna(Term, Name, Univ)
+        ;
+            NonCanon = canonicalize,
+            univ_named_arg_can(Term, Name, Univ)
+        ;
+            NonCanon = include_details_cc,
+            univ_named_arg_idcc(Term, Name, dummy_univ, Univ0, Success),
+            ( if Success = 0 then
+                unexpected($pred, "no argument with that name")
+            else
+                Univ = Univ0
+            )
+        )
+    then
+        Argument = univ_value(Univ)
+    else
+        unexpected($pred, "no argument with that name")
+    ).
+
+deconstruct(Term, NonCanon, Functor, Arity, Arguments) :-
+    (
+        NonCanon = do_not_allow,
+        deconstruct_dna(Term, Functor, _, Arity, Arguments)
+    ;
+        NonCanon = canonicalize,
+        deconstruct_can(Term, Functor, Arity, Arguments)
+    ;
+        NonCanon = include_details_cc,
+        deconstruct_idcc(Term, Functor, _, Arity, Arguments)
+    ).
+
+deconstruct_du(Term, NonCanon, FunctorNumber, Arity, Arguments) :-
+    ( if _ = construct.num_functors(type_of(Term)) then
+        (
+            NonCanon = do_not_allow,
+            deconstruct_dna(Term, _, FunctorNumber, Arity, Arguments)
+        ;
+            NonCanon = canonicalize,
+            unexpected($module, "deconstruct_du",
+                "canonicalize not supported")
+        ;
+            NonCanon = include_details_cc,
+            deconstruct_idcc(Term, _, FunctorNumber, Arity, Arguments)
+        ),
+        ( if FunctorNumber >= 0 then
+            true
+        else
+            unexpected($module, "deconstruct_du",
+                "internal error (recompile needed?)")
+        )
+    else
+        fail
+    ).
+
+limited_deconstruct(Term, NonCanon, MaxArity, Functor, Arity, Arguments) :-
+    (
+        NonCanon = do_not_allow,
+        limited_deconstruct_dna(Term, MaxArity, Functor, Arity, Arguments)
+    ;
+        NonCanon = canonicalize,
+        limited_deconstruct_can(Term, MaxArity, Functor, Arity, Arguments)
+    ;
+        NonCanon = include_details_cc,
+        unexpected($pred, "called with include_details_cc")
+    ).
+
+limited_deconstruct_cc(Term, MaxArity, MaybeResult) :-
+    limited_deconstruct_idcc(Term, MaxArity, Functor, Arity, Arguments),
+    ( if Arity =< MaxArity then
+        MaybeResult = yes({Functor, Arity, Arguments})
+    else
+        MaybeResult = no
+    ).
+
+%---------------------------------------------------------------------------%
 
 % XXX These predicates return univs instead of existentially typed arguments
 % in order to work around the typechecking bug reported on 30 Jan, 2002
@@ -618,6 +606,10 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
 % currently we don't support implementations in multiple languages
 % for procedures with existentially typed arguments.
 
+% XXX The Mercury implementations of these predicates are inefficient,
+% since they unnecessarily construct the list of univs for all the arguments,
+% rather than just constructing one univ for the argument selected.
+
 :- pred univ_arg_dna(T::in, int::in, univ::out) is semidet.
 :- pred univ_arg_can(T::in, int::in, univ::out) is semidet.
 
@@ -642,6 +634,8 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
 :- pred univ_named_arg_idcc(T::in, string::in, univ::in, univ::out, int::out)
     is cc_multi.
 
+%---------------------%
+
 :- pragma foreign_proc("C",
     univ_arg_dna(Term::in, Index::in, Argument::out),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -663,6 +657,10 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
 #undef  SAVE_SUCCESS
 }").
 
+univ_arg_dna(Term, Index, Arg) :-
+    local_deconstruct(Term, do_not_allow, _Functor, _, _Arity, Arguments),
+    list.index0(Arguments, Index, Arg).
+
 :- pragma foreign_proc("C",
     univ_arg_can(Term::in, Index::in, Argument::out),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -684,6 +682,10 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
 #undef  SAVE_SUCCESS
 }").
 
+univ_arg_can(Term, Index, Arg) :-
+    local_deconstruct(Term, canonicalize, _Functor, _, _Arity, Arguments),
+    list.index0(Arguments, Index, Arg).
+
 :- pragma foreign_proc("C",
     univ_arg_idcc(Term::in, Index::in, DummyUniv::in, Argument::out,
         Success::out),
@@ -711,6 +713,17 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
     }
 }").
 
+univ_arg_idcc(Term, Index, DummyUniv, Argument, Success) :-
+    local_deconstruct(Term, include_details_cc, _Functor, _FunctorNumber,
+        _Arity, Arguments),
+    ( if list.index0(Arguments, Index, Arg) then
+        Argument = Arg,
+        Success = 1
+    else
+        Argument = DummyUniv,
+        Success = 0
+    ).
+
 :- pragma foreign_proc("C",
     univ_named_arg_dna(Term::in, Name::in, Argument::out),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -734,6 +747,9 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
 #undef  SAVE_SUCCESS
 }").
 
+univ_named_arg_dna(Term, Name, Argument) :-
+    local_univ_named_arg(Term, do_not_allow, Name, Argument).
+
 :- pragma foreign_proc("C",
     univ_named_arg_can(Term::in, Name::in, Argument::out),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -757,6 +773,9 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
 #undef  SAVE_SUCCESS
 }").
 
+univ_named_arg_can(Term, Name, Argument) :-
+    local_univ_named_arg(Term, canonicalize, Name, Argument).
+
 :- pragma foreign_proc("C",
     univ_named_arg_idcc(Term::in, Name::in, DummyUniv::in,
         Argument::out, Success::out),
@@ -787,48 +806,22 @@ functor_number_cc(Term, FunctorNumber, Arity) :-
 
 }").
 
-% XXX These Mercury implementations are all inefficient, since they
-% unnecessarily construct the list of univs for all the arguments, rather than
-% just constructing one univ for the argument selected.
-
-univ_arg_dna(Term, Index, Arg) :-
-    local_deconstruct(Term, do_not_allow, _Functor, _, _Arity, Arguments),
-    list.index0(Arguments, Index, Arg).
-
-univ_arg_can(Term, Index, Arg) :-
-    local_deconstruct(Term, canonicalize, _Functor, _, _Arity, Arguments),
-    list.index0(Arguments, Index, Arg).
-
-univ_arg_idcc(Term, Index, DummyUniv, Argument, Success) :-
-    local_deconstruct(Term, include_details_cc, _Functor, _, _Arity, Arguments),
-    ( list.index0(Arguments, Index, Arg) ->
-        Argument = Arg,
-        Success = 1
-    ;
-        Argument = DummyUniv,
-        Success = 0
-    ).
-
-univ_named_arg_dna(Term, Name, Argument) :-
-    local_univ_named_arg(Term, do_not_allow, Name, Argument).
-
-univ_named_arg_can(Term, Name, Argument) :-
-    local_univ_named_arg(Term, canonicalize, Name, Argument).
-
 univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
-    ( local_univ_named_arg(Term, include_details_cc, Name, Arg) ->
+    ( if local_univ_named_arg(Term, include_details_cc, Name, Arg) then
         Argument = Arg,
         Success = 1
-    ;
-        Argument = DummyUniv,
-        Success = 0
-    ;
-        % Force cc_multi.
-        Argument = DummyUniv,
-        Success = 0
+    else
+        (
+            Argument = DummyUniv,
+            Success = 0
+        ;
+            % Force cc_multi.
+            Argument = DummyUniv,
+            Success = 0
+        )
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred deconstruct_dna(T::in, string::out,
     int::out, int::out, list(univ)::out) is det.
@@ -836,6 +829,10 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 :- pred deconstruct_idcc(T::in, string::out,
     int::out, int::out, list(univ)::out) is cc_multi.
 
+    % XXX The Mercury implementations of all of these limited_* procedures
+    % are inefficient -- they construct Functor and Arguments even in the case
+    % when Arity > MaxArity.
+
 :- pred limited_deconstruct_dna(T::in, int::in,
     string::out, int::out, list(univ)::out) is semidet.
 :- pred limited_deconstruct_can(T::in, int::in,
@@ -848,7 +845,7 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
         Arguments::out),
     [will_not_call_mercury, thread_safe, promise_pure],
 "{
-#define EXPAND_INFO_TYPE        MR_Expand_Functor_Args_Info
+#define EXPAND_INFO_TYPE        MR_ExpandFunctorArgsInfo
 #define EXPAND_INFO_CALL        MR_expand_functor_args
 #define TYPEINFO_ARG            TypeInfo_for_T
 #define TERM_ARG                Term
@@ -857,7 +854,7 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #define ARITY_ARG               Arity
 #define ARGUMENTS_ARG           Arguments
 #define NONCANON                MR_NONCANON_ABORT
-/* This comment tells the compiler to define MR_ALLOC_ID. */
+// This comment tells the compiler to define MR_ALLOC_ID.
 #include ""mercury_ml_deconstruct_body.h""
 #undef  EXPAND_INFO_TYPE
 #undef  EXPAND_INFO_CALL
@@ -870,11 +867,15 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #undef  NONCANON
 }").
 
+deconstruct_dna(Term, Functor, FunctorNumber, Arity, Arguments) :-
+    local_deconstruct(Term, do_not_allow, Functor, FunctorNumber, Arity,
+        Arguments).
+
 :- pragma foreign_proc("C",
     deconstruct_can(Term::in, Functor::out, Arity::out, Arguments::out),
     [will_not_call_mercury, thread_safe, promise_pure],
 "{
-#define EXPAND_INFO_TYPE        MR_Expand_Functor_Args_Info
+#define EXPAND_INFO_TYPE        MR_ExpandFunctorArgsInfo
 #define EXPAND_INFO_CALL        MR_expand_functor_args
 #define TYPEINFO_ARG            TypeInfo_for_T
 #define TERM_ARG                Term
@@ -883,7 +884,7 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #define ARITY_ARG               Arity
 #define ARGUMENTS_ARG           Arguments
 #define NONCANON                MR_NONCANON_ALLOW
-/* This comment tells the compiler to define MR_ALLOC_ID. */
+// This comment tells the compiler to define MR_ALLOC_ID.
 #include ""mercury_ml_deconstruct_body.h""
 #undef  EXPAND_INFO_TYPE
 #undef  EXPAND_INFO_CALL
@@ -895,12 +896,15 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #undef  NONCANON
 }").
 
+deconstruct_can(Term, Functor, Arity, Arguments) :-
+    local_deconstruct(Term, canonicalize, Functor, _, Arity, Arguments).
+
 :- pragma foreign_proc("C",
     deconstruct_idcc(Term::in, Functor::out, FunctorNumber::out,
         Arity::out, Arguments::out),
     [will_not_call_mercury, thread_safe, promise_pure],
 "{
-#define EXPAND_INFO_TYPE        MR_Expand_Functor_Args_Info
+#define EXPAND_INFO_TYPE        MR_ExpandFunctorArgsInfo
 #define EXPAND_INFO_CALL        MR_expand_functor_args
 #define TYPEINFO_ARG            TypeInfo_for_T
 #define TERM_ARG                Term
@@ -909,7 +913,7 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #define ARITY_ARG               Arity
 #define ARGUMENTS_ARG           Arguments
 #define NONCANON                MR_NONCANON_CC
-/* This comment tells the compiler to define MR_ALLOC_ID. */
+// This comment tells the compiler to define MR_ALLOC_ID.
 #include ""mercury_ml_deconstruct_body.h""
 #undef  EXPAND_INFO_TYPE
 #undef  EXPAND_INFO_CALL
@@ -922,12 +926,16 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #undef  NONCANON
 }").
 
+deconstruct_idcc(Term, Functor, FunctorNumber, Arity, Arguments) :-
+    local_deconstruct(Term, include_details_cc, Functor, FunctorNumber, Arity,
+        Arguments).
+
 :- pragma foreign_proc("C",
     limited_deconstruct_dna(Term::in, MaxArity::in,
         Functor::out, Arity::out, Arguments::out),
     [will_not_call_mercury, thread_safe, promise_pure],
 "{
-#define EXPAND_INFO_TYPE        MR_Expand_Functor_Args_Limit_Info
+#define EXPAND_INFO_TYPE        MR_ExpandFunctorArgsLimitInfo
 #define EXPAND_INFO_CALL        MR_expand_functor_args_limit
 #define TYPEINFO_ARG            TypeInfo_for_T
 #define TERM_ARG                Term
@@ -937,7 +945,7 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #define ARGUMENTS_ARG           Arguments
 #define NONCANON                MR_NONCANON_ABORT
 #define SAVE_SUCCESS
-/* This comment tells the compiler to define MR_ALLOC_ID. */
+// This comment tells the compiler to define MR_ALLOC_ID.
 #include ""mercury_ml_deconstruct_body.h""
 #undef  EXPAND_INFO_TYPE
 #undef  EXPAND_INFO_CALL
@@ -951,12 +959,16 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #undef  SAVE_SUCCESS
 }").
 
+limited_deconstruct_dna(Term, MaxArity, Functor, Arity, Arguments) :-
+    local_deconstruct(Term, do_not_allow, Functor, _, Arity, Arguments),
+    Arity =< MaxArity.
+
 :- pragma foreign_proc("C",
     limited_deconstruct_can(Term::in, MaxArity::in,
         Functor::out, Arity::out, Arguments::out),
     [will_not_call_mercury, thread_safe, promise_pure],
 "{
-#define EXPAND_INFO_TYPE        MR_Expand_Functor_Args_Limit_Info
+#define EXPAND_INFO_TYPE        MR_ExpandFunctorArgsLimitInfo
 #define EXPAND_INFO_CALL        MR_expand_functor_args_limit
 #define TYPEINFO_ARG            TypeInfo_for_T
 #define TERM_ARG                Term
@@ -966,7 +978,7 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #define ARGUMENTS_ARG           Arguments
 #define NONCANON                MR_NONCANON_ALLOW
 #define SAVE_SUCCESS
-/* This comment tells the compiler to define MR_ALLOC_ID. */
+// This comment tells the compiler to define MR_ALLOC_ID.
 #include ""mercury_ml_deconstruct_body.h""
 #undef  EXPAND_INFO_TYPE
 #undef  EXPAND_INFO_CALL
@@ -980,12 +992,16 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
 #undef  SAVE_SUCCESS
 }").
 
+limited_deconstruct_can(Term, MaxArity, Functor, Arity, Arguments) :-
+    local_deconstruct(Term, canonicalize, Functor, _, Arity, Arguments),
+    Arity =< MaxArity.
+
 :- pragma foreign_proc("C",
     limited_deconstruct_idcc(Term::in, MaxArity::in, Functor::out,
         Arity::out, Arguments::out),
     [will_not_call_mercury, thread_safe, promise_pure],
 "{
-    #define EXPAND_INFO_TYPE    MR_Expand_Functor_Args_Limit_Info
+    #define EXPAND_INFO_TYPE    MR_ExpandFunctorArgsLimitInfo
     #define EXPAND_INFO_CALL    MR_expand_functor_args_limit
     #define TYPEINFO_ARG        TypeInfo_for_T
     #define TERM_ARG            Term
@@ -994,7 +1010,7 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
     #define ARITY_ARG           Arity
     #define ARGUMENTS_ARG       Arguments
     #define NONCANON            MR_NONCANON_CC
-    /* This comment tells the compiler to define MR_ALLOC_ID. */
+    // This comment tells the compiler to define MR_ALLOC_ID.
     #include ""mercury_ml_deconstruct_body.h""
     #undef  EXPAND_INFO_TYPE
     #undef  EXPAND_INFO_CALL
@@ -1007,12 +1023,10 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
     #undef  NONCANON
 
     if (!success) {
-        /*
-        ** Fill in some dummy values, to ensure that we don't try to return
-        ** uninitialized memory to Mercury. It doesn't matter what we put here,
-        ** except that we must have Arity > MaxArity. The casts cast away
-        ** const.
-        */
+        // Fill in some dummy values, to ensure that we don't try to return
+        // uninitialized memory to Mercury. It doesn't matter what we put here,
+        // except that we must have Arity > MaxArity. The casts cast away
+        // const.
 
         Arity = MaxArity + 1;
         Functor = (MR_String) (MR_Integer) """";
@@ -1020,34 +1034,12 @@ univ_named_arg_idcc(Term, Name, DummyUniv, Argument, Success) :-
     }
 }").
 
-deconstruct_dna(Term, Functor, FunctorNumber, Arity, Arguments) :-
-    local_deconstruct(Term, do_not_allow, Functor, FunctorNumber, Arity,
-        Arguments).
-
-deconstruct_can(Term, Functor, Arity, Arguments) :-
-    local_deconstruct(Term, canonicalize, Functor, _, Arity, Arguments).
-
-deconstruct_idcc(Term, Functor, FunctorNumber, Arity, Arguments) :-
-    local_deconstruct(Term, include_details_cc, Functor, FunctorNumber, Arity,
-        Arguments).
-
-    % XXX The Mercury implementations of all of these limited_* procedures
-    % are inefficient -- they construct Functor and Arguments even in the case
-    % when Arity > MaxArity.
-limited_deconstruct_dna(Term, MaxArity, Functor, Arity, Arguments) :-
-    local_deconstruct(Term, do_not_allow, Functor, _, Arity, Arguments),
-    Arity =< MaxArity.
-
-limited_deconstruct_can(Term, MaxArity, Functor, Arity, Arguments) :-
-    local_deconstruct(Term, canonicalize, Functor, _, Arity, Arguments),
-    Arity =< MaxArity.
-
 limited_deconstruct_idcc(Term, _MaxArity, Functor, Arity, Arguments) :-
     % For this one, the caller checks Arity =< MaxArity.
     local_deconstruct(Term, include_details_cc, Functor, _, Arity, Arguments).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred local_deconstruct(T, noncanon_handling, string, int, int, list(univ)).
 :- mode local_deconstruct(in, in(do_not_allow), out, out, out, out) is det.
@@ -1055,30 +1047,22 @@ limited_deconstruct_idcc(Term, _MaxArity, Functor, Arity, Arguments) :-
 :- mode local_deconstruct(in, in(include_details_cc), out, out, out, out)
     is cc_multi.
 :- mode local_deconstruct(in, in, out, out, out, out) is cc_multi.
+:- pragma consider_used(pred(local_deconstruct/6)).
 
 local_deconstruct(Term, NonCanon, Functor, FunctorNumber, Arity, Arguments) :-
-    ( erlang_rtti_implementation.is_erlang_backend ->
-        erlang_rtti_implementation.deconstruct(Term, NonCanon, Functor, Arity,
-            Arguments),
-        % XXX incomplete
-        FunctorNumber = 0
-    ;
-        rtti_implementation.deconstruct(Term, NonCanon, Functor, FunctorNumber,
-            Arity, Arguments)
-    ).
+    rtti_implementation.deconstruct(Term, NonCanon, Functor, FunctorNumber,
+        Arity, Arguments).
 
 :- pred local_univ_named_arg(T, noncanon_handling, string, univ).
 :- mode local_univ_named_arg(in, in(do_not_allow), in, out) is semidet.
 :- mode local_univ_named_arg(in, in(canonicalize), in, out) is semidet.
 :- mode local_univ_named_arg(in, in(include_details_cc), in, out)
     is semidet. % conceptually committed-choice
+:- pragma consider_used(pred(local_univ_named_arg/4)).
 
 local_univ_named_arg(Term, NonCanon, Name, Argument) :-
-    ( erlang_rtti_implementation.is_erlang_backend ->
-        private_builtin.sorry("local_univ_named_arg")
-    ;
-        rtti_implementation.univ_named_arg(Term, NonCanon, Name, Argument)
-    ).
+    rtti_implementation.univ_named_arg(Term, NonCanon, Name, Argument).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module deconstruct.
+%---------------------------------------------------------------------------%
