@@ -341,6 +341,9 @@
 
     % rtc(G, RTC) is true if RTC is the reflexive transitive closure of G.
     %
+    % RTC is the reflexive closure of the transitive closure of G,
+    % or, equivalently, the transitive closure of the reflexive closure of G.
+    %
 :- func rtc(digraph(T)) = digraph(T).
 :- pred rtc(digraph(T)::in, digraph(T)::out) is det.
 
@@ -362,6 +365,24 @@
 
 :- implementation.
 
+% Everything below here is not intended to be part of the public interface,
+% and will not be included in the Mercury library reference manual.
+
+:- interface.
+
+    % Straightforward implementation of tc for debugging.
+    %
+:- pred slow_tc(digraph(T)::in, digraph(T)::out) is det.
+
+    % Straightforward implementation of rtc for debugging.
+    %
+:- pred slow_rtc(digraph(T)::in, digraph(T)::out) is det.
+
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+
+:- implementation.
+
 :- import_module bimap.
 :- import_module int.
 :- import_module require.
@@ -1048,14 +1069,23 @@ tc(G, Tc) :-
     % digraph.tc returns the transitive closure of a digraph.
     % We use this procedure:
     %
-    % - Compute the reflexive transitive closure.
+    % - Compute the reflexive transitive closure (denoted G*), which is
+    %   the reflexive closure of G+, the transitive closure of G.
+    %
     % - Find the "fake reflexives", that is, the set of vertices x for which
     %   (x,x) is not an edge in G+. This is done by noting that G+ = G . G*
     %   (where '.' denotes composition). Therefore x is a fake reflexive
     %   iff there is no y such that (x,y) is an edge in G and (y,x) is an edge
     %   in G*.
+    %
     % - Remove those edges from the reflexive transitive closure
     %   computed above.
+    %
+    % XXX Despite being "easier to debug", digraph.rtc was buggy for a long
+    % time, so implementing digraph.tc in terms of digraph.rtc was of no
+    % benefit. We should implement TC using a known efficient algorithm,
+    % then RTC can be implemented trivially on top of TC.
+    %
     digraph.rtc(G, Rtc),
 
     % Find the fake reflexives.
@@ -1099,43 +1129,62 @@ rtc(G, !:Rtc) :-
     % sorted order, compute the RTC for each element in the clique and then
     % add the appropriate edges.
 
-    digraph.dfs(G, Dfs),
+    % Start with G.
+    !:Rtc = G,
+
+    % Visit each clique in turn.
+    digraph.dfsrev(G, DfsRev),
+    digraph.inverse(G, GInv),
     sparse_bitset.init(Vis),
+    digraph.rtc_2(DfsRev, G, GInv, Vis, !Rtc).
 
-    % First start with all the vertices in G, but no edges.
-    G = digraph(NextKey, VMap, _, _),
-    map.init(FwdMap),
-    map.init(BwdMap),
-    !:Rtc = digraph(NextKey, VMap, FwdMap, BwdMap),
-
-    digraph.rtc_2(Dfs, G, Vis, !Rtc).
-
-:- pred digraph.rtc_2(list(digraph_key(T))::in, digraph(T)::in,
+:- pred digraph.rtc_2(list(digraph_key(T))::in, digraph(T)::in, digraph(T)::in,
     digraph_key_set(T)::in, digraph(T)::in, digraph(T)::out) is det.
 
-rtc_2([], _, _, !Rtc).
-rtc_2([X | Xs], G, !.Vis, !Rtc) :-
+rtc_2([], _, _, _, !Rtc).
+rtc_2([X | Xs], G, GInv, !.Vis, !Rtc) :-
     ( if sparse_bitset.contains(!.Vis, X) then
         true
     else
-        digraph.dfs_2(G, X, !Vis, [], CliqList),
+        % Do a DFS on GInv, starting from X, but not including visited
+        % vertices. This gives the clique which includes X.
+        digraph.dfs_2(GInv, X, !Vis, [], CliqList),
         sparse_bitset.list_to_set(CliqList, Cliq),
-        sparse_bitset.foldl(find_followers(G), Cliq,
-            Cliq, Followers0),
-        sparse_bitset.foldl(find_followers(!.Rtc), Followers0,
-            Cliq, Followers),
-        digraph.add_cartesian_product(Cliq, Followers, !Rtc)
-    ),
-    digraph.rtc_2(Xs, G, !.Vis, !Rtc).
 
-:- pred find_followers(digraph(T)::in, digraph_key(T)::in,
+        % Every vertex in the clique is reachable from every other vertex in
+        % the clique.
+        Descendants0 = Cliq,
+
+        % For all vertices in the clique, the set of reachable vertices is the
+        % same.
+        (
+            CliqList = [],
+            Descendants = Descendants0
+        ;
+            CliqList = [Root | _],
+            find_descendants(G, Root, sparse_bitset.init, _Visited,
+                Descendants0, Descendants)
+        ),
+
+        digraph.add_cartesian_product(Cliq, Descendants, !Rtc)
+    ),
+    digraph.rtc_2(Xs, G, GInv, !.Vis, !Rtc).
+
+:- pred find_descendants(digraph(T)::in, digraph_key(T)::in,
+    digraph_key_set(T)::in, digraph_key_set(T)::out,
     digraph_key_set(T)::in, digraph_key_set(T)::out) is det.
 
-find_followers(G, X, !Followers) :-
-    digraph.lookup_key_set_from(G, X, SuccXs),
-    sparse_bitset.union(SuccXs, !Followers).
+find_descendants(G, X, !Visited, !Reachable) :-
+    ( if sparse_bitset.contains(!.Visited, X) then
+        true
+    else
+        digraph.lookup_key_set_from(G, X, SuccXs),
+        sparse_bitset.insert(X, !Visited),
+        sparse_bitset.union(SuccXs, !Reachable),
+        sparse_bitset.foldl2(find_descendants(G), SuccXs, !Visited, !Reachable)
+    ).
 
-:- pred digraph.add_cartesian_product(digraph_key_set(T)::in,
+:- pred add_cartesian_product(digraph_key_set(T)::in,
     digraph_key_set(T)::in, digraph(T)::in, digraph(T)::out) is det.
 
 add_cartesian_product(KeySet1, KeySet2, !Rtc) :-
@@ -1147,42 +1196,73 @@ add_cartesian_product(KeySet1, KeySet2, !Rtc) :-
 %---------------------------------------------------------------------------%
 
 traverse(Graph, ProcessVertex, ProcessEdge, !Acc) :-
-    digraph.keys(Graph, VertexKeys),
-    digraph.traverse_vertex_list(Graph, ProcessVertex, ProcessEdge,
-        VertexKeys, !Acc).
+    VertexMap = Graph ^ vertex_map,
+    bimap.foldl(traverse_vertex(Graph, ProcessVertex, ProcessEdge),
+        VertexMap, !Acc).
 
-:- pred digraph.traverse_vertex_list(digraph(T),
-    pred(T, A, A), pred(T, T, A, A), list(digraph_key(T)), A, A).
-:- mode digraph.traverse_vertex_list(in, pred(in, di, uo) is det,
-    pred(in, in, di, uo) is det, in, di, uo) is det.
-:- mode digraph.traverse_vertex_list(in, pred(in, in, out) is det,
-    pred(in, in, in, out) is det, in, in, out) is det.
+:- pred traverse_vertex(digraph(T),
+    pred(T, A, A), pred(T, T, A, A), T, digraph_key(T), A, A).
+:- mode traverse_vertex(in, pred(in, di, uo) is det,
+    pred(in, in, di, uo) is det, in, in, di, uo) is det.
+:- mode traverse_vertex(in, pred(in, in, out) is det,
+    pred(in, in, in, out) is det, in, in, in, out) is det.
 
-traverse_vertex_list(_, _, _, [], !Acc).
-traverse_vertex_list(Graph, ProcessVertex, ProcessEdge,
-        [VertexKey | VertexKeys], !Acc) :-
-    % XXX avoid the sparse_bitset.to_sorted_list here
-    % (difficult to do using sparse_bitset.foldl because
-    % traverse_children has multiple modes).
-    Vertex = digraph.lookup_vertex(Graph, VertexKey),
+traverse_vertex(Graph, ProcessVertex, ProcessEdge, Vertex, VertexKey, !Acc) :-
     ProcessVertex(Vertex, !Acc),
-    ChildrenKeys = set.to_sorted_list(digraph.lookup_from(Graph, VertexKey)),
-    digraph.traverse_children(Graph, ProcessEdge, Vertex, ChildrenKeys, !Acc),
-    digraph.traverse_vertex_list(Graph, ProcessVertex, ProcessEdge,
-        VertexKeys, !Acc).
+    digraph.lookup_key_set_from(Graph, VertexKey, ChildrenKeys),
+    sparse_bitset.foldl(traverse_child(Graph, ProcessEdge, Vertex),
+        ChildrenKeys, !Acc).
 
-:- pred digraph.traverse_children(digraph(T), pred(T, T, A, A),
-    T, list(digraph_key(T)), A, A).
-:- mode digraph.traverse_children(in, pred(in, in, di, uo) is det,
+:- pred traverse_child(digraph(T), pred(T, T, A, A),
+    T, digraph_key(T), A, A).
+:- mode traverse_child(in, pred(in, in, di, uo) is det,
     in, in, di, uo) is det.
-:- mode digraph.traverse_children(in, pred(in, in, in, out) is det,
+:- mode traverse_child(in, pred(in, in, in, out) is det,
     in, in, in, out) is det.
 
-traverse_children(_, _, _, [], !Acc).
-traverse_children(Graph, ProcessEdge, Parent, [ChildKey | ChildKeys], !Acc) :-
+traverse_child(Graph, ProcessEdge, Parent, ChildKey, !Acc) :-
     Child = digraph.lookup_vertex(Graph, ChildKey),
-    ProcessEdge(Parent, Child, !Acc),
-    digraph.traverse_children(Graph, ProcessEdge, Parent, ChildKeys, !Acc).
+    ProcessEdge(Parent, Child, !Acc).
+
+%---------------------------------------------------------------------------%
+
+slow_tc(G, TC) :-
+    % First start with all the vertices in G, but no edges.
+    G = digraph(NextKey, VMap, FwdMap, _BwdMap),
+    TC0 = digraph(NextKey, VMap, map.init, map.init),
+
+    map.keys(FwdMap, FwdKeys),
+    list.foldl(add_edges_to_reachable(G), FwdKeys, TC0, TC).
+
+:- pred add_edges_to_reachable(digraph(T)::in, int::in,
+    digraph(T)::in, digraph(T)::out) is det.
+
+add_edges_to_reachable(G, XI, !TC) :-
+    X = digraph_key(XI),
+    find_descendants(G, X,
+        sparse_bitset.init, _Visited,
+        sparse_bitset.init, Reachable),
+    sparse_bitset.foldl(add_edge(X), Reachable, !TC).
+
+%---------------------------------------------------------------------------%
+
+slow_rtc(G, RTC) :-
+    slow_tc(G, TC),
+    rc(TC, RTC).
+
+    % Reflexive closure.
+    %
+:- pred rc(digraph(T)::in, digraph(T)::out) is det.
+
+rc(G, RC) :-
+    digraph.keys(G, Keys),
+    list.foldl(add_reflexive, Keys, G, RC).
+
+:- pred add_reflexive(digraph_key(T)::in,
+    digraph(T)::in, digraph(T)::out) is det.
+
+add_reflexive(X, !G) :-
+    add_edge(X, X, !G).
 
 %---------------------------------------------------------------------------%
 :- end_module digraph.
