@@ -1,9 +1,9 @@
 %---------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
+% vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
 % Copyright (C) 2001-2006, 2009, 2011 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% Copyright (C) 2014-2015, 2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %---------------------------------------------------------------------------%
 %
 % File: robdd.m.
@@ -47,8 +47,8 @@
 % included in the publically available part of the library, so at the moment
 % it is not included in the list of modules mentioned in the library manual.
 %
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module robdd.
 :- interface.
@@ -102,7 +102,7 @@
     %
 :- func (~ robdd(T)) = robdd(T).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % var(X) is the ROBDD that is true iff X is true.
     %
@@ -157,7 +157,7 @@
     %
 :- func var_restrict_false(var(T), robdd(T)) = robdd(T).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % X `entails` Y:
     %
@@ -296,11 +296,11 @@
     %
 :- func remove_implications(imp_vars(T), robdd(T)) = robdd(T).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type literal(T)
     --->    pos(var(T))
-    ;   neg(var(T)).
+    ;       neg(var(T)).
 
     % Convert the ROBDD to disjunctive normal form.
     %
@@ -310,28 +310,35 @@
 %   %
 % :- func cnf(robdd(T)) = list(list(literal(T))).
 
-    % Print out the ROBDD in disjunctive normal form.
+    % Print out the ROBDD in disjunctive normal form. either to the
+    % current output stream, or to the specified output stream.
     %
-:- pred print_robdd(robdd(T)::in, io::di, io::uo) is det.
+:- pred print_robdd(robdd(T)::in,
+    io::di, io::uo) is det.
+:- pred print_robdd(io.text_output_stream::in, robdd(T)::in,
+    io::di, io::uo) is det.
+
+:- type var_to_string(T) == (func(var(T)) = string).
 
     % robdd_to_dot(ROBDD, WriteVar, FileName, !IO):
     %
     % Output the ROBDD in a format that can be processed by the
-    % graph-drawing program `dot'.
+    % graph-drawing program `dot' to the specified filename.
     %
-:- pred robdd_to_dot(robdd(T)::in, write_var(T)::in(write_var), string::in,
-    io::di, io::uo) is det.
+:- pred robdd_to_dot(robdd(T)::in,
+    var_to_string(T)::in, string::in, io::di, io::uo) is det.
 
     % robdd_to_dot(ROBDD, WriteVar, !IO):
+    % robdd_to_dot_stream(Stream, ROBDD, WriteVar, !IO):
     %
     % Output the ROBDD in a format that can be processed by the
-    % graph-drawing program `dot'.
+    % graph-drawing program `dot', either to the current output stream,
+    % or to the specified output stream.
     %
-:- pred robdd_to_dot(robdd(T)::in, write_var(T)::in(write_var),
-    io::di, io::uo) is det.
-
-:- type write_var(T) == pred(var(T), io, io).
-:- inst write_var == (pred(in, di, uo) is det).
+:- pred robdd_to_dot(robdd(T)::in,
+    var_to_string(T)::in, io::di, io::uo) is det.
+:- pred robdd_to_dot_stream(io.text_output_stream::in, robdd(T)::in,
+    var_to_string(T)::in, io::di, io::uo) is det.
 
     % Apply the variable substitution to the ROBDD.
     %
@@ -358,7 +365,7 @@
     %
 :- pred vars_are_constrained(robdd(T)::in, vars(T)::in) is semidet.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % labelling(Vars, ROBDD, TrueVars, FalseVars):
     %
@@ -388,7 +395,7 @@
 :- pred minimal_model(vars(T)::in, robdd(T)::in, vars(T)::out, vars(T)::out)
     is nondet.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Zero the internal caches used for ROBDD operations.
     % This allows nodes in the caches to be garbage-collected.
@@ -400,14 +407,13 @@
 
 :- impure pred clear_caches is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 :- import_module assoc_list.
 :- import_module bool.
-:- import_module hash_table.
 :- import_module int.
 :- import_module multi_map.
 :- import_module pair.
@@ -435,11 +441,7 @@ empty_vars_set = sparse_bitset.init.
 
 #include ""bryant.h""
 
-#ifdef  MR_HIGHLEVEL_DATA
-  typedef   MR_Box  MR_ROBDD_NODE_TYPE;
-#else
-  typedef   MR_Word MR_ROBDD_NODE_TYPE;
-#endif
+typedef   MR_Word MR_ROBDD_NODE_TYPE;
 ").
 
 :- pragma foreign_code("C", "
@@ -447,7 +449,7 @@ empty_vars_set = sparse_bitset.init.
 #include ""bryant.c""
 ").
 
-:- pragma no_inline(one/0).
+:- pragma no_inline(func(one/0)).
 :- pragma foreign_proc("C",
     one = (F::out),
     [will_not_call_mercury, promise_pure],
@@ -455,7 +457,7 @@ empty_vars_set = sparse_bitset.init.
     F = (MR_ROBDD_NODE_TYPE) MR_ROBDD_trueVar();
 ").
 
-:- pragma no_inline(zero/0).
+:- pragma no_inline(func(zero/0)).
 :- pragma foreign_proc("C",
     zero = (F::out),
     [will_not_call_mercury, promise_pure],
@@ -463,7 +465,7 @@ empty_vars_set = sparse_bitset.init.
     F = (MR_ROBDD_NODE_TYPE) MR_ROBDD_falseVar();
 ").
 
-:- pragma no_inline(var/1).
+:- pragma no_inline(func(var/1)).
 :- pragma foreign_proc("C",
     var(V::in) = (F::out),
     [will_not_call_mercury, promise_pure],
@@ -471,7 +473,7 @@ empty_vars_set = sparse_bitset.init.
     F = (MR_ROBDD_NODE_TYPE) MR_ROBDD_variableRep(V);
 ").
 
-:- pragma no_inline(ite/3).
+:- pragma no_inline(func(ite/3)).
 :- pragma foreign_proc("C",
     ite(F::in, G::in, H::in) = (ITE::out),
     [will_not_call_mercury, promise_pure],
@@ -480,7 +482,7 @@ empty_vars_set = sparse_bitset.init.
         (MR_ROBDD_node *) G, (MR_ROBDD_node *) H);
 ").
 
-:- pragma no_inline(ite_var/3).
+:- pragma no_inline(func(ite_var/3)).
 :- pragma foreign_proc("C",
     ite_var(V::in, G::in, H::in) = (ITE::out),
     [will_not_call_mercury, promise_pure],
@@ -489,15 +491,15 @@ empty_vars_set = sparse_bitset.init.
         (MR_ROBDD_node *) H);
 ").
 
-:- pragma promise_pure('*'/2).
+:- pragma promise_pure(func('*'/2)).
 X * Y = R :-
     R = glb(X, Y),
 
     % XXX debugging code.
-    %( R = zero ->
-    ( (X = zero ; Y = zero) ->
+    % ( if R = zero then
+    ( if (X = zero ; Y = zero) then
         impure report_zero_constraint
-    ;
+    else
         true
     ).
 
@@ -519,7 +521,7 @@ X * Y = R :-
     fprintf(stderr, ""Zero constraint!!!\\n"");
 ").
 
-:- pragma no_inline((+)/2).
+:- pragma no_inline(func((+)/2)).
 :- pragma foreign_proc("C",
     (X::in) + (Y::in) = (F::out),
     [will_not_call_mercury, promise_pure],
@@ -528,7 +530,7 @@ X * Y = R :-
         (MR_ROBDD_node *) Y);
 ").
 
-:- pragma no_inline((=<)/2).
+:- pragma no_inline(func((=<)/2)).
 :- pragma foreign_proc("C",
     ((X::in) =< (Y::in)) = (F::out),
     [will_not_call_mercury, promise_pure],
@@ -543,7 +545,7 @@ X * Y = R :-
 
 (~F) = ite(F, zero, one).
 
-:- pragma no_inline(entails/2).
+:- pragma no_inline(pred(entails/2)).
 :- pragma foreign_proc("C",
     entails(X::in, Y::in),
     [will_not_call_mercury, promise_pure],
@@ -552,7 +554,7 @@ X * Y = R :-
         (MR_ROBDD_node *) Y, MR_ROBDD_one) == MR_ROBDD_one);
 ").
 
-:- pragma no_inline(var_entailed/2).
+:- pragma no_inline(pred(var_entailed/2)).
 :- pragma foreign_proc("C",
     var_entailed(F::in, V::in),
     [will_not_call_mercury, promise_pure],
@@ -564,15 +566,15 @@ X * Y = R :-
 % :- pragma memo(vars_entailed/1).
 
 vars_entailed(R) =
-    ( R = one ->
+    ( if R = one then
         some_vars(empty_vars_set)
-    ; R = zero ->
+    else if R = zero then
         all_vars
-    ;
-        ( R ^ fa = zero ->
+    else
+        ( if R ^ fa = zero then
             (vars_entailed(R ^ tr) `intersection` vars_entailed(R ^ fa))
                 `insert` R ^ value
-        ;
+        else
             vars_entailed(R ^ tr) `intersection` vars_entailed(R ^ fa)
         )
     ).
@@ -580,15 +582,15 @@ vars_entailed(R) =
 % :- pragma memo(vars_disentailed/1).
 
 vars_disentailed(R) =
-    ( R = one ->
+    ( if R = one then
         some_vars(empty_vars_set)
-    ; R = zero ->
+    else if R = zero then
         all_vars
-    ;
-        ( R ^ tr = zero ->
+    else
+        ( if R ^ tr = zero then
             (vars_disentailed(R ^ tr) `intersection`
                 vars_disentailed(R ^ fa)) `insert` R ^ value
-        ;
+        else
             vars_disentailed(R ^ tr) `intersection`
                 vars_disentailed(R ^ fa)
         )
@@ -597,24 +599,24 @@ vars_disentailed(R) =
 % :- pragma memo(definite_vars/3).
 
 definite_vars(R, T, F) :-
-    ( R = one ->
+    ( if R = one then
         T = some_vars(empty_vars_set),
         F = some_vars(empty_vars_set)
-    ; R = zero ->
+    else if R = zero then
         T = all_vars,
         F = all_vars
-    ;
+    else
         definite_vars(R ^ tr, T_tr, F_tr),
         definite_vars(R ^ fa, T_fa, F_fa),
         T0 = T_tr `intersection` T_fa,
         F0 = F_tr `intersection` F_fa,
-        ( R ^ fa = zero ->
+        ( if R ^ fa = zero then
             T = T0 `insert` R ^ value,
             F = F0
-        ; R ^ tr = zero ->
+        else if R ^ tr = zero then
             T = T0,
             F = F0 `insert` R ^ value
-        ;
+        else
             T = T0,
             F = F0
         )
@@ -630,11 +632,11 @@ equivalent_vars(R) = rev_map(equivalent_vars_2(R)).
 % :- pragma memo(equivalent_vars_2/1).
 
 equivalent_vars_2(R) = EQ :-
-    ( R = one ->
+    ( if R = one then
         EQ = some_vars(leader_to_eqvclass(map.init))
-    ; R = zero ->
+    else if R = zero then
         EQ = all_vars
-    ;
+    else
         EQVars = vars_entailed(R ^ tr) `intersection`
                 vars_disentailed(R ^ fa),
         EQ0 = equivalent_vars_2(R ^ tr) `intersection`
@@ -647,9 +649,9 @@ equivalent_vars_2(R) = EQ :-
             % both branches of R must have been zero.
         ;
             EQVars = some_vars(Vars),
-            ( empty(Vars) ->
+            ( if is_empty(Vars) then
                 EQ = EQ0
-            ;
+            else
                 (
                     EQ0 = all_vars,
                     error("equivalent_vars: unexpected result")
@@ -675,9 +677,9 @@ rev_map(all_vars) = all_vars.
 rev_map(some_vars(leader_to_eqvclass(EQ0))) = some_vars(equiv_vars(EQ)) :-
     map.foldl2(
         ( pred(V::in, Vs::in, Seen0::in, Seen::out, in, out) is det -->
-            ( { Seen0 `contains` V } ->
+            ( if { Seen0 `contains` V } then
                 { Seen = Seen0 }
-            ;
+            else
                 ^ elem(V) := V,
                 sparse_bitset.foldl((pred(Ve::in, in, out) is det -->
                     ^ elem(Ve) := V
@@ -704,17 +706,17 @@ extract_implications(R) = implication_result_to_imp_vars(implications_2(R)).
 % :- pragma memo(implications_2/1).
 
 implications_2(R) = implication_result(Imps, RevImps, DisImps, RevDisImps) :-
-    ( R = one ->
+    ( if R = one then
         Imps = some_vars(imps(map.init)),
         RevImps = Imps,
         DisImps = Imps,
         RevDisImps = Imps
-    ; R = zero ->
+    else if R = zero then
         Imps = all_vars,
         RevImps = Imps,
         DisImps = Imps,
         RevDisImps = Imps
-    ;
+    else
         TTVars = vars_entailed(R ^ tr),
         FFVars = vars_disentailed(R ^ fa),
         TFVars = vars_disentailed(R ^ tr),
@@ -759,8 +761,8 @@ merge_imp_res_2(EntailedVarsA, EntailedVarsB, imps(ImpsA), imps(ImpsB)) =
     KeysB = map.sorted_keys(ImpsB),
     Keys = list.merge_and_remove_dups(KeysA, KeysB),
     Imps = list.foldl((func(V, M) = M ^ elem(V) := VsA `intersection` VsB :-
-            VsA = ( VsA0 = ImpsA ^ elem(V) -> VsA0 ; EntailedVarsA ),
-            VsB = ( VsB0 = ImpsB ^ elem(V) -> VsB0 ; EntailedVarsB )
+            VsA = ( if VsA0 = ImpsA ^ elem(V) then VsA0 else EntailedVarsA ),
+            VsB = ( if VsB0 = ImpsB ^ elem(V) then VsB0 else EntailedVarsB )
         ), Keys, map.init).
 
 :- func implication_result_to_imp_vars(implication_result(T)) = imp_vars(T).
@@ -778,12 +780,12 @@ implication_result_to_imp_vars(ImpRes) = ImpVars :-
 imp_res_to_imp_map(all_vars) = map.init.
 imp_res_to_imp_map(some_vars(imps(IRMap))) =
     map.foldl(func(V, MaybeVs, M) =
-        (
+        ( if
             MaybeVs = some_vars(Vs),
-            \+ empty(Vs)
-        ->
+            is_non_empty(Vs)
+        then
             M ^ elem(V) := Vs
-        ;
+        else
             M
         ), IRMap, init).
 
@@ -796,15 +798,15 @@ remove_implications(ImpRes, R0) = R :-
     robdd_cache(T)::in, robdd_cache(T)::out) is det.
 
 remove_implications_2(ImpRes, True, False, R0, R) -->
-    ( { is_terminal(R0) } ->
+    ( if { is_terminal(R0) } then
         { R = R0 }
-    ; { True `contains` R0 ^ value } ->
+    else if { True `contains` R0 ^ value } then
         remove_implications_2(ImpRes, True, False, R0 ^ tr, R)
-    ; { False `contains` R0 ^ value } ->
+    else if { False `contains` R0 ^ value } then
         remove_implications_2(ImpRes, True, False, R0 ^ fa, R)
-    ; R1 =^ elem(R0) ->
+    else if R1 =^ elem(R0) then
         { R = R1 }
-    ;
+    else
         { TrueT = True `union` ImpRes ^ imps ^ get(R0 ^ value) },
         { FalseT = False `union` ImpRes ^ dis_imps ^ get(R0 ^ value) },
         remove_implications_2(ImpRes, TrueT, FalseT, R0 ^ tr, RT),
@@ -820,10 +822,10 @@ remove_implications_2(ImpRes, True, False, R0, R) -->
 :- func get(var(T), imp_map(T)) = vars(T).
 
 get(K, IM) =
-    ( Vs = IM ^ elem(K) ->
+    ( if Vs = IM ^ elem(K) then
         % In case Vs doesn't already contain K
         Vs `insert` K
-    ;
+    else
         init
     ).
 
@@ -845,13 +847,13 @@ get(K, IM) =
 :- instance intersectable(leader_to_eqvclass(T)) where [
     ( leader_to_eqvclass(MapA) `intersection` leader_to_eqvclass(MapB) =
         leader_to_eqvclass(map.foldl((func(V, VsA, M) =
-            ( Vs = VsA `intersect` (MapB ^ elem(V)) ->
-                ( empty(Vs) ->
+            ( if Vs = VsA `intersect` (MapB ^ elem(V)) then
+                ( if is_empty(Vs) then
                     M
-                ;
+                else
                     M ^ elem(V) := Vs
                 )
-            ;
+            else
                 M
             )), MapA, map.init))
     )
@@ -879,7 +881,7 @@ some_vars(Vs) `insert` V = some_vars(Vs `insert` V).
 :- func tr(robdd(T)) = robdd(T).
 :- func fa(robdd(T)) = robdd(T).
 
-:- pragma no_inline(value/1).
+:- pragma no_inline(func(value/1)).
 :- pragma foreign_proc("C",
     value(F::in) = (Value::out),
     [will_not_call_mercury, promise_pure],
@@ -887,7 +889,7 @@ some_vars(Vs) `insert` V = some_vars(Vs `insert` V).
     Value = (MR_ROBDD_NODE_TYPE) ((MR_ROBDD_node *) F)->value;
 ").
 
-:- pragma no_inline(tr/1).
+:- pragma no_inline(func(tr/1)).
 :- pragma foreign_proc("C",
     tr(F::in) = (Tr::out),
     [will_not_call_mercury, promise_pure],
@@ -895,7 +897,7 @@ some_vars(Vs) `insert` V = some_vars(Vs `insert` V).
     Tr = (MR_ROBDD_NODE_TYPE) ((MR_ROBDD_node *) F)->tr;
 ").
 
-:- pragma no_inline(fa/1).
+:- pragma no_inline(func(fa/1)).
 :- pragma foreign_proc("C",
     fa(F::in) = (Fa::out),
     [will_not_call_mercury, promise_pure],
@@ -903,26 +905,26 @@ some_vars(Vs) `insert` V = some_vars(Vs `insert` V).
     Fa = (MR_ROBDD_NODE_TYPE) ((MR_ROBDD_node *) F)->fa;
 ").
 
-%------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 % :- pragma memo(dnf/1).
 
 dnf(R) =
-    ( R = zero ->
+    ( if R = zero then
         []
-    ; R = one ->
+    else if R = one then
         [[]]
-    ;
+    else
         list.map(func(L) = [pos(R ^ value) | L], dnf(R ^ tr)) ++
         list.map(func(L) = [neg(R ^ value) | L], dnf(R ^ fa))
     ).
 
 % cnf(R) =
-%   ( R = zero ->
+%   ( if R = zero then
 %       [[]]
-%   ; R = one ->
+%   else if R = one then
 %       []
-%   ;
+%   else
 %       [pos(R ^ value) | cnf(R ^ tr)] `merge_cnf`
 %       [neg(R ^ value) | cnf(R ^ fa)]
 %   ).
@@ -935,7 +937,7 @@ dnf(R) =
 %       Bs
 %   ; Bs = [] ->
 %       As
-%   ; As = [[]] ->
+%   ; if As = [[]] ->
 %       As
 %   ; Bs = [[]] % XXX check
 %   ;
@@ -952,43 +954,49 @@ dnf(R) =
 %   update_io(IO0, IO);
 % ").
 
-print_robdd(F) -->
-    ( { F = one } ->
-        io.write_string("TRUE\n")
-    ; { F = zero } ->
-        io.write_string("FALSE\n")
-    ;
-        { init(Trues) },
-        { init(Falses) },
-        print_robdd_2(F, Trues, Falses)
+print_robdd(F, !IO) :-
+    io.output_stream(Stream, !IO),
+    print_robdd(Stream, F, !IO).
+
+print_robdd(Stream, F, !IO) :-
+    ( if F = one then
+        io.write_string(Stream, "TRUE\n", !IO)
+    else if F = zero then
+        io.write_string(Stream, "FALSE\n", !IO)
+    else
+        init(Trues),
+        init(Falses),
+        print_robdd_2(Stream, F, Trues, Falses, !IO)
     ).
 
-:- pred print_robdd_2(robdd(T)::in, set_unordlist(var(T))::in,
-    set_unordlist(var(T))::in, io::di, io::uo) is det.
+:- pred print_robdd_2(io.text_output_stream::in, robdd(T)::in,
+    set_unordlist(var(T))::in, set_unordlist(var(T))::in,
+    io::di, io::uo) is det.
 
-print_robdd_2(F, Trues, Falses) -->
-    ( { F = one } ->
-        { All = to_sorted_list(Trues `union` Falses) },
-        io.write_string("("),
-        list.foldl((pred(Var::in, di, uo) is det -->
-            { Var `set_unordlist.member` Trues ->
-                C = ' '
-            ;
-                C = ('~')
-            },
-            { term.var_to_int(Var, N) },
-            io.format(" %c%02d", [c(C), i(N)])
-        ), All),
-        io.write_string(")\n")
-    ; { F \= zero } ->
-        print_robdd_2(F ^ tr, Trues `insert` F ^ value, Falses),
-        print_robdd_2(F ^ fa, Trues, Falses `insert` F ^ value)
-    ;
-        % Don't do anything for zero terminal
-        []
+print_robdd_2(Stream, F, Trues, Falses, !IO) :-
+    ( if F = one then
+        All = to_sorted_list(Trues `union` Falses),
+        io.write_string(Stream, "(", !IO),
+        list.foldl(
+            ( pred(Var::in, IO0::di, IO::uo) is det :-
+                ( if Var `set_unordlist.member` Trues then
+                    C = ' '
+                else
+                    C = ('~')
+                ),
+                term.var_to_int(Var, N),
+                io.format(Stream, " %c%02d", [c(C), i(N)], IO0, IO)
+            ), All, !IO),
+        io.write_string(Stream, ")\n", !IO)
+    else if F = zero then
+        % Don't do anything for zero terminal.
+        true
+    else
+        print_robdd_2(Stream, F ^ tr, Trues `insert` F ^ value, Falses, !IO),
+        print_robdd_2(Stream, F ^ fa, Trues, Falses `insert` F ^ value, !IO)
     ).
 
-:- pragma no_inline(restrict/2).
+:- pragma no_inline(func(restrict/2)).
 :- pragma foreign_proc("C",
     restrict(V::in, F::in) = (R::out),
     [will_not_call_mercury, promise_pure],
@@ -996,7 +1004,7 @@ print_robdd_2(F, Trues, Falses) -->
     R = (MR_ROBDD_NODE_TYPE) MR_ROBDD_restrict(V, (MR_ROBDD_node *) F);
 ").
 
-:- pragma no_inline(restrict_threshold/2).
+:- pragma no_inline(func(restrict_threshold/2)).
 :- pragma foreign_proc("C",
     restrict_threshold(V::in, F::in) = (R::out),
     [will_not_call_mercury, promise_pure],
@@ -1008,9 +1016,9 @@ print_robdd_2(F, Trues, Falses) -->
 % :- pragma memo(rename_vars/2).
 
 rename_vars(Subst, F) =
-    ( is_terminal(F) ->
+    ( if is_terminal(F) then
         F
-    ;
+    else
         ite(var(Subst(F ^ value)),
             rename_vars(Subst, F ^ tr),
             rename_vars(Subst, F ^ fa))
@@ -1022,7 +1030,7 @@ rename_vars(Subst, F) =
 % variables in both the Then and Else sub graphs are > Var.
 
 :- func make_node(var(T), robdd(T), robdd(T)) = robdd(T).
-:- pragma no_inline(make_node/3).
+:- pragma no_inline(func(make_node/3)).
 :- pragma foreign_proc("C",
     make_node(Var::in, Then::in, Else::in) = (Node::out),
     [will_not_call_mercury, promise_pure],
@@ -1093,9 +1101,9 @@ at_most_one_of_2(Vars, OneOf0, NoneOf0) = R :-
 % :- pragma memo(var_restrict_true/2).
 
 var_restrict_true(V, F0) = F :-
-    ( is_terminal(F0) ->
+    ( if is_terminal(F0) then
         F = F0
-    ;
+    else
         compare(R, F0 ^ value, V),
         (
             R = (<),
@@ -1114,9 +1122,9 @@ var_restrict_true(V, F0) = F :-
 % :- pragma memo(var_restrict_false/2).
 
 var_restrict_false(V, F0) = F :-
-    ( is_terminal(F0) ->
+    ( if is_terminal(F0) then
         F = F0
-    ;
+    else
         compare(R, F0 ^ value, V),
         (
             R = (<),
@@ -1146,26 +1154,26 @@ restrict_true_false_vars(TrueVars, FalseVars, R0) = R :-
     robdd_cache(T)::in, robdd_cache(T)::out) is det.
 
 restrict_true_false_vars_2(TrueVars0, FalseVars0, R0, R, Seen0, Seen) :-
-    ( is_terminal(R0) ->
+    ( if is_terminal(R0) then
         R = R0,
         Seen = Seen0
-    ; empty(TrueVars0), empty(FalseVars0) ->
+    else if is_empty(TrueVars0), is_empty(FalseVars0) then
         R = R0,
         Seen = Seen0
-    ; search(Seen0, R0, R1) ->
+    else if search(Seen0, R0, R1) then
         R = R1,
         Seen = Seen0
-    ;
+    else
         Var = R0 ^ value,
         TrueVars = TrueVars0 `remove_leq` Var,
         FalseVars = FalseVars0 `remove_leq` Var,
-        ( TrueVars0 `contains` Var ->
+        ( if TrueVars0 `contains` Var then
             restrict_true_false_vars_2(TrueVars, FalseVars,
                 R0 ^ tr, R, Seen0, Seen2)
-        ; FalseVars0 `contains` Var ->
+        else if FalseVars0 `contains` Var then
             restrict_true_false_vars_2(TrueVars, FalseVars,
                 R0 ^ fa, R, Seen0, Seen2)
-        ;
+        else
             restrict_true_false_vars_2(TrueVars, FalseVars,
                 R0 ^ tr, R_tr, Seen0, Seen1),
             restrict_true_false_vars_2(TrueVars, FalseVars,
@@ -1175,11 +1183,6 @@ restrict_true_false_vars_2(TrueVars0, FalseVars0, R0, R, Seen0, Seen) :-
         Seen = det_insert(Seen2, R0, R)
     ).
 
-:- pred robdd_hash(robdd(T)::in, int::out) is det.
-
-robdd_hash(R, H) :-
-    int_hash(node_num(R), H).
-
 restrict_filter(P, F0) =
     restrict_filter(P, (pred(_::in) is semidet :- true), F0).
 
@@ -1195,25 +1198,25 @@ restrict_filter(P, D, F0) = F :-
     in, out) is det.
 
 filter_2(P, D, F0, F, SeenVars0, SeenVars, SeenNodes0, SeenNodes) :-
-    ( is_terminal(F0) ->
+    ( if is_terminal(F0) then
         F = F0,
         SeenVars = SeenVars0,
         SeenNodes = SeenNodes0
-    ; \+ D(F0 ^ value) ->
+    else if not D(F0 ^ value) then
         F = F0,
         SeenVars = SeenVars0,
         SeenNodes = SeenNodes0
-    ; map.search(SeenNodes0, F0, F1) ->
+    else if map.search(SeenNodes0, F0, F1) then
         F = F1,
         SeenVars = SeenVars0,
         SeenNodes = SeenNodes0
-    ;
+    else
         filter_2(P, D, F0 ^ tr, Ftrue, SeenVars0, SeenVars1, SeenNodes0,
             SeenNodes1),
         filter_2(P, D, F0 ^ fa, Ffalse, SeenVars1, SeenVars2, SeenNodes1,
             SeenNodes2),
         V = F0 ^ value,
-        ( map.search(SeenVars0, V, SeenF) ->
+        ( if map.search(SeenVars0, V, SeenF) then
             SeenVars = SeenVars2,
             (
                 SeenF = yes,
@@ -1222,10 +1225,10 @@ filter_2(P, D, F0, F, SeenVars0, SeenVars, SeenNodes0, SeenNodes) :-
                 SeenF = no,
                 F = Ftrue + Ffalse
             )
-        ; P(V) ->
+        else if P(V) then
             F = make_node(V, Ftrue, Ffalse),
             map.det_insert(V, yes, SeenVars2, SeenVars)
-        ;
+        else
             F = Ftrue + Ffalse,
             map.det_insert(V, no, SeenVars2, SeenVars)
         ),
@@ -1233,15 +1236,15 @@ filter_2(P, D, F0, F, SeenVars0, SeenVars, SeenNodes0, SeenNodes) :-
     ).
 
 squeeze_equiv(equiv_vars(LeaderMap), R0) =
-    ( Max = map.max_key(LeaderMap) ->
+    ( if Max = map.max_key(LeaderMap) then
         restrict_filter(
             ( pred(V::in) is semidet :-
                 map.search(LeaderMap, V, L) => L = V
             ),
             ( pred(V::in) is semidet :-
-                \+ compare(>, V, Max)
+                not compare(>, V, Max)
             ), R0)
-    ;
+    else
         R0
     ).
 
@@ -1252,12 +1255,12 @@ make_equiv(equiv_vars(LeaderMap)) =
 
 make_equiv_2([], _) = one.
 make_equiv_2([Var - LeaderVar | Vs], Trues) = Robdd :-
-    ( Var = LeaderVar ->
+    ( if Var = LeaderVar then
         Robdd = make_node(Var, make_equiv_2(Vs, Trues `insert` Var),
             make_equiv_2(Vs, Trues))
-    ; Trues `contains` LeaderVar ->
+    else if Trues `contains` LeaderVar then
         Robdd = make_node(Var, make_equiv_2(Vs, Trues), zero)
-    ;
+    else
         var_to_int(Var, VarNum),
         var_to_int(LeaderVar, LeaderVarNum),
         require(LeaderVarNum < VarNum, "make_equiv_2: unordered vars"),
@@ -1278,14 +1281,14 @@ add_equivalences(equiv_vars(LeaderMap), R0) = R :-
 
 add_equivalences_2([], _, R, R, !Cache).
 add_equivalences_2([Var - LeaderVar | Vs], Trues, R0, R, !Cache) :-
-    ( R0 = zero ->
+    ( if R0 = zero then
         R = zero
-    ; R1 = !.Cache ^ elem(R0) ->
+    else if R1 = !.Cache ^ elem(R0) then
         R = R1
-    ; R0 = one ->
+    else if R0 = one then
         R = make_equiv_2([Var - LeaderVar | Vs], Trues),
         !:Cache = !.Cache ^ elem(R0) := R
-    ; compare((<), R0 ^ value, Var) ->
+    else if compare((<), R0 ^ value, Var) then
         add_equivalences_2([Var - LeaderVar | Vs], Trues,
             R0 ^ tr, Rtr, !Cache),
         add_equivalences_2([Var - LeaderVar | Vs], Trues,
@@ -1293,19 +1296,19 @@ add_equivalences_2([Var - LeaderVar | Vs], Trues, R0, R, !Cache) :-
         % This step can make R exponentially bigger than R0.
         R = make_node(R0 ^ value, Rtr, Rfa),
         !:Cache = !.Cache ^ elem(R0) := R
-    ; compare((<), Var, R0 ^ value) ->
-        ( LeaderVar = Var ->
+    else if compare((<), Var, R0 ^ value) then
+        ( if LeaderVar = Var then
             add_equivalences_2(Vs, Trues `insert` Var,
                 R0, Rtr, !Cache),
             add_equivalences_2(Vs, Trues, R0, Rfa, !Cache),
             % This step can make R exponentially bigger than R0.
             R = make_node(Var, Rtr, Rfa),
             !:Cache = !.Cache ^ elem(R0) := R
-        ; Trues `contains` LeaderVar ->
+        else if Trues `contains` LeaderVar then
             add_equivalences_2(Vs, Trues, R0, Rtr, !Cache),
             R = make_node(Var, Rtr, zero),
             !:Cache = !.Cache ^ elem(R0) := R
-        ;
+        else
             add_equivalences_2(Vs, Trues, R0, Rfa, !Cache),
             % Since LeaderVar < Var, and every leader must
             % appear in an equivalence with itself, an ancestor
@@ -1315,16 +1318,16 @@ add_equivalences_2([Var - LeaderVar | Vs], Trues, R0, R, !Cache) :-
             R = make_node(Var, zero, Rfa),
             !:Cache = !.Cache ^ elem(R0) := R
         )
-    ; LeaderVar = Var ->
+    else if LeaderVar = Var then
         add_equivalences_2(Vs, Trues `insert` Var, R0 ^ tr, Rtr, !Cache),
         add_equivalences_2(Vs, Trues, R0 ^ fa, Rfa, !Cache),
         R = make_node(Var, Rtr, Rfa),
         !:Cache = !.Cache ^ elem(R0) := R
-    ; Trues `contains` LeaderVar ->
+    else if Trues `contains` LeaderVar then
         add_equivalences_2(Vs, Trues, R0 ^ tr, Rtr, !Cache),
         R = make_node(Var, Rtr, zero),
         !:Cache = !.Cache ^ elem(R0) := R
-    ;
+    else
         add_equivalences_2(Vs, Trues, R0 ^ fa, Rfa, !Cache),
         % Since LeaderVar < Var, and every leader must
         % appear in an equivalence with itself, an ancestor
@@ -1335,7 +1338,7 @@ add_equivalences_2([Var - LeaderVar | Vs], Trues, R0, R, !Cache) :-
         !:Cache = !.Cache ^ elem(R0) := R
     ).
 
-%------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 % XXX this could be made much more efficient by doing something similar
 % to what we do in add_equivalences.
@@ -1355,9 +1358,9 @@ add_implications_2(FA, FB, IM, R0) =
         foldl(func(VB, R2) = R2 * (FA(VA) + FB(VB)), Vs, R1),
         IM, R0).
 
-%------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-:- pragma no_inline(is_terminal/1).
+:- pragma no_inline(pred(is_terminal/1)).
 :- pragma foreign_proc("C",
     is_terminal(F::in),
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -1377,13 +1380,13 @@ size(F, Nodes, Depth, Vars) :-
         set_bbbtree(robdd(T))::in, set_bbbtree(robdd(T))::out) is det.
 
 size_2(F, Nodes0, Nodes, Depth0, Depth, Val0, Seen0, Seen) :-
-    ( is_terminal(F) ->
+    ( if is_terminal(F) then
         Nodes = Nodes0, Depth = Depth0, Seen = Seen0
-    ; term.var_to_int(F ^ value) =< Val0 ->
+    else if term.var_to_int(F ^ value) =< Val0 then
         error("robdd invariant broken (possible loop)")
-    ; F `member` Seen0 ->
+    else if F `member` Seen0 then
         Nodes = Nodes0, Depth = Depth0, Seen = Seen0
-    ;
+    else
         Val = term.var_to_int(F ^ value),
         size_2(F ^ tr, Nodes0+1, Nodes1, Depth0, Depth1, Val, Seen0, Seen1),
         size_2(F ^ fa, Nodes1, Nodes, Depth0, Depth2, Val, Seen1, Seen2),
@@ -1393,9 +1396,9 @@ size_2(F, Nodes0, Nodes, Depth0, Depth, Val0, Seen0, Seen) :-
     ).
 
 var_is_constrained(F, V) :-
-    ( is_terminal(F) ->
+    ( if is_terminal(F) then
         fail
-    ;
+    else
         compare(R, F ^ value, V),
         (
             R = (<),
@@ -1415,9 +1418,9 @@ vars_are_constrained(F, Vs) :-
 vars_are_constrained_2(_, []).
 vars_are_constrained_2(F, Vs) :-
     Vs = [V | Vs1],
-    ( is_terminal(F) ->
+    ( if is_terminal(F) then
         fail
-    ;
+    else
         compare(R, F ^ value, V),
         (
             R = (<),
@@ -1431,81 +1434,87 @@ vars_are_constrained_2(F, Vs) :-
         )
     ).
 
-robdd_to_dot(Robdd, WV, Filename) -->
-    io.tell(Filename, Result),
+robdd_to_dot(Robdd, VarToString, Filename, !IO) :-
+    io.open_output(Filename, Result, !IO),
     (
-        { Result = ok },
-        robdd_to_dot(Robdd, WV),
-        io.told
+        Result = ok(Stream),
+        robdd_to_dot_stream(Stream, Robdd, VarToString, !IO),
+        io.close_output(Stream, !IO)
     ;
-        { Result = error(Err) },
-        io.stderr_stream(StdErr),
-        io.nl(StdErr),
-        io.write_string(StdErr, io.error_message(Err)),
-        io.nl(StdErr)
+        Result = error(Err),
+        io.stderr_stream(StdErr, !IO),
+        io.format(StdErr, "\n%s\n", [s(io.error_message(Err))], !IO)
     ).
 
-robdd_to_dot(Robdd, WV) -->
-    io.write_string(
+robdd_to_dot(Robdd, VarToString, !IO) :-
+    io.output_stream(Stream, !IO),
+    robdd_to_dot_stream(Stream, Robdd, VarToString, !IO).
+
+robdd_to_dot_stream(Stream, Robdd, VarToString, !IO) :-
+    Header =
 "digraph G{
     center=true;
     size=""7,11"";
     ordering=out;
     node [shape=record,height=.1];
     concentrate=true;
-"),
-    { multi_map.init(Ranks0) },
-    robdd_to_dot_2(Robdd, WV, set_bbbtree.init, _, Ranks0, Ranks),
-    map.foldl((pred(_::in, Nodes::in, di, uo) is det -->
-        io.write_string("{rank = same; "),
-        list.foldl((pred(Node::in, di, uo) is det -->
-            io.format("%s; ", [s(node_name(Node))])), Nodes),
-        io.write_string("}\n")
-        ), Ranks),
-    io.write_string("}\n").
+",
+    io.write_string(Stream, Header, !IO),
+    multi_map.init(Ranks0),
+    robdd_to_dot_2(Stream, Robdd, VarToString,
+        set_bbbtree.init, _, Ranks0, Ranks, !IO),
+    map.foldl(
+        ( pred(_::in, Nodes::in, di, uo) is det -->
+            io.write_string(Stream, "{rank = same; "),
+            list.foldl(
+                ( pred(Node::in, di, uo) is det -->
+                    io.format(Stream, "%s; ", [s(node_name(Node))])
+                ), Nodes),
+            io.write_string(Stream, "}\n")
+        ), Ranks, !IO),
+    io.write_string(Stream, "}\n", !IO).
 
     % XXX should see if sparse_bitset is more efficient than set_bbbtree.
-:- pred robdd_to_dot_2(robdd(T)::in, write_var(T)::in(write_var),
+:- pred robdd_to_dot_2(io.text_output_stream::in, robdd(T)::in,
+    var_to_string(T)::in,
     set_bbbtree(robdd(T))::in, set_bbbtree(robdd(T))::out,
     multi_map(var(T), robdd(T))::in,
     multi_map(var(T), robdd(T))::out,
     io::di, io::uo) is det.
 
-robdd_to_dot_2(Robdd, WV, Seen0, Seen, Ranks0, Ranks) -->
-    ( { is_terminal(Robdd) } ->
-        { Seen = Seen0 },
-        { Ranks = Ranks0 }
-    ; { Robdd `member` Seen0 } ->
-        { Seen = Seen0 },
-        { Ranks = Ranks0 }
-    ;
-        robdd_to_dot_2(Robdd ^ tr, WV, Seen0, Seen1, Ranks0, Ranks1),
-        robdd_to_dot_2(Robdd ^ fa, WV, Seen1, Seen2, Ranks1, Ranks2),
-        write_node(Robdd, WV),
-        write_edge(Robdd, Robdd ^ tr, yes),
-        write_edge(Robdd, Robdd ^ fa, no),
-        { Seen = Seen2 `insert` Robdd },
-        { multi_map.set( Robdd ^ value, Robdd, Ranks2, Ranks) }
+robdd_to_dot_2(Stream, Robdd, VarToString, !Seen, !Ranks, !IO) :-
+    ( if is_terminal(Robdd) then
+        true
+    else if Robdd `member` !.Seen then
+        true
+    else
+        robdd_to_dot_2(Stream, Robdd ^ tr, VarToString, !Seen, !Ranks, !IO),
+        robdd_to_dot_2(Stream, Robdd ^ fa, VarToString, !Seen, !Ranks, !IO),
+        write_node(Stream, Robdd, VarToString, !IO),
+        write_edge(Stream, Robdd, Robdd ^ tr, yes, !IO),
+        write_edge(Stream, Robdd, Robdd ^ fa, no, !IO),
+        !:Seen = !.Seen `insert` Robdd,
+        multi_map.set(Robdd ^ value, Robdd, !Ranks)
     ).
 
-:- pred write_node(robdd(T)::in, write_var(T)::in(write_var),
-    io::di, io::uo) is det.
+:- pred write_node(io.text_output_stream::in, robdd(T)::in,
+    var_to_string(T)::in, io::di, io::uo) is det.
 
-write_node(R, WV) -->
-    io.format("%s [label=""<f0> %s|<f1> ",
-        [s(node_name(R)), s(terminal_name(R ^ tr))]),
-    WV(R ^ value),
-    io.format("|<f2> %s", [s(terminal_name(R ^ fa))]),
-    io.write_string("""];\n").
+write_node(Stream, R, VarToString, !IO) :-
+    io.format(Stream, "%s [label=""<f0> %s|<f1> ",
+        [s(node_name(R)), s(terminal_name(R ^ tr))], !IO),
+    io.write_string(Stream, VarToString(R ^ value), !IO),
+    io.format(Stream, "|<f2> %s", [s(terminal_name(R ^ fa))], !IO),
+    io.write_string(Stream, """];\n", !IO).
 
 :- func node_name(robdd(T)) = string.
 
 node_name(R) =
-    ( R = one ->
+    ( if R = one then
         "true"
-    ; R = zero ->
+    else if R = zero then
         "false"
-    ;
+    else
         string.format("node%d", [i(node_num(R))])
     ).
 
@@ -1520,24 +1529,24 @@ node_name(R) =
 :- func terminal_name(robdd(T)) = string.
 
 terminal_name(R) =
-    ( R = zero ->
+    ( if R = zero then
         "0"
-    ; R = one ->
+    else if R = one then
         "1"
-    ;
+    else
         ""
     ).
 
-:- pred write_edge(robdd(T)::in, robdd(T)::in, bool::in,
-    io::di, io::uo) is det.
+:- pred write_edge(io.text_output_stream::in, robdd(T)::in, robdd(T)::in,
+    bool::in, io::di, io::uo) is det.
 
-write_edge(R0, R1, Arc) -->
-    ( { is_terminal(R1) } ->
-        []
-    ;
-        io.format("""%s"":%s -> ""%s"":f1 [label=""%s""];\n",
-            [s(node_name(R0)), s(Arc = yes -> "f0" ; "f2"),
-            s(node_name(R1)), s(Arc = yes -> "t" ; "f")])
+write_edge(Stream, R0, R1, Arc, !IO) :-
+    ( if is_terminal(R1) then
+        true
+    else
+        io.format(Stream, """%s"":%s -> ""%s"":f1 [label=""%s""];\n",
+            [s(node_name(R0)), s(if Arc = yes then "f0" else "f2"),
+            s(node_name(R1)), s(if Arc = yes then "t" else "f")], !IO)
     ).
 
 labelling(Vars, R, TrueVars, FalseVars) :-
@@ -1558,10 +1567,10 @@ labelling_2([V | Vs], R0, TrueVars0, TrueVars, FalseVars0, FalseVars) :-
     labelling_2(Vs, R, TrueVars0 `insert` V, TrueVars, FalseVars0, FalseVars).
 
 minimal_model(Vars, R, TrueVars, FalseVars) :-
-    ( empty(Vars) ->
+    ( if is_empty(Vars) then
         TrueVars = empty_vars_set,
         FalseVars = empty_vars_set
-    ;
+    else
         minimal_model_2(to_sorted_list(Vars), R, empty_vars_set,
             TrueVars0, empty_vars_set, FalseVars0),
         (
@@ -1579,23 +1588,23 @@ minimal_model(Vars, R, TrueVars, FalseVars) :-
 minimal_model_2([], _, TrueVars, TrueVars, FalseVars, FalseVars).
 minimal_model_2([V | Vs], R0, TrueVars0, TrueVars, FalseVars0, FalseVars) :-
     R1 = var_restrict_false(V, R0),
-    ( R1 \= zero ->
-        minimal_model_2(Vs, R1, TrueVars0, TrueVars,
-            FalseVars0 `insert` V, FalseVars)
-    ;
+    ( if R1 = zero then
         R2 = var_restrict_true(V, R0),
         R2 \= zero,
         minimal_model_2(Vs, R2, TrueVars0 `insert` V, TrueVars,
             FalseVars0, FalseVars)
+    else
+        minimal_model_2(Vs, R1, TrueVars0, TrueVars,
+            FalseVars0 `insert` V, FalseVars)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-:- pragma promise_pure(clear_caches/2).
-clear_caches -->
-    { impure clear_caches }.
+:- pragma promise_pure(pred(clear_caches/2)).
+clear_caches(!IO) :-
+    impure clear_caches.
 
-:- pragma no_inline(clear_caches/0).
+:- pragma no_inline(pred(clear_caches/0)).
 :- pragma foreign_proc("C",
     clear_caches,
     [will_not_call_mercury],
@@ -1603,5 +1612,6 @@ clear_caches -->
     MR_ROBDD_init_caches();
 ").
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module robdd.
+%---------------------------------------------------------------------------%
