@@ -1,7 +1,6 @@
-%-----------------------------------------------------------------------------%
-% version_hash_table_test.m
-% vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
 
 :- module version_hash_table_test.
 :- interface.
@@ -10,21 +9,22 @@
 
 :- pred main(io::di, io::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 :- import_module list.
 :- import_module pair.
+:- import_module string.
 :- import_module version_hash_table.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 main(!IO) :-
     % Test `fold' which had an off-by-one bug.
     some [!HT] (
-        !:HT = version_hash_table.init_default(generic_hash),
+        !:HT = version_hash_table.init_default(string.hash),
         version_hash_table.set("one", 1, !HT),
         version_hash_table.set("two", 2, !HT),
         version_hash_table.set("three", 3, !HT),
@@ -38,5 +38,4 @@ main(!IO) :-
 
 concat(K, V, Acc) = [K - V | Acc].
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
