@@ -1,3 +1,7 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+
 :- module type_spec.
 
 :- interface.
@@ -5,7 +9,7 @@
 :- import_module list.
 
 :- pred type_spec1(list(T)::in) is semidet.
-:- external(type_spec1/1).
+:- pragma external_pred(type_spec1/1).
 
 :- pragma type_spec(type_spec1/1, U = int).
 :- pragma type_spec(type_spec1(out), T = int).
@@ -13,13 +17,13 @@
 :- pragma type_spec(type_spec1/2, T = int).
 
 :- typeclass fooable(T) where [
-                pred foo(T),
-                mode foo(in) is semidet
-	].
+    pred foo(T),
+    mode foo(in) is semidet
+].
 
 :- type the_type(T, U).
 :- some [U] pred type_spec2(the_type(T, U)::in) is semidet => fooable(U).
-:- external(type_spec2/1).
+:- pragma external_pred(type_spec2/1).
 
 :- pragma type_spec(type_spec2/1, U = int).
 
@@ -29,5 +33,5 @@
 
 :- implementation.
 
-:- type the_type(T, U) ---> type_type(T, U).
-
+:- type the_type(T, U)
+    --->    type_type(T, U).
