@@ -1,3 +1,7 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
 % This is a regression test. The compiler used to abort in this code
 % when both frame optimization and value numbering were turned on but
 % full jump optimization wasn't.
@@ -11,33 +15,29 @@
 
 :- module livevals_seq.
 :- interface.
-:- import_module list, tree234.
 
-:- type map(K,V)	==	tree234(K,V).
+:- import_module list.
+:- import_module tree234.
 
-:- pred det_insert_fcl(map(K,V), list(K),
-						list(V), map(K,V)).
+:- type map(K, V) == tree234(K, V).
+
+:- pred det_insert_fcl(map(K, V), list(K), list(V), map(K, V)).
 :- mode det_insert_fcl(in, in, in, out) is det.
 
 :- implementation.
 :- import_module require.
 
 det_insert_fcl(Map0, Ks, Vs, Map) :-
-	(
-		Ks = [Key | Keys], Vs = [Value | Values]
-	->
-		det_insert(Map0, Key, Value, Map1),
-		det_insert_fcl(Map1, Keys, Values, Map)
-	;
-		Ks = [], Vs = []
-	->
-		Map = Map0
-	;
-		error("lists do not correspond")
-	).
+    ( if Ks = [Key | Keys], Vs = [Value | Values] then
+        det_insert(Map0, Key, Value, Map1),
+        det_insert_fcl(Map1, Keys, Values, Map)
+    else if Ks = [], Vs = [] then
+        Map = Map0
+    else
+        error("lists do not correspond")
+    ).
 
-:- pred det_insert(map(K,V), K, V, map(K,V)).
-:- mode det_insert(in, in, in, out) is det.
+:- pred det_insert(map(K, V)::in, K::in, V::in, map(K, V)::out) is det.
 :- pragma no_inline(det_insert/4).
 
 det_insert(M, _, _, M).
